/*
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 *  Copyright (c) 2011 digitalSTROM.org, Zurich, Switzerland
 *  Author: Andreas Brauchli <andreas.brauchli@aizo.com>
 */

Ext.namespace('DSS.types');

/**
 * @class DSS.Scenes An enumeration of scene constants
 */
Ext.define('DSS.Scenes', {
    statics: {
        // See http://redmine.digitalstrom.org/projects/dss/wiki/Scene_table
        // for a complete list

        // User scenes
        OFF: 0,
        USER1: 5,
        USER2: 17,
        USER3: 18,
        USER4: 19,

        // Zone scenes
        ZONE_SCENES_START_AT: 11,
        DEC: 11,
        INC: 12,
        MIN: 13,
        MAX: 14,
        STOP: 15,
        AUTO_OFF: 40,
        IMPULSE: 41,
        LOCAL_OFF: 50,
        LOCAL_ON: 51,
        SUN_PROTECTION: 56,
        ZONE_SCENES_END_AT: 64,  // use <

        // apartment scenes
        AUTO_STANDBY: 64,
        PANIC: 65,
        ENERGY_OVERLOAD: 66,
        STANDBY: 67,
        DEEP_OFF: 68,
        SLEEPING: 69,
        WAKE_UP: 70,
        PRESENT: 71,
        ABSENT: 72,
        BELL: 73,
        DOORBELL: 73,
        ALARM: 74,

        ZONE_ACTIVE: 75,
        FIRE: 76,
        ALARM_1: 74,
        ALARM_2: 83,
        ALARM_3: 84,
        ALARM_4: 85,
        WIND: 86,
        NO_WIND: 87,
        RAIN: 88,
        NO_RAIN: 89,
        HAIL: 90,
        NO_HAIL: 91,
        POLLUTION: 92,
        BURGLARY: 93,

        /**
         * Gets the corresponding SceneModel to given scene Note that the custom per zone scene names are not used
         *
         * @param scn
         *            {DSS.Scenes} The scene to convert
         * @return {DSS.apartment.SceneModel} A new model of given scene
         */
        toModel: function(scn) {
            // TODO: get from apartment scene store if loaded
            return Ext.create('DSS.apartment.SceneModel', {
                id: scn,
                text: DSS.Scenes.toString(scn)
            // TODO: icon: DSS.Scenes.getIcon(scn)
            });
        },
        /**
         * Gets the human readable and translated description of a scene If a group is specified the group specific name will be returned
         *
         * @param scn
         *            {DSS.Scenes} The scene to get the name
         * @param grp
         *            {DSS.Groups} (optional) Choose the scene name specific to this group
         * @return {String} The associated description
         */
        toString: function(scn, grp) {
            switch (scn) {
                case DSS.Scenes.OFF:
                    switch (grp) {
                        case DSS.Groups.LIGHT:
                        case DSS.Groups.VIDEO:
                            return _("Off");
                        case DSS.Groups.CLIMATE:
                            return _("Close");
                        default:
                            return _("Scene 0");
                    }
                case DSS.Scenes.USER1:
                    switch (grp) {
                        case DSS.Groups.CLIMATE:
                            return _("Close");
                        default:
                            return _("Scene 1");
                    }
                case DSS.Scenes.USER2:
                    return _("Scene 2");
                case DSS.Scenes.USER3:
                    return _("Scene 3");
                case DSS.Scenes.USER4:
                    return _("Scene 4");
                case DSS.Scenes.DEC:
                    return _("Dim down");
                case DSS.Scenes.INC:
                    return _("Dim up");
                case DSS.Scenes.MIN:
                    return _("Off");
                case DSS.Scenes.MAX:
                    return _("On");
                case DSS.Scenes.STOP:
                    return _("Stop");
                case DSS.Scenes.AUTO_OFF:
                    return _("Auto off");
                case DSS.Scenes.IMPULSE:
                    return _("Impulse");
                case DSS.Scenes.STOP:
                    return _("Stop");
                case DSS.Scenes.LOCAL_ON:
                    return _("On with local priority");
                case DSS.Scenes.LOCAL_OFF:
                    return _("Off with local priority");
                case DSS.Scenes.SUN_PROTECTION:
                    return _("Sun protection");
                case DSS.Scenes.AUTO_STANDBY:
                    return _("Auto standby");
                case DSS.Scenes.PANIC:
                    return _("Panic");
                case DSS.Scenes.ENERGY_OVERLOAD:
                    return _("Energy overload");
                case DSS.Scenes.STANDBY:
                    return _("Standby");
                case DSS.Scenes.DEEP_OFF:
                    return _("Deep off");
                case DSS.Scenes.SLEEPING:
                    return _("Sleeping");
                case DSS.Scenes.WAKE_UP:
                    return _("Wake up");
                case DSS.Scenes.PRESENT:
                    return _("Present");
                case DSS.Scenes.ABSENT:
                    return _("Absent");
                case DSS.Scenes.DOORBELL:
                    return _("Doorbell");
                case DSS.Scenes.BELL:
                    return _("Bell");
                case DSS.Scenes.ZONE_ACTIVE:
                    return _("Zone active");
                case DSS.Scenes.FIRE:
                    return _("Fire");
                case DSS.Scenes.ALARM_1:
                    return _('Alarm') + ' 1';
                case DSS.Scenes.ALARM_2:
                    return _('Alarm') + ' 2';
                case DSS.Scenes.ALARM_3:
                    return _('Alarm') + ' 3';
                case DSS.Scenes.ALARM_4:
                    return _('Alarm') + ' 4';
                case DSS.Scenes.ALARM:
                    return _("Alarm");
                case DSS.Scenes.WIND:
                    return _("Wind");
                case DSS.Scenes.NO_WIND:
                    return _("no Wind");
                case DSS.Scenes.RAIN:
                    return _("Rain");
                case DSS.Scenes.NO_RAIN:
                    return _("No rain");
                case DSS.Scenes.HAIL:
                    return _("Hail");
                case DSS.Scenes.NO_HAIL:
                    return _("hail inactive");
                case DSS.Scenes.POLLUTION:
                    return _("Pollution");
                case DSS.Scenes.BURGLARY:
                    return _("Burglary");
                default:
                    return _("error");
            }
        }
    }
});

Ext.define('DSS.Zones', {
    statics: {
        ALL_DEVICES: 0,
        NEW_DEVICES: 65534,
        toString: function (zoneId) {
            switch (zoneId) {
                case DSS.Zones.ALL_DEVICES:
                    return _('All devices');
                case DSS.Zones.NEW_DEVICES:
                    return _('New devices');
                default:
                    return _('Zone') + ' #' + zoneId;
            }
        }
    }
});

Ext.define('DSS.BinaryInputTypes', {
    statics: {
        GENERIC: 0,
        PRESENCE: 1,
        BRIGHTNESS: 2,
        PRESENCE_IN_DARKNESS: 3,
        TWILIGHT: 4,
        MOTION: 5,
        MOTION_IN_DARKNESS: 6,
        SMOKE: 7,
        WIND_ABOVE_LIMIT: 8,
        RAIN: 9,
        SUN_RADIATION: 10,
        TEMPERATURE_BELOW_LIMIT: 11,
        BATTERY_LOW: 12,
        WINDOW_OPEN: 13,
        DOOR_OPEN: 14,
        WINDOW_TILTED: 15,
        GARAGE_DOOR_OPEN: 16,
        SUN_PROTECTION: 17,
        FROST: 18,
        HEATING_ENABLED: 19,
        CHANGE_OVER_SIGNAL: 20,
        INITIALIZATION: 21,
        MALFUNCTION: 22,
        SERVICE: 23,
        toString: function (binaryInputTypeId) {
            switch (binaryInputTypeId) {
                case DSS.BinaryInputTypes.GENERIC:
                    return _('Generic');
                case DSS.BinaryInputTypes.PRESENCE:
                    return _('Presence');
                case DSS.BinaryInputTypes.BRIGHTNESS:
                    return _('Brightness');
                case DSS.BinaryInputTypes.PRESENCE_IN_DARKNESS:
                    return _('Presence in darkness');
                case DSS.BinaryInputTypes.TWILIGHT:
                    return _('Twilight');
                case DSS.BinaryInputTypes.MOTION:
                    return _('Motion');
                case DSS.BinaryInputTypes.MOTION_IN_DARKNESS:
                    return _('Motion in darkness');
                case DSS.BinaryInputTypes.SMOKE:
                    return _('Smoke');
                case DSS.BinaryInputTypes.WIND_ABOVE_LIMIT:
                    return _('Wind strength above limit');
                case DSS.BinaryInputTypes.RAIN:
                    return _('Rain');
                case DSS.BinaryInputTypes.SUN_RADIATION:
                    return _('Sun radiation');
                case DSS.BinaryInputTypes.TEMPERATURE_BELOW_LIMIT:
                    return _('Temperature below limit');
                case DSS.BinaryInputTypes.BATTERY_LOW:
                    return _('Battery low');
                case DSS.BinaryInputTypes.WINDOW_OPEN:
                    return _('Window open');
                case DSS.BinaryInputTypes.DOOR_OPEN:
                    return _('Door open');
                case DSS.BinaryInputTypes.WINDOW_TILTED:
                    return _('Window tilted');
                case DSS.BinaryInputTypes.GARAGE_DOOR_OPEN:
                    return _('Garage door open');
                case DSS.BinaryInputTypes.SUN_PROTECTION:
                    return _('Sun protection');
                case DSS.BinaryInputTypes.FROST:
                    return _('Frost');
                case DSS.BinaryInputTypes.HEATING_ENABLED:
                    return _('Heating enabled');
                case DSS.BinaryInputTypes.CHANGE_OVER_SIGNAL:
                    return _('Change-over signal');
                case DSS.BinaryInputTypes.INITIALIZATION:
                    return _('Initialization');
                case DSS.BinaryInputTypes.MALFUNCTION:
                    return _('Malfunction');
                case DSS.BinaryInputTypes.SERVICE:
                    return _('Service');
                default:
                    return _('Binary input type') + ' #' + binaryInputTypeId;
            }
        }
    }
});

Ext.define('DSS.SensorTypes', {
    statics: {
        TEMPERATURE_INDOORS: 9,
        TEMPERATURE_OUTDOORS: 10,
        BRIGHTNESS_INDOORS: 11,
        BRIGHTNESS_OUTDOORS: 12,
        RELATIVE_HUMIDITY_INDOORS: 13,
        RELATIVE_HUMIDITY_OUTDOORS: 14,
        AIR_PRESSURE: 15,
        WIND_GUST: 16,
        WIND_SPEED: 18,
        WIND_DIRECTION: 19,
        PRECIPITATION: 20,
        CARBON_DIOXIDE: 21,
        SOUND_PRESSURE_LEVEL: 25,
        ROOM_TEMPERATURE_SETPOINT: 50,
        ROOM_TEMPERATURE_CONTROL_VARIABLE: 51,
        GENERATED_ENERGY: 70,
        WATER_QUANTITY: 71,
        WATER_FLOW_RATE: 72,
        LENGTH: 73,
        MASS: 74,
        TIME: 75,
        SUN_AZIMUTH: 76,
        SUN_ELEVATION: 77,
        toString: function (typeId) {
            switch (typeId) {
                case DSS.SensorTypes.TEMPERATURE_INDOORS:
        			return _('Temperature indoors');
        		case DSS.SensorTypes.TEMPERATURE_OUTDOORS:
        			return _('Temperature outdoors');
        		case DSS.SensorTypes.BRIGHTNESS_INDOORS:
        			return _('Brightness indoors');
        		case DSS.SensorTypes.BRIGHTNESS_OUTDOORS:
        			return _('Brightness outdoors');
        		case DSS.SensorTypes.RELATIVE_HUMIDITY_INDOORS:
        			return _('Relative humidity indoors');
        		case DSS.SensorTypes.RELATIVE_HUMIDITY_OUTDOORS:
        			return _('Relative humidity outdoors');
        		case DSS.SensorTypes.AIR_PRESSURE:
        			return _('Air pressure');
        		case DSS.SensorTypes.WIND_GUST:
        			return _('Wind gust');
        		case DSS.SensorTypes.WIND_SPEED:
        			return _('Wind speed');
        		case DSS.SensorTypes.WIND_DIRECTION:
        			return _('Wind direction');
        		case DSS.SensorTypes.PRECIPITATION:
        			return _('Precipitation');
        		case DSS.SensorTypes.CARBON_DIOXIDE:
        			return _('Carbon Dioxide');
        		case DSS.SensorTypes.SOUND_PRESSURE_LEVEL:
        			return _('Sound pressure level');
        		case DSS.SensorTypes.ROOM_TEMPERATURE_SETPOINT:
        			return _('Room temperature set point');
        		case DSS.SensorTypes.ROOM_TEMPERATURE_CONTROL_VARIABLE:
        			return _('Room temperature control variable');
                case DSS.SensorTypes.GENERATED_ENERGY:
                    return _('Generated Energy');
                case DSS.SensorTypes.WATER_QUANTITY:
                    return _('Water Quantity');
                case DSS.SensorTypes.WATER_FLOW_RATE:
                    return _('Water Flow Rate');
                case DSS.SensorTypes.LENGTH:
                    return _('Length');
                case DSS.SensorTypes.MASS:
                    return _('Mass');
                case DSS.SensorTypes.TIME:
                    return _('Time');
                case DSS.SensorTypes.SUN_AZIMUTH:
                    return _('Sun azimuth');
                case DSS.SensorTypes.SUN_ELEVATION:
                    return _('Sun elevation');
        		default:
        			return _('Sensor type') + " " + typeId;
            }
        },
        getUnit: function (typeId) {
            // TODO: add missing ones, e.g. mass
            switch (typeId) {
                case DSS.SensorTypes.TEMPERATURE_INDOORS:
                case DSS.SensorTypes.TEMPERATURE_OUTDOORS:
                case DSS.SensorTypes.ROOM_TEMPERATURE_SETPOINT:
                    return '°C';
                case DSS.SensorTypes.WIND_DIRECTION:
                case DSS.SensorTypes.SUN_AZIMUTH:
                case DSS.SensorTypes.SUN_ELEVATION:
                    return '°';
                case DSS.SensorTypes.BRIGHTNESS_INDOORS:
                case DSS.SensorTypes.BRIGHTNESS_OUTDOORS:
                    return 'Lx';
                case DSS.SensorTypes.ROOM_TEMPERATURE_CONTROL_VARIABLE:
                case DSS.SensorTypes.RELATIVE_HUMIDITY_INDOORS:
                case DSS.SensorTypes.RELATIVE_HUMIDITY_OUTDOORS:
                    return '%';
                case DSS.SensorTypes.AIR_PRESSURE:
                    return 'hPa';
                case DSS.SensorTypes.WIND_GUST:
                case DSS.SensorTypes.WIND_SPEED:
                    return 'm/s';
                case DSS.SensorTypes.PRECIPITATION:
                    return 'mm/m²';
                case DSS.SensorTypes.CARBON_DIOXIDE:
                    return 'ppm';
                case DSS.SensorTypes.SOUND_PRESSURE_LEVEL:
                    return 'dB';
                case DSS.SensorTypes.GENERATED_ENERGY:
                    return 'kWh';
                case DSS.SensorTypes.WATER_QUANTITY:
                    return 'l';
                case DSS.SensorTypes.WATER_FLOW_RATE:
                    return 'l/s';
                case DSS.SensorTypes.LENGTH:
                    return 'm';
                case DSS.SensorTypes.MASS:
                    return 'g';
                case DSS.SensorTypes.TIME:
                    return 's';
            }
            return '';
        }
    }
});

/**
 * @class DSS.Groups An enumeration of group constants
 */
Ext.define('DSS.Groups', {
    statics : {
        ALL: 0,
        ANY: 0,
        LIGHT: 1,
        SHADOW: 2,
        CLIMATE: 3,
        AUDIO: 4,
        VIDEO: 5,
        SECURITY: 6,
        ACCESS: 7,
        JOKER: 8,
        COOLING: 9,
        WHITE: 9,
        VENTILATION: 10,
        DISPLAYS: 10,
        WINDOW: 11,
        RECIRCULATION: 12,
        AWNINGS: 13,
        TEMPERATURE: 48,
        APARTMENT_VENTILATION: 64,

        /**
         * Gets the human readable and translated description of a group
         *
         * @param grp
         *            {DSS.Groups} The group
         * @return {String} The associated description
         */
        toString: function(grp) {
            switch (grp) {
                case DSS.Groups.ALL:
                    return _("All groups");
                case DSS.Groups.ANY:
                    return _("Any group");
                case DSS.Groups.LIGHT:
                    return _("Light");
                case DSS.Groups.SHADOW:
                    return _("Shadow");
                case DSS.Groups.CLIMATE:
                    return _("Climate");
                case DSS.Groups.AUDIO:
                    return _("Audio");
                case DSS.Groups.VIDEO:
                    return _("Video");
                case DSS.Groups.SECURITY:
                    return _("Security");
                case DSS.Groups.ACCESS:
                    return _("Access");
                case DSS.Groups.JOKER:
                    return _("Joker");
                case DSS.Groups.COOLING:
                    return _("Cooling");
                case DSS.Groups.WHITE:
                    return _("White");
                case DSS.Groups.VENTILATION:
                    return _("Ventilation");
                case DSS.Groups.DISPLAYS:
                    return _("Displays");
                case DSS.Groups.WINDOW:
                    return _("Window");
                case DSS.Groups.RECIRCULATION:
                    return _("Recirculation");
                case DSS.Groups.AWNINGS:
                    return _("Awnings");
                case DSS.Groups.TEMPERATURE:
                    return _("Temperature");
                case DSS.Groups.APARTMENT_VENTILATION:
                    return _("Apartment ventilation");
                default:
                    return _("error");
            }
        },
        /**
         * Gets the color associated with a group
         *
         * @param grp
         *            {DSS.Group} The group for which to get the color
         * @return {String} The associated color
         */
        toColorString: function(grp) {
            switch (grp) {
                case DSS.Groups.ALL:
                case DSS.Groups.ANY:
                    return 'rainbow';
                case DSS.Groups.LIGHT:
                    return 'yellow';
                case DSS.Groups.SHADOW:
                case DSS.Groups.AWNINGS:
                    return 'grey';
                case DSS.Groups.CLIMATE:
                case DSS.Groups.TEMPERATURE:
                case DSS.Groups.VENTILATION:
                case DSS.Groups.RECIRCULATION:
                case DSS.Groups.APARTMENT_VENTILATION:
                    return 'blue';
                case DSS.Groups.AUDIO:
                    return 'cyan';
                case DSS.Groups.VIDEO:
                    return 'magenta';
                case DSS.Groups.SECURITY:
                    return 'red';
                case DSS.Groups.ACCESS:
                    return 'green';
                case DSS.Groups.JOKER:
                    return 'black';
                case DSS.Groups.WHITE:
                    return 'white';
                    // case DSS.Groups.DISPLAYS:
                default:
                    return 'black'; // fallback color
            }
        },
        /**
         * Gets the corresponding GroupModel to given group
         *
         * @param scn
         *            {DSS.Groups} The group to convert
         * @param zone
         *            {DSS.apartment.ZoneModel} (optional) The zone to extract the group from
         * @return {DSS.apartment.GroupModel} A new model of given group
         */
        toModel: function(groupId, zone) {
            var model;
            if (zone) {
                model = zone.groups().getById(groupId);
                if (model) {
                    return model;
                }
            }
            return Ext.create('DSS.apartment.GroupModel', {
                id: groupId,
                text: DSS.Groups.toString(groupId)
            // TODO: icon: DSS.Groups.getIcon(groupId)
            });
        },

        isCluster: function(groupId) {
            return (groupId >= 16 && groupId < 40);
        },

        isApartmentGroup: function (groupId) {
            return groupId >= DSS.Groups.APARTMENT_VENTILATION;
        }
    }
});

Ext.define('DSS.ActionTypes', {     // TODO: trigger types, condition types, see dss-grid columns, dss-trigger-selector and dss-condition-selector
    statics: {
        CHANGE_STATE: 'change-state',
        CHANGE_ADDON_STATE: 'change-addon-state',
        DEVICE_MOC_VALUE: 'device-moc-value',
        DEVICE_ACTION: 'device-action',
        HEATING_MODE: 'heating-mode',
        ZONE_SCENE: 'zone-scene',
        AUDIO_ACTIVITY: 'play-audio-activity',
        UNDO_ZONE_SCENE: 'undo-zone-scene',
        DEVICE_SCENE: 'device-scene',
        DEVICE_VALUE: 'device-value',
        ZONE_VALUE: 'zone-value',
        DEVICE_BLINK: 'device-blink',
        ZONE_BLINK: 'zone-blink',
        CUSTOM_EVENT: 'custom-event',
        URL: 'url'
    }
});

Ext.define('DSS.Weekdays', {     // TODO: trigger types, condition types, see dss-grid columns, dss-trigger-selector and dss-condition-selector
    statics: {
        MO: 'monday',
        TU: 'tuesday',
        WE: 'wednesday',
        TH: 'thursday',
        FR: 'friday',
        SA: 'saturday',
        SU: 'sunday',
        toString: function (day) {
            return _(DSS.Weekdays[day]);
        },
        toDayString: function (day) {
            return _('on ' + day);
        },
        weekStartingFromSU: function () {
            return [ 'SU', 'MO', 'TU', 'WE', 'TH', 'FR', 'SA' ];
        },
        weekStartingFromMO: function () {
            return [ 'MO', 'TU', 'WE', 'TH', 'FR', 'SA', 'SU' ];
        },
    }
});

/**
 * @class DSS.types.ActionModel
 * @extends Ext.data.Model An action model for a single dSS action
 */
Ext
        .define(
                'DSS.types.ActionModel',
                {
                    extend : 'Ext.data.Model',
                    // requires: ['Ext.data.SequentialIdGenerator'],
                    // idgen: 'sequential',
                    fields : [ {
                        name : 'id',
                        type : 'integer',
                        persist: false
                    }, {
                        name : 'text',
                        type : 'string',
                        persist: false
                    }, {
                        name : 'icon',
                        type : 'string',
                        persist: false
                    }, {
                        name : 'incomplete',
                        type : 'boolean',
                        defaultValue: true
                    },

                    // Action
                    'url', // set on action == ActionType.URL
                    'hle', // set on action == ActionType.HLE
                    // + scene on ActionType.APT_SCENE

                    // ActivitySelection
                    {
                        name : 'activity_type',
                        type : 'auto'
                    },
                    // + group on ActivityType.GROUP_ACTIVITY
                    // + scene on ActivityType.ZONE_SCENE

                    // Scene
                    // + scene

                    // Device
                    {
                        name : 'output',
                        type : 'integer'
                    } ],
                    hasMany : [ {
                        name : 'zone',
                        model : 'DSS.apartment.ZoneModel'
                    }, {
                        name : 'group',
                        model : 'DSS.apartment.GroupModel'
                    }, {
                        name : 'device',
                        model : 'DSS.apartment.DeviceModel'
                    }, {
                        name : 'scene',
                        model : 'DSS.apartment.SceneModel'
                    } ],
                    statics : {
                        /** @private */
                        _id: 1,
                        genId: function() {
                            return ++DSS.types.ActionModel._id;
                        },
                        /** @private */
                        apartmentStore: null,
                        /**
                         * Gets the apartmentStore. DO NOT MODIFY THIS STORE. Note that it might not (yet) be done loading. To remedy this, it is suggested to call this getter on page load and wait
                         * until the load event is fired
                         *
                         * @return {Object} The apartment store.
                         */
                        getApartmentStore: function() {
                            if (!DSS.types.ActionModel.apartmentStore) {
                                DSS.types.ActionModel.apartmentStore = Ext.create('DSS.apartment.ApartmentStore');
                                DSS.types.ActionModel.apartmentStore.load();
                            }
                            return DSS.types.ActionModel.apartmentStore;
                        },
                        /**
                         * Create an ActionModel instance from "raw" id values (creating and setting zone, group, device and scene from id as well as setting the corresponding text and icon) Note:
                         * Make sure to preload the custom scene names with DSS.apartment.SceneStore.preloadCustomNameCache() if you would like to have these appear instaed of the default scene names
                         *
                         * @param values
                         *            {Object} The values to use for reconstruction
                         * @return {Object} An ActionModel (with the incomplete flag set if it can't be reconstructed)
                         */
                        reconstructFromIds: function(values) {
                            var data = DSS.types.ActionModel.reconstructValues(values);
                            if (data.valid) {
                                return new DSS.types.ActionModel(data);
                            }
                            return new DSS.types.ActionModel();
                        },
                        /**
                         * @private Recreate missing fields like targetType and actionType from set values and set the zone, group and/or scene to a correct ZoneModel/GroupModel/SceneModel from only
                         *          it's id.
                         */
                        reconstructValues: function(raw) {
                            var values = Ext.apply({}, raw), valid = true, zoneStore = DSS.types.ActionModel.getApartmentStore().first().zones(), sceneStore, zone = null, device = null, i = 0;

                            // targetType
                            if (values.device !== undefined && values.device.length !== 0) {
                                values.targetType = DSS.component.TargetType.DEVICE;
                                for (i = 0; i < zoneStore.getCount(); ++i) {
                                    zone = zoneStore.getAt(i);
                                    device = zone.devices().getById(values.device);
                                    // assume i=0 implies 'all zones'
                                    if (device && i !== 0) {
                                        // only abort search if the device is not in the 'all zones'
                                        // zone, otherwise keep looking
                                        break;
                                    }
                                }
                                if (device !== null) {
                                    values.device = device;
                                    // if wanted, we could choose the zone this device is in
                                    // instead of using the "all rooms" zone 0
                                    values.zone = zone;
                                } else {
                                    values.zone = zoneStore.first();
                                }
                                valid = (!!values.device);

                            } else if (values.zone >= 0) {
                                values.targetType = DSS.component.TargetType.ZONE;
                                values.zone = zoneStore.getById(values.zone);
                                valid = (!!values.zone);
                            }
                            if (values.group >= 0) {
                                if (!!values.zone) {
                                    values.group = values.zone.groups().getById(values.group);
                                    if (!values.group) {
                                        valid = false;
                                    }
                                }
                            }

                            // actionType
                            if (values.blink) {
                                values.actionType = DSS.component.ActionType.BLINK;
                            } else if (values.outputValue >= 0) {
                                values.actionType = DSS.component.ActionType.OUTPUT;
                                values.outputValue = Math.round(values.outputValue / 2.55);
                            } else if (values.scene >= 0) {
                                values.actionType = DSS.component.ActionType.ACTIVITY;
                                sceneStore = DSS.apartment.Helper.getDefaultSceneStore();

                                if (values.zone && values.group) {
                                    // create a copy of the store, load the custom scene names
                                    sceneStore = sceneStore.clone();
                                    // make sure DSS.apartment.SceneStore.preloadCustomNameCache()
                                    // has been called and has returned, as this is otherwise asynchronous
                                    sceneStore.loadCustomNames(values.zone, values.group);
                                }
                                values.scene = sceneStore.getById(values.scene);
                                if (!values.scene) {
                                    valid = false;
                                }
                            } else {
                                valid = false;
                            }
                            values.valid = valid;
                            return values;
                        }

                    },
                    /**
                     * Creates a new ActionModel from valid data
                     *
                     * @param data
                     *            {Object} Initialization data
                     * @param id
                     *            {Number} (Optional) This model's id
                     */
                    constructor: function(data, id) {
                        if (!id) {
                            id = DSS.types.ActionModel.genId();
                        }
                        this.callParent(arguments);
                        if (data) {
                            // All these stores seem to be preloaded with a filter
                            this.zone().clearFilter();
                            this.group().clearFilter();
                            this.device().clearFilter();
                            this.scene().clearFilter();

                            if (data.zone) {
                                // delete to avoid confusion, as it will never be updated later on
                                delete this.data.zone;
                                this.zone().loadData([ data.zone.data ]);
                            }
                            if (data.group) {
                                delete this.data.group;
                                this.group().loadData([ data.group.data ]);
                            }
                            if (data.device) {
                                delete this.data.device;
                                this.device().loadData([ data.device.data ]);
                            }
                            if (data.scene) {
                                delete this.data.scene;
                                this.scene().loadData([ data.scene.data ]);
                            }
                            if (!data.incomplete) {
                                this.set('incomplete', false);
                            }
                        }
                        this.setIcon();
                        this.setText();
                        this.commit();
                    },
                    /**
                     * @override Create a deep clone of this ActionModel
                     * @return {Object} A new ActionModel instance bearing the same but deep-cloned data as this instance
                     */
                    copy: function() {
                        var data = Ext.clone(this.data);
                        if (this.zone().getCount() > 0) {
                            data.zone = this.zone().first().copy();
                        }
                        if (this.group().getCount() > 0) {
                            data.group = this.group().first().copy();
                        }
                        if (this.device().getCount() > 0) {
                            data.device = this.device().first().copy();
                        }
                        if (this.scene().getCount() > 0) {
                            data.scene = this.scene().first().copy();
                        }
                        return new DSS.types.ActionModel(data);
                    },
                    /**
                     * Reset this model to its default, unset values
                     */
                    reset: function() {
                        // All these stores seem to be preloaded with a filter
                        this.zone().clearFilter();
                        this.group().clearFilter();
                        this.device().clearFilter();
                        this.scene().clearFilter();

                        // reset everything expect the id
                        this.zone().removeAll();
                        this.group().removeAll();
                        this.device().removeAll();
                        this.scene().removeAll();
                        this.set('targetType', undefined);
                        this.set('actionType', undefined);
                        this.set('outputValue', undefined);
                        this.set('icon', undefined);
                        this.set('text', undefined);
                        this.set('incomplete', true);
                        this.commit();
                    },

                    /**
                     * @private Sets the icon field from the model's values
                     */
                    setIcon: function() {
                        if (this.data.incomplete) {
                            this.set('icon', 'missing.png');
                            return;
                        }

                        var icon;
                        var st;
                        if (this.data.actionType === DSS.component.ActionType.ACTIVITY) {
                            st = this.scene();
                            if (st.getCount() > 0) {
                                icon = st.first().get('icon');
                                if (icon && icon.length > 0) {
                                    this.set('icon', icon);
                                    return;
                                }
                            }
                        }
                        st = this.group().getCount() > 0 ? this.group() : this.device();
                        var val = st.first();
                        if (val) {
                            icon = val.get('icon');
                            if (icon && icon.length > 0) {
                                this.set('icon', icon);
                                return;
                            }
                        }
                        this.set('icon', null);
                        return;
                    },
                    /**
                     * @private Sets the text field from the model's values
                     */
                    setText: function() {
                        if (this.data.incomplete) {
                            this.set('text', _("Incomplete action..."));
                            return;
                        }

                        var dev = this.device().first(), zon = this.zone().first(), grp = this.group().first(), scn = this.scene().first();

                        if (this.data.actionType === DSS.component.ActionType.OUTPUT) {
                            if (dev) {
                                if (zon.get('id') === 0) {
                                    // Don't display 'set Dev X in All rooms to N%'
                                    this.set('text', Ext.String.format('{0}: {1}%', dev.get('text'), this.data.outputValue));
                                    return;
                                }
                                this.set('text', Ext.String.format('{0} in {1}: {2}%', dev.get('text'), zon.get('text'), this.data.outputValue));
                                return;
                            }
                            if (grp) {
                                this.set('text', Ext.String.format('{0} in {1}: {2}%', grp.get('text'), zon.get('text'), this.data.outputValue));
                                return;
                            }
                        }
                        if (this.data.actionType === DSS.component.ActionType.ACTIVITY) {
                            if (dev) {
                                if (zon.get('id') === 0) {
                                    // Don't display 'D in All rooms on N%'
                                    this.set('text', Ext.String.format(_('{0} for {1}'), scn.get('text'), dev.get('text')));
                                    return;
                                }
                                this.set('text', Ext.String.format(_('{0} for {1} in {2}'), scn.get('text'), dev.get('text'), zon.get('text')));
                                return;
                            }
                            if (grp) {
                                this.set('text', Ext.String.format(_('{0} for {1} in {2}'), scn.get('text'), grp.get('text'), zon.get('text')));
                                return;
                            }
                        }
                        if (this.data.actionType === DSS.component.ActionType.BLINK) {
                            if (dev) {
                                if (zon.get('id') === 0) {
                                    // Don't display 'D in All rooms on N%'
                                    this.set('text', Ext.String.format(_('Blink {0}'), dev.get('text')));
                                    return;
                                }
                                this.set('text', Ext.String.format(_('Blink {0} in {1}'), dev.get('text'), zon.get('text')));
                                return;
                            }
                            if (grp) {
                                this.set('text', Ext.String.format(_('Blink {0} in {1}'), grp.get('text'), zon.get('text')));
                                return;
                            }
                        }

                        this.set('text', _("error"));
                    }
                }); // DSS.types.ActionModel

/**
 * @class DSS.types.ActionType Possible action types enumeration
 */
Ext.define('DSS.types.ActionType', {
    statics : {
        APT_SCENE : 'ACT_APT_SCENE',
        DEVICE : 'ACT_DEVICE',
        HLE : 'ACT_HLE',
        URL : 'ACT_URL',
        ZONE_ACTIVITY : 'ACT_ZONE',
        toString: function(at) {
            switch (at) {
                case DSS.types.ActionType.APT_SCENE:
                    return _("Call an apartment activity");
                case DSS.types.ActionType.DEVICE:
                    return _("Set a device");
                case DSS.types.ActionType.HLE:
                    return _("Use a high level event");
                case DSS.types.ActionType.URL:
                    return _("Retrieve a URL");
                case DSS.types.ActionType.ZONE_ACTIVITY:
                    return _("Call activity in a room");
                default:
                    return _("error");
            }
        }
    }
});

/**
 * @class DSS.types.ActionTypeModel A model to carry multiple (different) DSS.types.ActionType
 */
Ext.define('DSS.types.ActionTypeModel', {
    extend : 'Ext.data.Model',
    idProperty : 'action',
    fields : [ 'action', 'icon', 'text' ]
});

/**
 * @class DSS.types.TriggerType Possible action types enumeration
 */
Ext.define('DSS.types.TriggerType', {
    statics : {
        APT_SCENE : 'TRG_APT_SCENE',
        DEVICE : 'TRG_DEVICE',
        HLE : 'TRG_HLE',
        ZONE_ACTIVITY : 'TRG_ZONE',
        toString: function(tt) {
            switch (tt) {
                case DSS.types.TriggerType.APT_SCENE:
                    return _("Call an apartment activity");
                case DSS.types.TriggerType.DEVICE:
                    return _("Set a device");
                case DSS.types.TriggerType.HLE:
                    return _("Use a high level event");
                case DSS.types.TriggerType.ZONE_ACTIVITY:
                    return _("Call activity in a room");
                default:
                    return _("error");
            }
        }
    }
});

/**
 * @class DSS.types.ActivityType Possible activity types enumeration
 */
Ext.define('DSS.types.ActivityType', {
    statics : {
        BLINK : 'ATY_BLINK',
        GROUP_ACTIVITY : 'ATY_GRP_ACTIVITY',
        ZONE_SCENE : 'ATY_ZONE_SCENE',
        toString: function(at) {
            switch (at) {
                case DSS.types.ActivityType.GROUP_ACTIVITY:
                    return _("Call activity in a group");
                case DSS.types.ActivityType.BLINK:
                    return _("Blink");
                case DSS.types.ActivityType.ZONE_SCENE:
                    return _("Call a room activity");
                default:
                    return _("error");
            }
        }
    }
});

/**
 * @class DSS.types.ActionSelectionModel
 * @extends Ext.data.Model Model typically used for ActionSelectionStore
 */
Ext.define('DSS.types.ActionSelectionModel', {
    extend : 'Ext.data.Model',
    fields : [ {
        name : 'id',
        type : 'string',
        persist: false
    }, {
        name : 'action_type',
        type : 'auto'
    }, {
        name : 'zone_scene',
        type : 'integer',
        defaultValue : -1
    }, 'url', // set on action == ActionType.URL
    'hle', // set on action == ActionType.HLE
    {
        name : 'text',
        type : 'string',
        persist: false
    }, {
        name : 'icon',
        type : 'string',
        persist: false
    } ]
});

/**
 * @class DSS.types.ActivitySelectionModel
 * @extends Ext.data.Model Model typically used for ActivitySelectionStore
 */
Ext.define('DSS.types.ActivitySelectionModel', {
    extend : 'Ext.data.Model',
    fields : [ {
        name : 'id',
        type : 'string',
        persist: false
    },
    /** One of DSS.types.ActivityType */
    {
        name : 'activity_type',
        type : 'auto'
    },
    /** The actual apartment scene */
    {
        name : 'apt_scene',
        type : 'integer',
        defaultValue : -1
    }, // set on APT_SCENE
    /** The actual group */
    {
        name : 'group',
        type : 'integer',
        defaultValue : -1
    }, // set on !APT_SCENE

    {
        name : 'text',
        type : 'string',
        persist: false
    }, {
        name : 'icon',
        type : 'string',
        persist: false
    } ],
    constructor: function(data) {
        if (!data.id) {
            // assign an (suposedly unique per store) id
            data.id = data.activity_type;
            if (data.group >= 0) {
                data.id += '_' + data.group;
            }
            if (data.apt_scene >= 0) {
                data.id += '_' + data.apt_scene;
            }
        }
        this.callParent(arguments);
    }
});
