/* globals DSS, dss, devLog, Ext, Gettext, ActiveXObject, rgb2hex, hsv2rgb */

var cachingURL = 'cachekey=' + ('' + Math.random()).substr(2);

DSS.types = {};
DSS.types.dsuid = {};

DSS.types.dsuid.dsuid_from_dsid = function (dSID) {
    // copy the first 6 bytes (12 characters to the new dsuid
    var dSUID = undefined;
    if (dSID) {
        dSUID = dSID.slice(0, 12);
        // insert 4 bytes of 0
        dSUID += '00000000';
        // append the remaining 6 bytes (12 chars) of the dsid
        dSUID += dSID.slice(12, 24);
        // finaly add 1 byte of 0
        dSUID += '00';
    }
    return dSUID;
};
DSS.types.dsuid.dsuid_to_dsid = function (dSUID) {
    var dSID = undefined;
    if (dSUID && dSUID.slice(12, 20) === '00000000') {
        dSID = dSUID.slice(0, 12) + dSUID.slice(20, 32);
    }
    return dSID;
};
DSS.types.dsuid.isDSUID = function (id) {
    if (id.length === 34) {
        return true;
    } else {
        return false;
    }
};
DSS.types.dsuid.isDSID = function (id) {
    if (id.length === 24) {
        return true;
    } else {
        return false;
    }
};
DSS.types.dsuid.shortdSUID = function (dsuid) {
    return dsuid.slice(dsuid.length - 10, dsuid.length - 2);
};

dss = {};

dss.version = 1;

dss.commonConsts = {
    SECONDS_PER_MINUTE: 60,
    MINUTES_PER_HOUR: 60,
    SECONDS_PER_HOUR: 3600
};

dss.unescapeProperty = function (sString) {
    if (typeof sString === 'string') {
        return sString
        .replace(new RegExp('&amp;', 'g'), '&')
        .replace(new RegExp('&gt;', 'g'), '>')
        .replace(new RegExp('&quot;', 'g'), '"')
        .replace(new RegExp('&#039;', 'g'), '\'')
        .replace(new RegExp('&lt;', 'g'), '<');
    } else {
        return sString;
    }
};

dss.escapeProperty = function (sString) {
    if (typeof sString === 'string') {
        return sString
        .replace(new RegExp('&', 'g'), '&amp;')
        .replace(new RegExp('>', 'g'), '&gt;')
        .replace(new RegExp('"', 'g'), '&quot;')
        .replace(new RegExp('\'', 'g'), '&#039;')
        .replace(new RegExp('<', 'g'), '&lt;');
    } else {
        return sString;
    }
};

dss.validValue = {
    ok: 0,
    temporaryWrong: 1,
    completlyWrong: 2
};

dss.whState = {
    WH_Invalid: 0,
    WH_Closed: 1,
    WH_Open: 2,
    WH_Tilted: 3,
    WH_Unknown: 4
};

dss.mocChannelIds = {
    BRIGHTNESS: 1,
    HUE: 2,
    SATURATION: 3,
    COLOR_TEMPERATURE: 4
};
dss.parseMoc = function parseMoc (outputChannels) {
    const channelValues = {};
    let i = 0;
    while (outputChannels['channelid-' + i] != null) {
        channelValues[outputChannels['channelid-' + i]] = outputChannels['channelvalue-' + i];
        i++;
    }
    return channelValues;
};
dss.mocToHexColor = function mocToHexColor (outputChannels) {        // eslint-disable-line no-unused-vars
    const channelValues = dss.parseMoc(outputChannels);
    const RANGE = 255.0;
    if (channelValues[dss.mocChannelIds.BRIGHTNESS] != null &&
        channelValues[dss.mocChannelIds.HUE] != null &&
        channelValues[dss.mocChannelIds.SATURATION] != null) {
        var hsv = {
            hue: channelValues[dss.mocChannelIds.HUE] / RANGE,
            bright: channelValues[dss.mocChannelIds.BRIGHTNESS] / RANGE,
            sat: channelValues[dss.mocChannelIds.SATURATION] / RANGE
        };
        return rgb2hex(hsv2rgb(hsv));
    }
};
dss.mocToBrightnessTemperatureColor = function mocToBrightnessTemperatureColor (outputChannels) {        // eslint-disable-line no-unused-vars
    const channelValues = dss.parseMoc(outputChannels);
    if (channelValues[dss.mocChannelIds.BRIGHTNESS] != null &&
            channelValues[dss.mocChannelIds.COLOR_TEMPERATURE] != null) {
        return {
            bright: Math.round(channelValues[dss.mocChannelIds.BRIGHTNESS] / 2.55),     // eslint-disable-line no-magic-numbers
            temperature: dss.MiredToKelvin(dss.MOCValueToMired(channelValues[dss.mocChannelIds.COLOR_TEMPERATURE]))
        };
    }
};

dss.getCookie = function (cname) {
    var name = cname + '=';
    var ca = document.cookie.split(';');
    for (var i = 0; i < ca.length; i++) {
        var c = ca[i];
        while (c.charAt(0) == ' ') { c = c.substring(1); }
        if (c.indexOf(name) == 0) {
            return c.substring(name.length, c.length);
        }
    }
    return '';
};

dss.ajaxSyncRequest = function (surl, params, defaultAnswer) {
    var xmlHttp = null;
    params.a = Math.random();

    var paramString = '?';
    var csrf = dss.getCookie('csrf-token');
    for (var key in params) {
        paramString += key + '=' + encodeURIComponent(params[key]) + '&';
    }

    if (typeof XMLHttpRequest !== 'undefined') {
        xmlHttp = new XMLHttpRequest();
    }
    if (!xmlHttp) {
        try {
            xmlHttp = new ActiveXObject('Msxml2.XMLHTTP');
        } catch (e) {
            devLog && devLog(e);
            try {
                xmlHttp = new ActiveXObject('Microsoft.XMLHTTP');
            } catch (e) {
                devLog && devLog(e);
                xmlHttp = null;
            }
        }
    }
    if (xmlHttp) {
        xmlHttp.open('GET', surl + paramString, false);
        if (csrf != '') {
            xmlHttp.setRequestHeader('X-Csrf-Token', csrf);
        }
        xmlHttp.send(null);
        if (defaultAnswer != null) {
            if (xmlHttp.status == 404) {
                return defaultAnswer;
            }
        }
        return xmlHttp.responseText.replace(/-nan/g, '0');
    }
};

dss.ajaxAssyncRequestWithoutAnswer = function (surl, params, callback) {
    var xmlHttp = null;
    params.a = Math.random();

    var paramString = '?';
    var csrf = dss.getCookie('csrf-token');
    for (var key in params) {
        paramString += key + '=' + encodeURIComponent(params[key]) + '&';
    }

    if (typeof XMLHttpRequest !== 'undefined') {
        xmlHttp = new XMLHttpRequest();
    }
    if (!xmlHttp) {
        try {
            xmlHttp = new ActiveXObject('Msxml2.XMLHTTP');
        } catch (e) {
            devLog && devLog(e);
            try {
                xmlHttp = new ActiveXObject('Microsoft.XMLHTTP');
            } catch (e) {
                devLog && devLog(e);
                xmlHttp = null;
            }
        }
    }
    if (xmlHttp) {
        if (callback != null) {
            xmlHttp.onreadystatechange = function () {
                if (xmlHttp.readyState == 4) {
                    callback(xmlHttp.responseText.replace(/-nan/g, '0'));
                }
            };
        }
        xmlHttp.open('GET', surl + paramString, true);
        if (csrf != '') {
            xmlHttp.setRequestHeader('X-Csrf-Token', csrf);
        }
        xmlHttp.send(null);
    }
};

dss.ajaxAssyncRequestWithoutAnswerPost = function (surl, params, callback) {
    var xmlHttp = null;
    params.a = Math.random();

    var paramString = '';
    var csrf = dss.getCookie('csrf-token');
    for (var key in params) {
        paramString += key + '=' + encodeURIComponent(params[key]) + '&';
    }

    if (typeof XMLHttpRequest !== 'undefined') {
        xmlHttp = new XMLHttpRequest();
    }
    if (!xmlHttp) {
        try {
            xmlHttp = new ActiveXObject('Msxml2.XMLHTTP');
        } catch (e) {
            devLog && devLog(e);
            try {
                xmlHttp = new ActiveXObject('Microsoft.XMLHTTP');
            } catch (e) {
                devLog && devLog(e);
                xmlHttp = null;
            }
        }
    }
    if (xmlHttp) {
        if (callback != null) {
            xmlHttp.onreadystatechange = function () {
                if (xmlHttp.readyState == 4) {
                    callback(xmlHttp.responseText.replace(/-nan/g, '0'));
                }
            };
        }
        xmlHttp.open('POST', surl, true);
        if (csrf != '') {
            xmlHttp.setRequestHeader('X-Csrf-Token', csrf);
        }
        xmlHttp.send(paramString);
    }
};

dss.raiseEvent = function (name, params) {
    var sURL = '/json/event/raise';
    var paramString = '';
    for (var key in params) {
        paramString += key + '=' + encodeURIComponent(params[key]) + ';';
    }
    dss.ajaxAssyncRequestWithoutAnswerPost(sURL, {
        name: name,
        parameter: paramString.substr(0, paramString.length - 1)
    });
};

dss.getPropertyInteger = function (sProp) {
    var sURL = '/json/property/getInteger';
    var sPfad = escape(sProp);
    var sReturn = dss.ajaxSyncRequest(sURL, {
        path: sPfad
    });
    var obj = eval('(' + sReturn + ')');
    if (obj.ok) {
        return obj.result.value;
    } else {
        return null;
    }
};

dss.staticDataModel = {};
dss.staticDataModel.baseImagePath = '/framework/images/';
dss.staticDataModel.sQueryStringDevices = '/apartment/zones/zone0/devices/*(*)/*(*)/*(*)';
dss.staticDataModel.sQueryStringStructure = '/apartment/zones/*(ZoneID,name)/groups/*(group,name,color,valid,connectedDevices,activeBasicScenes)/scenes/*(scene,name)';
dss.staticDataModel.sQueryStringHLE = '/usr/events/*(name,id)';
dss.staticDataModel.sQueryStringUSR = '/usr/*/*(name,id,value)/*(*)';
dss.staticDataModel.sQueryStringMeters = '/apartment/dSMeters/*(*)/capabilities(*)';
dss.staticDataModel.sQueryStringClimateSettings = '/scripts/system-addon-heating-controller/generalSettings(*)/*(*)';
dss.staticDataModel.sQueryStringZoneSensor = '/apartment/zones/*(ZoneID)/groups/*(group)/sensor/*(*)';
dss.staticDataModel.nameCache = null;
dss.staticDataModel.deviceCache = null;
dss.staticDataModel.HLECache = null;
dss.staticDataModel.StateCache = null;
dss.staticDataModel.MeterCache = null;
dss.staticDataModel.activeLanguage = 'de_DE';
dss.staticDataModel.deviceWithSensorsCache = null;
dss.staticDataModel.deviceAKMCache = null;
dss.staticDataModel.userGroups = null;
dss.staticDataModel.userStates = null;
dss.staticDataModel.audioActivities = null;
dss.staticDataModel.stateArray = null;
dss.staticDataModel.heating = null;
dss.staticDataModel.clusterAvaible = false;
dss.staticDataModel.ClimateControllerSettingsCache = null;
dss.staticDataModel.globalApplicationsAvaible = false;

dss.staticDataModel.getStateArray = function () {
    if (dss.staticDataModel.stateArray === null) {
        dss.staticDataModel.initStateArray();
    }
    return dss.staticDataModel.stateArray;
};

dss.staticDataModel.initStateArray = function () {
    var result;
    try {
        var sString = dss.ajaxSyncRequest('/json/property/query2', {
            query: '/usr/states/*(*)'
        });
        result = Ext.JSON.decode(sString);
    } catch (e) {
        devLog && devLog(e);
    }
    if (result && result.ok) {
        dss.staticDataModel.stateArray = result.result;
    }
    for (var sKey in dss.staticDataModel.stateArray) {
        if (!dss.staticDataModel.clusterAvaible && sKey.split('.')[0] === 'cluster') {
            dss.staticDataModel.clusterAvaible = true;
        }
        if (!dss.staticDataModel.globalApplicationsAvaible && sKey.indexOf('zone.0.group.64') > -1) {
            dss.staticDataModel.globalApplicationsAvaible = dss.staticDataModel.getDevicesForZone(0, 64).length > 0;
        }
    }
};

dss.staticDataModel.getUsergroups = function () {
    return dss.staticDataModel.userGroups;
};

dss.staticDataModel.getUserStates = function () {
    if (dss.staticDataModel.userStates == null) {
        dss.staticDataModel.initUserStateCache();
    }
    return dss.staticDataModel.userStates;
};

dss.staticDataModel.getAudioActivities = function () {
    if (dss.staticDataModel.MeterCache == null) {
        dss.staticDataModel.initMeterCache();
    }
    const sonosVdc = dss.staticDataModel.MeterCache && dss.staticDataModel.MeterCache.filter(function (meter) {
        return meter.ImplementationId === 'Sonos_Container' && meter.isValid && meter.present;
    });
    if (!sonosVdc) {
        return;
    }
    const sonosVdcDsuid = sonosVdc[0] && sonosVdc[0].dSUID;
    if (dss.staticDataModel.playlists == null) {
        dss.staticDataModel.initPlaylistCache(sonosVdcDsuid);
    }
    if (dss.staticDataModel.favorites == null) {
        dss.staticDataModel.initFavoriteCache(sonosVdcDsuid);
    }
    if (dss.staticDataModel.clips == null) {
        dss.staticDataModel.initClipsCache();
    }
    if (dss.staticDataModel.audioActivities == null) {
        dss.staticDataModel.initAudioActivityCache(sonosVdcDsuid);
    }
    return dss.staticDataModel.audioActivities;
};

dss.staticDataModel.getUserStateValueName = function (state) {
    if (dss.staticDataModel.userStates) {
        for (var iIndex = 0; iIndex < dss.staticDataModel.userStates.length; iIndex++) {
            if (dss.staticDataModel.userStates[iIndex].name == state) {
                if (dss.staticDataModel.userStates[iIndex].value == 1) {
                    return dss.staticDataModel.userStates[iIndex].setName;
                }
                if (dss.staticDataModel.userStates[iIndex].value == 2) {
                    return dss.staticDataModel.userStates[iIndex].resetName;
                }
            }
        }
    }
};

dss.staticDataModel.getImagePathForAudioActivity = function (activityType) {
    switch (activityType) {
        // todo: in the future add command, text2speech etc.
        case 'playlist':
        case 'favorite':
        default: {
            return dss.staticDataModel.baseImagePath + 'note.png';
        }
    }
};

dss.staticDataModel.getUserStateValue = function (state) {
    if (dss.staticDataModel.userStates) {
        for (var iIndex = 0; iIndex < dss.staticDataModel.userStates.length; iIndex++) {
            if (dss.staticDataModel.userStates[iIndex].name == state) {
                if (dss.staticDataModel.userStates[iIndex].value == 1) {
                    return true;
                }
                if (dss.staticDataModel.userStates[iIndex].value == 2) {
                    return false;
                }
            }
        }
    }
};

dss.staticDataModel.getHLENameFromId = function (id) {
    if (dss.staticDataModel.HLECache) {
        for (var iIndex = 0; iIndex < dss.staticDataModel.HLECache.length; iIndex++) {
            if (dss.staticDataModel.HLECache[iIndex].id == id) {
                return dss.staticDataModel.HLECache[iIndex].name;
            }
        }
    }
    return id;
};

dss.staticDataModel.refreshDataModel = function () {
    cachingURL = 'cachekey=' + ('' + Math.random()).substr(2);
    dss.staticDataModel.nameCache = null;
    dss.staticDataModel.deviceCache = null;
    dss.staticDataModel.HLECache = null;
    dss.staticDataModel.StateCache = null;
    dss.staticDataModel.MeterCache = null;
    dss.staticDataModel.deviceWithSensorsCache = null;
    dss.staticDataModel.deviceAKMCache = null;
    dss.staticDataModel.userGroups = null;
    dss.staticDataModel.userStates = null;
    dss.staticDataModel.stateArray = null;
    dss.staticDataModel.ClimateControllerSettingsCache = null;
    dss.staticDataModel.initCaches();
    if (Ext != null) {
        if (Ext.data != null) {
            if (Ext.data.StoreManager != null) {
                Ext.data.StoreManager.clear();
            }
        }
    }
};

dss.staticDataModel.initHeatingCache = function () {
    dss.staticDataModel.heating = {};
    dss.staticDataModel.heating[0] = {};
    dss.staticDataModel.heating[0].configured = true;
    dss.staticDataModel.heating[0].active = true;
    dss.staticDataModel.heating[0].receiveOperationMode = true;

    var result;
    try {
        var sString = dss.ajaxSyncRequest('/json/apartment/getTemperatureControlConfig', {});
        result = Ext.JSON.decode(sString);
    } catch (e) {
        devLog && devLog(e);
    }
    if (result && result.ok && result.result && result.result.zones) {
        for (var iIndex = 0; iIndex < result.result.zones.length; iIndex++) {
            var zoneNode = result.result.zones[iIndex];
            dss.staticDataModel.heating[zoneNode.id] = {
                configured: true
            };
            dss.staticDataModel.heating[zoneNode.id].active = false;
            dss.staticDataModel.heating[zoneNode.id].receiveOperationMode = false;
            if (zoneNode.ControlMode == 0) {
                dss.staticDataModel.heating[zoneNode.id].active = false;
            } else {
                dss.staticDataModel.heating[zoneNode.id].active = true;
            }
            if ((zoneNode.ControlMode == 1) || (zoneNode.ControlMode == 3)) {
                dss.staticDataModel.heating[zoneNode.id].receiveOperationMode = true;
            } else {
                dss.staticDataModel.heating[zoneNode.id].receiveOperationMode = false;
            }
        }
    }
};

dss.staticDataModel.initCaches = function () {
    dss.staticDataModel.initNameCache();
    dss.staticDataModel.initSpecialActions();
    dss.staticDataModel.initDeviceCache();
    dss.staticDataModel.initHLECache();
    dss.staticDataModel.initMeterCache();
    dss.staticDataModel.initSunriseTimes();
    dss.staticDataModel.initUserStateCache();
    dss.staticDataModel.initStateArray();
    dss.staticDataModel.getDSSTime();
    dss.staticDataModel.initHeatingCache();
    dss.staticDataModel.initZoneSensorCache();
    dss.staticDataModel.initClimateControllerSettingsCache();
};

dss.staticDataModel.localizeName = function (nameObject) {
    var sName = '';
    if (nameObject[dss.staticDataModel.activeLanguage] != null) {
        sName = nameObject[dss.staticDataModel.activeLanguage];
    } else if (nameObject.en_US != null) {
        sName = nameObject.en_US;
    } else if (nameObject.de_DE != null) {
        sName = nameObject.de_DE;
    }
    return dss.unescapeProperty(sName);
};

dss.staticDataModel.initSpecialActions = function () {
    var resultObj;
    try {
        var sResult = dss.ajaxSyncRequest('/json/database/query', {
            sql: 'select devices.gtin,elements.type,elements.id,parameter.value,parameter.name ' +
                'from (devices inner join elements on devices.id=elements.device_id) inner join parameter on parameter.element_id=elements.id ' +
                'where parameter.name=\'type\' ',
            database: 'system-addon-scene-responder'
        });
        resultObj = Ext.JSON.decode(sResult);
    } catch (e) {
        devLog && devLog(e);
    }
    if (resultObj && resultObj.ok) {
        dss.staticDataModel.specialAction = {};
        if (resultObj.result && resultObj.result.data) {
            for (var iIndex = 0; iIndex < resultObj.result.data.length; iIndex++) {
                var gtin = resultObj.result.data[iIndex].gtin;
                var type = resultObj.result.data[iIndex].type;
                var elementid = resultObj.result.data[iIndex].id;
                var value = resultObj.result.data[iIndex].value;
                if (dss.staticDataModel.specialAction[gtin] == null) {
                    dss.staticDataModel.specialAction[gtin] = {};
                }
                switch (type) {
                    case '0': {
                        if (dss.staticDataModel.specialAction[gtin].triggers == null) {
                            dss.staticDataModel.specialAction[gtin].triggers = {};
                        }
                        switch (value) {
                            case 'zone-scene':
                                if (dss.staticDataModel.specialAction[gtin].triggers.zoneScene == null) {
                                    dss.staticDataModel.specialAction[gtin].triggers.zoneScene = {};
                                }
                                dss.staticDataModel.specialAction[gtin].triggers.zoneScene[elementid] = {};
                                break;
                            case 'device': {
                                if (dss.staticDataModel.specialAction[gtin].triggers.deviceMsg == null) {
                                    dss.staticDataModel.specialAction[gtin].triggers.deviceMsg = {};
                                }
                                dss.staticDataModel.specialAction[gtin].triggers.deviceMsg[elementid] = {};
                                break;
                            }
                            case 'device-binary-input':
                                // not supportet yet -> RK ist hier zu faul vorab zu denken
                                if (dss.staticDataModel.specialAction[gtin].triggers.binaryInputs == null) {
                                    dss.staticDataModel.specialAction[gtin].triggers.binaryInputs = {};
                                }
                                dss.staticDataModel.specialAction[gtin].triggers.binaryInputs[elementid] = {};
                                break;
                            case 'device-sensor':
                                if (dss.staticDataModel.specialAction[gtin].triggers.deviceEvents == null) {
                                    dss.staticDataModel.specialAction[gtin].triggers.deviceEvents = {};
                                }
                                dss.staticDataModel.specialAction[gtin].triggers.deviceEvents[elementid] = {};
                                break;
                            case 'device-action': {
                                if (dss.staticDataModel.specialAction[gtin].triggers.deviceActions == null) {
                                    dss.staticDataModel.specialAction[gtin].triggers.deviceActions = {};
                                }
                                dss.staticDataModel.specialAction[gtin].triggers.deviceActions[elementid] = {};
                                break;
                            }
                        }
                        break;
                    }
                    case '1': {
                        if (dss.staticDataModel.specialAction[gtin].actions == null) {
                            dss.staticDataModel.specialAction[gtin].actions = {};
                        }
                        switch (value) {
                            case 'zone-action':
                                if (dss.staticDataModel.specialAction[gtin].actions.zoneScene == null) {
                                    dss.staticDataModel.specialAction[gtin].actions.zoneScene = {};
                                }
                                dss.staticDataModel.specialAction[gtin].actions.zoneScene[elementid] = {};
                                break;
                            case 'device-action-scene': {
                                if (dss.staticDataModel.specialAction[gtin].actions.deviceActions == null) {
                                    dss.staticDataModel.specialAction[gtin].actions.deviceActions = {};
                                }
                                dss.staticDataModel.specialAction[gtin].actions.deviceActions[elementid] = {};
                                break;
                            }
                            case 'device-action-value': {
                                if (dss.staticDataModel.specialAction[gtin].actions.deviceActions == null) {
                                    dss.staticDataModel.specialAction[gtin].actions.deviceActions = {};
                                }
                                dss.staticDataModel.specialAction[gtin].actions.deviceActions[elementid] = {};
                                break;
                            }
                        }
                        break;
                    }
                    case '2': {
                        if (dss.staticDataModel.specialAction[gtin].conditions == null) {
                            dss.staticDataModel.specialAction[gtin].conditions = {};
                        }
                        switch (value) {
                            case 'sensor-event':
                                // not supportet yet -> RK ist hier zu faul vorab zu denken
                                if (dss.staticDataModel.specialAction[gtin].conditions.akmEvent == null) {
                                    dss.staticDataModel.specialAction[gtin].conditions.akmEvent = {};
                                }
                                dss.staticDataModel.specialAction[gtin].conditions.akmEvent[elementid] = {};
                                break;
                        }
                        break;
                    }
                }
            }
        }
    }

    var resultObj;
    try {
        var sResult = dss.ajaxSyncRequest('/json/database/query', {
            sql: 'select parameter.value,parameter.name,parameter.element_id from parameter where not(parameter.name=\'type\')',
            database: 'system-addon-scene-responder'
        });
        resultObj = Ext.JSON.decode(sResult);
    } catch (e) {
        devLog && devLog(e);
    }
    if (resultObj && resultObj.ok && resultObj.result && resultObj.result.data) {
        for (var gtin in dss.staticDataModel.specialAction) {
            for (var type in dss.staticDataModel.specialAction[gtin]) {
                for (var subtype in dss.staticDataModel.specialAction[gtin][type]) {
                    for (var elementID in dss.staticDataModel.specialAction[gtin][type][subtype]) {
                        for (var iIndex = 0; iIndex < resultObj.result.data.length; iIndex++) {
                            if (elementID == resultObj.result.data[iIndex].element_id) {
                                dss.staticDataModel.specialAction[gtin][type][subtype][elementID][resultObj.result.data[iIndex].name] = resultObj.result.data[iIndex].value;
                            }
                        }
                    }
                }
            }
        }
    }

    var resultObj;
    try {
        sResult = dss.ajaxSyncRequest('/json/database/query', {
            sql: 'select name.name,name.langcode,name.element_id from name',
            database: 'system-addon-scene-responder'
        });
        resultObj = Ext.JSON.decode(sResult);
    } catch (e) {
        devLog && devLog(e);
    }
    if (resultObj && resultObj.ok) {
        for (var gtin in dss.staticDataModel.specialAction) {
            for (var type in dss.staticDataModel.specialAction[gtin]) {
                for (var subtype in dss.staticDataModel.specialAction[gtin][type]) {
                    for (var elementID in dss.staticDataModel.specialAction[gtin][type][subtype]) {
                        if (resultObj.result && resultObj.result.data) {
                            for (var iIndex = 0; iIndex < resultObj.result.data.length; iIndex++) {
                                if (elementID == resultObj.result.data[iIndex].element_id) {
                                    if (dss.staticDataModel.specialAction[gtin][type][subtype][elementID].name == null) {
                                        dss.staticDataModel.specialAction[gtin][type][subtype][elementID].name = {};
                                    }
                                    dss.staticDataModel.specialAction[gtin][type][subtype][elementID]
                                        .name[resultObj.result.data[iIndex].langcode] = resultObj.result.data[iIndex].name;
                                }
                            }
                        }
                    }
                }
            }
        }
    }

    for (var sEANKey in dss.staticDataModel.specialAction) {
        if (dss.staticDataModel.specialAction[sEANKey].actions != null) {
            for (var sActionType in dss.staticDataModel.specialAction[sEANKey].actions) {
                for (var sActionIndex in dss.staticDataModel.specialAction[sEANKey].actions[sActionType]) {
                    dss.staticDataModel.specialAction[sEANKey].actions[sActionType][sActionIndex].name =
                            dss.staticDataModel.localizeName(dss.staticDataModel.specialAction[sEANKey].actions[sActionType][sActionIndex].name);
                }

            }
        }
        if (dss.staticDataModel.specialAction[sEANKey].triggers != null) {
            for (var sActionType in dss.staticDataModel.specialAction[sEANKey].triggers) {
                for (var sActionIndex in dss.staticDataModel.specialAction[sEANKey].triggers[sActionType]) {
                    dss.staticDataModel.specialAction[sEANKey].triggers[sActionType][sActionIndex].name = dss.staticDataModel
                            .localizeName(dss.staticDataModel.specialAction[sEANKey].triggers[sActionType][sActionIndex].name);
                }

            }
        }
        if (dss.staticDataModel.specialAction[sEANKey].conditions != null) {
            for (var sActionType in dss.staticDataModel.specialAction[sEANKey].conditions) {
                for (var sActionIndex in dss.staticDataModel.specialAction[sEANKey].conditions[sActionType]) {
                    dss.staticDataModel.specialAction[sEANKey].conditions[sActionType][sActionIndex].name = dss.staticDataModel
                            .localizeName(dss.staticDataModel.specialAction[sEANKey].conditions[sActionType][sActionIndex].name);
                }

            }
        }
    }

    var resultObj;
    try {
        var sResult = dss.ajaxSyncRequest('/json/property/query2', {
            query: '/scripts/system-addon-scene-responder/specialDSIDAction/*(*)/*(*)/*(*)/*(*)/*(*)'
        });
        resultObj = Ext.JSON.decode(sResult);
    } catch (e) {
        devLog && devLog(e);
    }
    if (resultObj && resultObj.ok) {
        dss.staticDataModel.specialDSIDAction = resultObj.result;

        for (var sEANKey in dss.staticDataModel.specialDSIDAction) {
            if (dss.staticDataModel.specialDSIDAction[sEANKey].actions != null) {
                for (var sActionType in dss.staticDataModel.specialDSIDAction[sEANKey].actions) {
                    for (var sActionIndex in dss.staticDataModel.specialDSIDAction[sEANKey].actions[sActionType]) {
                        dss.staticDataModel.specialDSIDAction[sEANKey].actions[sActionType][sActionIndex].name = dss.staticDataModel
                                .localizeName(dss.staticDataModel.specialDSIDAction[sEANKey].actions[sActionType][sActionIndex].name);
                    }

                }
            }
            if (dss.staticDataModel.specialDSIDAction[sEANKey].triggers != null) {
                for (var sActionType in dss.staticDataModel.specialDSIDAction[sEANKey].triggers) {
                    for (var sActionIndex in dss.staticDataModel.specialDSIDAction[sEANKey].triggers[sActionType]) {
                        dss.staticDataModel.specialDSIDAction[sEANKey].triggers[sActionType][sActionIndex].name = dss.staticDataModel
                                .localizeName(dss.staticDataModel.specialDSIDAction[sEANKey].triggers[sActionType][sActionIndex].name);
                    }

                }
            }
        }
    }
};

dss.staticDataModel.initUserStateCache = function () {
    var result;
    try {
        var sString = dss.ajaxSyncRequest('/json/property/query', {
            query: '/usr/addon-states/system-addon-user-defined-states/*(name,displayName,setName,resetName,value,type)'
        });
        result = Ext.JSON.decode(sString);
    } catch (e) {
        devLog && devLog(e);
    }
    if (result && result.ok && result.result) {
        dss.staticDataModel.userStates = result.result['system-addon-user-defined-states'];
        if (dss.staticDataModel.userStates === undefined) {
            dss.staticDataModel.userStates = [];
        }
        for (var iIndex = 0; iIndex < dss.staticDataModel.userStates.length; iIndex++) {
            dss.staticDataModel.userStates[iIndex].displayName = Ext.String.htmlEncode(dss.unescapeProperty(dss.staticDataModel.userStates[iIndex].displayName));
            dss.staticDataModel.userStates[iIndex].setName = Ext.String.htmlEncode(dss.unescapeProperty(dss.staticDataModel.userStates[iIndex].setName));
            dss.staticDataModel.userStates[iIndex].resetName = Ext.String.htmlEncode(dss.unescapeProperty(dss.staticDataModel.userStates[iIndex].resetName));
        }
        dss.staticDataModel.userStates = dss.staticDataModel.userStates.sort(function (s1, s2) {
            if (s1.displayName != s2.displayName) {
                if ((s1.displayName).toUpperCase() < (s2.displayName).toUpperCase()) {
                    return -1;
                }
                if ((s1.displayName).toUpperCase() > (s2.displayName).toUpperCase()) {
                    return 1;
                }
            }
            return 0;
        });
    }
};

dss.staticDataModel.initPlaylistCache = function (vdcDsuid) {
    if (!vdcDsuid) {
        return;
    }
    try {
        const response = dss.ajaxSyncRequest('/json/property/vdcquery', {
            query: 'getProperty(deviceId=' + vdcDsuid + ')/playlists(*)'
        });
        const result = Ext.JSON.decode(response);
        const playlistsString = result && result.ok && result.result && result.result.elements &&
                result.result.elements && result.result.elements[0] && result.result.elements[0].value;
        dss.staticDataModel.playlists = JSON.parse(playlistsString);
    } catch (e) {
        devLog && devLog(e);
    }
};

dss.staticDataModel.initFavoriteCache = function (vdcDsuid) {
    if (!vdcDsuid) {
        return;
    }
    try {
        const response = dss.ajaxSyncRequest('/json/property/vdcquery', {
            query: 'getProperty(deviceId=' + vdcDsuid + ')/favorites(*)'
        });
        const result = Ext.JSON.decode(response);
        const favoritesString = result && result.ok && result.result && result.result.elements &&
                result.result.elements && result.result.elements[0] && result.result.elements[0].value;
        dss.staticDataModel.favorites = JSON.parse(favoritesString);
    } catch (e) {
        devLog && devLog(e);
    }
};

dss.staticDataModel.initClipsCache = function () {
    try {
        const response = dss.ajaxSyncRequest('/api/v1/apartment/mediaLibrary/clips/', {});
        const result = JSON.parse(response);
        dss.staticDataModel.clips = result && result.data && result.data.clips;
    } catch (e) {
        devLog && devLog(e);
    }
};

dss.staticDataModel.initAudioActivityCache = function (vdcDsuid) {
    if (!vdcDsuid) {
        return;
    }
    try {
        const response = dss.ajaxSyncRequest('/json/property/vdcquery', {
            query: 'getProperty(deviceId=' + vdcDsuid + ')/activities(*)'
        });
        const result = Ext.JSON.decode(response);
        const activitiesString = result && result.ok && result.result && result.result.elements &&
                result.result.elements && result.result.elements[0] && result.result.elements[0].value;
        dss.staticDataModel.audioActivities = JSON.parse(activitiesString)
                .map(function (activity) {
                    const activityWithVdcDsuid = {
                        id: activity.id,
                        vdcDsuid: vdcDsuid,
                        name: activity.name,
                        whatToPlay: activity.whatToPlay,
                        zones: activity.zones,
                        repeat: activity.whatToPlay.repeatAll,
                        restart: activity.restart,
                        shuffle: activity.whatToPlay.shuffle,
                        notification: activity.notification,
                        volume: activity.volume
                    };
                    return activityWithVdcDsuid;
                });
    } catch (e) {
        devLog && devLog(e);
    }
};

dss.staticDataModel.initMeterCache = function () {
    var resultObj;
    try {
        var sResult = dss.ajaxSyncRequest('/json/property/query', {
            query: dss.staticDataModel.sQueryStringMeters
        });
        resultObj = Ext.JSON.decode(sResult);
    } catch (e) {
        devLog && devLog(e);
    }
    if (resultObj && resultObj.ok) {
        if (resultObj.result.dSMeters != null) {
            dss.staticDataModel.MeterCache = resultObj.result.dSMeters;
        }
    }
};

dss.staticDataModel.getMeter = function (dSUID) {
    var tempdSUID;
    if (dss.staticDataModel.MeterCache === null) {
        dss.staticDataModel.initCaches();
    }
    if (DSS.types.dsuid.isDSUID(dSUID)) {
        if (dss.staticDataModel.MeterCache) {
            for (var iIndex = 0; iIndex < dss.staticDataModel.MeterCache.length; iIndex++) {
                if (dss.staticDataModel.MeterCache[iIndex].dSUID && (dss.staticDataModel.MeterCache[iIndex].dSUID === dSUID)) {
                    return dss.staticDataModel.MeterCache[iIndex];
                }
            }
        }
    } else {
        // we still got a dSID so lets try whether we find a matching
        if (dss.staticDataModel.MeterCache) {
            for (var iIndex = 0; iIndex < dss.staticDataModel.MeterCache.length; iIndex++) {
                if (dss.staticDataModel.MeterCache[iIndex].dSID && (dss.staticDataModel.MeterCache[iIndex].dSID === dSUID)) {
                    return dss.staticDataModel.MeterCache[iIndex];
                } else if (dss.staticDataModel.MeterCache[iIndex].dSUID) {
                    tempdSUID = DSS.types.dsuid.dsid_to_dsuid(dSUID);
                    if (dss.staticDataModel.MeterCache[iIndex].dSUID === tempdSUID) {
                        return dss.staticDataModel.MeterCache[iIndex];
                    }
                }
            }
        }
    }
    return null;
};

dss.staticDataModel.getMeterName = function (MeterDSUID) {
    if (dss.staticDataModel.MeterCache == null) {
        dss.staticDataModel.initCaches();
    }
    var baseName = Ext.String.format(_('Meter dSUID {dsuid}'), {
        dsuid: DSS.types.dsuid.shortdSUID(MeterDSUID)
    });

    if (dss.staticDataModel.MeterCache) {
        for (var iIndex = 0; iIndex < dss.staticDataModel.MeterCache.length; iIndex++) {
            if (dss.staticDataModel.MeterCache[iIndex].dSUID == MeterDSUID) {
                if (dss.staticDataModel.MeterCache[iIndex].name != '') {
                    return dss.staticDataModel.MeterCache[iIndex].name;
                }
            }
        }
    }
    return baseName;
};

dss.staticDataModel.getImagePathForState = function (stateName, stateValue) {
    if (stateName == null) {
        return '';
    }

    switch (stateName.split('.')[0]) {
        case 'rain':
            if ((stateValue === 'active') || (stateValue == '1')) {
                return dss.staticDataModel.baseImagePath + 'state/rain.png';
            }
            if ((stateValue === 'inactive') || (stateValue == '2')) {
                return dss.staticDataModel.baseImagePath + 'state/rain_No.png';
            }
            break;

        case 'wind':
            if ((stateValue === 'active') || (stateValue == '1')) {
                return dss.staticDataModel.baseImagePath + 'state/wind.png';
            }
            if ((stateValue === 'inactive') || (stateValue == '2')) {
                return dss.staticDataModel.baseImagePath + 'state/wind_No.png';
            }
            break;

        case 'hail':
            if ((stateValue === 'active') || (stateValue == '1')) {
                return dss.staticDataModel.baseImagePath + 'state/hail.png';
            }
            if ((stateValue === 'inactive') || (stateValue == '2')) {
                return dss.staticDataModel.baseImagePath + 'state/hail_No.png';
            }
            break;

        case 'panic':
            if ((stateValue === 'active') || (stateValue == '1')) {
                return dss.staticDataModel.baseImagePath + 'state/panic.png';
            }
            if ((stateValue === 'inactive') || (stateValue == '2')) {
                return dss.staticDataModel.baseImagePath + 'state/panic_No.png';
            }
            break;

        case 'fire':
            if ((stateValue === 'active') || (stateValue == '1')) {
                return dss.staticDataModel.baseImagePath + 'state/fire.png';
            }
            if ((stateValue === 'inactive') || (stateValue == '2')) {
                return dss.staticDataModel.baseImagePath + 'state/fire_No.png';
            }
            break;

        case 'alarm':
            if ((stateValue === 'active') || (stateValue == '1')) {
                return dss.staticDataModel.baseImagePath + 'state/alarm.png';
            }
            if ((stateValue === 'inactive') || (stateValue == '2')) {
                return dss.staticDataModel.baseImagePath + 'state/alarm_No.png';
            }
            break;

        case 'alarm2':
            if ((stateValue === 'active') || (stateValue == '1')) {
                return dss.staticDataModel.baseImagePath + 'state/alarm2.png';
            }
            if ((stateValue === 'inactive') || (stateValue == '2')) {
                return dss.staticDataModel.baseImagePath + 'state/alarm2_No.png';
            }
            break;

        case 'alarm3':
            if ((stateValue === 'active') || (stateValue == '1')) {
                return dss.staticDataModel.baseImagePath + 'state/alarm3.png';
            }
            if ((stateValue === 'inactive') || (stateValue == '2')) {
                return dss.staticDataModel.baseImagePath + 'state/alarm3_No.png';
            }
            break;

        case 'alarm4':
            if ((stateValue === 'active') || (stateValue == '1')) {
                return dss.staticDataModel.baseImagePath + 'state/alarm4.png';
            }
            if ((stateValue === 'inactive') || (stateValue == '2')) {
                return dss.staticDataModel.baseImagePath + 'state/alarm4_No.png';
            }
            break;

        case 'holiday':
            if ((stateValue === 'active') || (stateValue == '1') || (stateValue === 'on')) {
                return dss.staticDataModel.baseImagePath + 'state/simulation_on.png';
            }
            if ((stateValue === 'inactive') || (stateValue == '2') || (stateValue === 'off')) {
                return dss.staticDataModel.baseImagePath + 'state/simulation_off.png';
            }
            break;

        case 'daylight':
        case 'daylight_state':
            if ((stateValue === 'active') || (stateValue == '1') || (stateValue == true)) {
                return dss.staticDataModel.baseImagePath + 'state/daylight_true.png';
            }
            if ((stateValue === 'inactive') || (stateValue == '2') || (stateValue == false)) {
                return dss.staticDataModel.baseImagePath + 'state/daylight_false.png';
            }
            break;

        case 'daynight':
        case 'daynight_state':
            if ((stateValue === 'active') || (stateValue == '1') || (stateValue == true)) {
                return dss.staticDataModel.baseImagePath + 'state/daynight_true.png';
            }
            if ((stateValue === 'inactive') || (stateValue == '2') || (stateValue == false)) {
                return dss.staticDataModel.baseImagePath + 'state/daynight_false.png';
            }
            break;

        case 'daynight_indoors':
        case 'daynight_indoors_state':
            if ((stateValue === 'active') || (stateValue == '1') || (stateValue == true)) {
                return dss.staticDataModel.baseImagePath + 'state/daynight_indoors_true.png';
            }
            if ((stateValue === 'inactive') || (stateValue == '2') || (stateValue == false)) {
                return dss.staticDataModel.baseImagePath + 'state/daynight_indoors_false.png';
            }
            break;

        case 'presence':
            if ((stateValue === 'active') || (stateValue == '1') || (stateValue === 'present')) {
                return dss.staticDataModel.baseImagePath + 'state/present.png';
            }
            if ((stateValue === 'inactive') || (stateValue == '2') || (stateValue === 'absent')) {
                return dss.staticDataModel.baseImagePath + 'state/present_No.png';
            }
            break;

        case 'frost':
            if ((stateValue === 'active') || (stateValue == '1')) {
                return dss.staticDataModel.baseImagePath + 'state/frost.png';
            }
            if ((stateValue === 'inactive') || (stateValue == '2')) {
                return dss.staticDataModel.baseImagePath + 'state/frost_no.png';
            }
            break;

        case 'sun':
            if ((stateValue === 'active') || (stateValue == '1')) {
                return dss.staticDataModel.baseImagePath + 'state/sun.png';
            }
            if ((stateValue === 'inactive') || (stateValue == '2')) {
                return dss.staticDataModel.baseImagePath + 'state/sun_no.png';
            }
            break;

        case 'heating_water_system':
            if ((stateValue === 'off') || (stateValue == '0')) {
                return dss.staticDataModel.baseImagePath + 'state/heating_mode_off.png';
            }
            if ((stateValue == 'hot water') || (stateValue == '1')) {
                return dss.staticDataModel.baseImagePath + 'state/heating_mode_heating.png';
            }
            if ((stateValue == 'cold water') || (stateValue == '2')) {
                return dss.staticDataModel.baseImagePath + 'state/heating_mode_cooling.png';
            }
            if ((stateValue === 'auto') || (stateValue == '3')) {
                return dss.staticDataModel.baseImagePath + 'state/heating_mode_auto.png';
            }
            break;

        case 'service':
            if ((stateValue === 'active') || (stateValue == '1')) {
                return dss.staticDataModel.baseImagePath + 'state/service.png';
            }
            if ((stateValue === 'inactive') || (stateValue == '2')) {
                return dss.staticDataModel.baseImagePath + 'state/service_no.png';
            }
            break;

        case 'operation_lock':
            if ((stateValue === 'active') || (stateValue == '1')) {
                return dss.staticDataModel.baseImagePath + 'state/operation_lock.png';
            }
            if ((stateValue === 'inactive') || (stateValue == '2')) {
                return dss.staticDataModel.baseImagePath + 'state/operation_lock_no.png';
            }
            break;
    }

    if (stateName.match(/^zone\.[0-9]+\.group\.[0-9]+\.status\.malfunction$/gi)) {
        return dss.staticDataModel.baseImagePath + (stateValue === 'active' || stateValue === '1' ? 'state/malfunction.png' : 'state/malfunction_no.png');
    }
    if (stateName.match(/^zone\.[0-9]+\.group\.[0-9]+\.status\.service$/gi)) {
        return dss.staticDataModel.baseImagePath + (stateValue === 'active' || stateValue === '1' ? 'state/akm_service.png' : 'state/akm_service_no.png');
    }

    return null;
};

dss.staticDataModel.getImagePathForConditionState = function (ConditionActive) {
    var imagePath = '';
    if (ConditionActive) {
        imagePath = dss.staticDataModel.baseImagePath + 'dss/bedingung_aktiv.png';
    } else {
        imagePath = dss.staticDataModel.baseImagePath + 'dss/bedingung_inaktiv.png';
    }
    return imagePath;
};

dss.staticDataModel.getNameForState = function (stateName, stateValue) {
    switch (stateName) {

        case 'rain':
            if (stateValue === 'active') {
                return _('rain active');
            }
            if (stateValue === 'inactive') {
                return _('rain inactive');
            }
            break;

        case 'wind':
            if (stateValue === 'active') {
                return _('wind active');
            }
            if (stateValue === 'inactive') {
                return _('wind inactive');
            }
            break;

        case 'hail':
            if (stateValue === 'active') {
                return _('hail active');
            }
            if (stateValue === 'inactive') {
                return _('hail inactive');
            }
            break;

        case 'fire':
            if (stateValue === 'active') {
                return _('fire active');
            }
            if (stateValue === 'inactive') {
                return _('fire inactive');
            }
            break;

        case 'frost':
            if (stateValue === 'active') {
                return _('frost active');
            }
            if (stateValue === 'inactive') {
                return _('frost inactive');
            }
            break;

        case 'sun':
            if (stateValue === 'active') {
                return _('sun active');
            }
            if (stateValue === 'inactive') {
                return _('sun inactive');
            }
            break;

        case 'heating_water_system':
            if (stateValue === 'off') {
                return _('heating mode off');
            }
            if (stateValue == 'hot water') {
                return _('heating mode heating');
            }
            if (stateValue == 'cold water') {
                return _('heating mode cooling');
            }
            if (stateValue === 'auto') {
                return _('heating mode auto');
            }
            break;

        case 'heating_mode':
            if (stateValue === 'off') {
                return _('heating mode off');
            }
            if (stateValue === 'heating') {
                return _('heating mode heating');
            }
            if (stateValue === 'cooling') {
                return _('heating mode cooling');
            }
            if (stateValue === 'auto') {
                return _('heating mode auto');
            }
            break;

        case 'light':
            if (stateValue === 'active') {
                return Ext.String.format2(_('{#editor_condition_zonestates_light}light is on'), {});
            }
            if (stateValue === 'inactive') {
                return _(Ext.String.format2(_('{#editor_condition_zonestates_light}light is off'), {}));
            }
            break;

        case 'service':
            if (stateValue === 'active') {
                return _('Service needed');
            }
            if (stateValue === 'inactive') {
                return _('no Service needed');
            }
            break;

        case 'malfunction':
            if (stateValue === 'active') {
                return _('Malfunction detected');
            }
            if (stateValue === 'inactive') {
                return _('no Malfunction');
            }
            break;
    }

    // currently: apartment ventilation, zone ventilation and air recirculation
    // if (stateName.match(/^zone\.[0-9]+\.group\.[0-9]+\.status\.malfunction$/gi)) {
    //     return (stateValue === 'active' || stateValue === '1' ? _('Malfunction detected') : _('no Malfunction'));
    // }
    // if (stateName.match(/^zone\.[0-9]+\.group\.[0-9]+\.status\.service$/gi)) {
    //     return (stateValue === 'active' || stateValue === '1' ? _('Service needed') : _('no Service needed'));
    // }

    return _(stateName + '_' + stateValue);
};

dss.staticDataModel.getSunrise = function () {
    if (dss.staticDataModel.nowSunrise == null) {
        dss.staticDataModel.initSunriseTimes();
    }
    return dss.staticDataModel.nowSunrise;
};

dss.staticDataModel.getSunset = function () {
    if (dss.staticDataModel.nowSunset == null) {
        dss.staticDataModel.initSunriseTimes();
    }
    return dss.staticDataModel.nowSunset;
};

dss.staticDataModel.getDawn = function () {
    if (dss.staticDataModel.nowDawn == null) {
        dss.staticDataModel.initSunriseTimes();
    }
    return dss.staticDataModel.nowDawn;
};

dss.staticDataModel.getDusk = function () {
    if (dss.staticDataModel.nowDusk == null) {
        dss.staticDataModel.initSunriseTimes();
    }
    return dss.staticDataModel.nowDusk;
};

dss.staticDataModel.deltaTime = null;

dss.staticDataModel.getDSSTime = function () {
    var dReference = new Date(); // 10:00
    if (dss.staticDataModel.deltaTime == null) {
        var result;
        try {
            var sResult = dss.ajaxSyncRequest('/json/property/query2', {
                query: '/system/time(current_sec,gmt_offset)'
            });
            result = Ext.JSON.decode(sResult);
        } catch (e) {
            devLog && devLog(e);
        }
        if (result && result.ok) {
            if (result.result && result.result.time != null) {
                dss.staticDataModel.deltaTime = dReference.getTime() - (result.result.time.current_sec * 1000); // -1:00
                var oBrowserOffset = dReference.getTimezoneOffset() * 60 * 1000;
                var oDSSOffset = result.result.time.gmt_offset * 1000;
                dss.staticDataModel.deltaTime -= (oDSSOffset + oBrowserOffset);
            } else {
                var sResponseText = dss.ajaxSyncRequest('/cgi-bin/dateinfo.sh', {});
                if (sResponseText.substring(0, 9) == 'Error 404') {
                    dss.staticDataModel.deltaTime = 0;
                } else {
                    var resultObj;
                    try {
                        resultObj = Ext.JSON.decode(sResponseText);
                    } catch (e) {
                        devLog && devLog(e);
                    }
                    if (resultObj && resultObj.success) { // 10:00 //11:00
                        dss.staticDataModel.deltaTime = (
                            resultObj.data
                                ? dReference.getTime() - (resultObj.data.utcsecs * 1000)
                                : dReference.getTime()
                        ); // -1:00
                        var oBrowserOffset = dReference.getTimezoneOffset() * 60 * 1000;
                        var oDSSOffset = (resultObj.data ? resultObj.data.gmtoffset * 60 * 60 * 1000 : 0);
                        dss.staticDataModel.deltaTime -= (oDSSOffset + oBrowserOffset);
                    }
                }
            }
        }
        dReference = new Date();

    } // 10:00 //-1:00
    return new Date(dReference.getTime() - dss.staticDataModel.deltaTime); // 11:00
};

dss.staticDataModel.initSunriseTimeAsync = function () {
    dss.ajaxAssyncRequestWithoutAnswer('/json/property/query2', {
        query: '/config/geodata(sunset,sunrise,civil_dawn,civil_dusk)'
    }, function (sString) {
        var result;
        try {
            result = Ext.JSON.decode(sString);
        } catch (e) {
            devLog && devLog(e);
        }
        if (result && result.ok && result.result && typeof result.result.geodata !== 'undefined') {
            dss.staticDataModel.nowDawn = result.result.geodata.civil_dawn;
            dss.staticDataModel.nowDusk = result.result.geodata.civil_dusk;
            dss.staticDataModel.nowSunrise = result.result.geodata.sunrise;
            dss.staticDataModel.nowSunset = result.result.geodata.sunset;
        }
    });
};

dss.staticDataModel.initSunriseTimes = function () {
    var result;
    try {
        var sResult = dss.ajaxSyncRequest('/json/property/query2', {
            query: '/config/geodata(sunset,sunrise,civil_dawn,civil_dusk)'
        });
        result = Ext.JSON.decode(sResult);
    } catch (e) {
        devLog && devLog(e);
    }
    if (result && result.ok && result.result && typeof result.result.geodata !== 'undefined') {
        dss.staticDataModel.nowDawn = result.result.geodata.civil_dawn;
        dss.staticDataModel.nowDusk = result.result.geodata.civil_dusk;
        dss.staticDataModel.nowSunrise = result.result.geodata.sunrise;
        dss.staticDataModel.nowSunset = result.result.geodata.sunset;
    }
};

/**
        * Returns the distroVersion of the dSS. If not yet stored it will be read from
        * the property tree.
        *
        * @return distroVersionString
        */
dss.staticDataModel.getdSSVersion = function () {
    if (dss.staticDataModel.dSSVersion == null) {
        var result;
        try {
            var sResult = dss.ajaxSyncRequest('/json/property/query2', {
                query: '/system/version(distroVersion)'
            });
            result = Ext.JSON.decode(sResult);
        } catch (e) {
            devLog && devLog(e);
        }
        if (result && result.ok && result.result && typeof result.result.version) {
            dss.staticDataModel.dSSVersion = result.result.version.distroVersion;
        } else {
            dss.staticDataModel.dSSVersion = 'unknown';
        }
    }
    return dss.staticDataModel.dSSVersion;
};

/**
        * Returns the distroVersion of the dSS. If not yet stored it will be read from
        * the property tree.
        *
        * @return distroVersionString
        */
dss.staticDataModel.getdSSFeed = function () {
    if (dss.staticDataModel.dSSFeed == null) {
        var result;
        try {
            var sResult = dss.ajaxSyncRequest('/json/property/query2', {
                query: '/system/version(releasetype)'
            });
            result = Ext.JSON.decode(sResult);
        } catch (e) {
            devLog && devLog(e);
        }
        if (result && result.ok && result.result.version) {
            dss.staticDataModel.dSSFeed = result.result.version.releasetype;
        } else {
            dss.staticDataModel.dSSFeed = 'unknown';
        }
    }
    return dss.staticDataModel.dSSFeed;
};

dss.staticDataModel.initNameCache = function () {
    var resultObj;
    try {
        var sResult = dss.ajaxSyncRequest('/json/property/query', {
            query: dss.staticDataModel.sQueryStringStructure
        });
        resultObj = Ext.JSON.decode(sResult);
    } catch (e) {
        devLog && devLog(e);
    }
    if (resultObj && resultObj.ok && resultObj.result) {
        if (resultObj.result.zones != null) {
            dss.staticDataModel.nameCache = resultObj.result.zones;

            for (var i = 0; i < dss.staticDataModel.nameCache.length; i++) {
                for (var j = 0; j < dss.staticDataModel.nameCache[i].groups.length; j++) {
                    if (dss.staticDataModel.nameCache[i].groups[j].group !== 10 && dss.staticDataModel.nameCache[i].groups[j].group !== 12) {
                        continue;
                    }
                    var scenes = dss.staticDataModel.nameCache[i].groups[j].activeBasicScenes.split(',');
                    dss.staticDataModel.nameCache[i].groups[j].activeBasicScenes = scenes.join(',');
                }
            }
            /* TEMP end */
        }
    }
};

dss.staticDataModel.initDeviceCache = function () {
    var oVDC_DSID = [];
    var resultObj;
    try {
        var sResult = dss.ajaxSyncRequest('/json/property/query', {
            query: dss.staticDataModel.sQueryStringDevices
        });
        resultObj = Ext.JSON.decode(sResult);
    } catch (e) {
        devLog && devLog(e);
    }
    if (resultObj && resultObj.ok) {
        dss.staticDataModel.deviceCache = [];
        dss.staticDataModel.deviceWithSensorsCache = [];
        dss.staticDataModel.deviceAKMCache = [];
        if (resultObj.result && resultObj.result.devices != null) {
            for (var iIndex = 0; iIndex < resultObj.result.devices.length; iIndex++) {
                var oDevObject = resultObj.result.devices[iIndex];

                if (oDevObject.ZoneID === 65534) {
                    continue;
                }

                var newObject = {
                    dSID: oDevObject.dSID,
                    dSUID: oDevObject.dSUID,
                    name: oDevObject.name,
                    present: oDevObject.present,
                    functionID: oDevObject.functionID,
                    productID: oDevObject.productID,
                    outputMode: oDevObject.outputMode,
                    ZoneID: oDevObject.ZoneID,
                    DSMeterDSID: oDevObject.DSMeterDSID,
                    DSMeterDSUID: oDevObject.DSMeterDSUID,
                    HWInfo: oDevObject.HWInfo,
                    DisplayID: oDevObject.DisplayID,
                    isVdcDevice: oDevObject.isVdcDevice,
                    GroupMembership: [],
                    sensors: [],
                    specialActions: {},
                    moc: [],
                    modelFeatures: {}
                };
                if (oDevObject.hasActions != null) {
                    newObject.hasActions = oDevObject.hasActions;
                } else {
                    newObject.hasActions = false;
                }
                newObject.hasStates = false;
                newObject.hasEvents = false;
                if (oDevObject.isVdcDevice) {
                    if (oDevObject[oDevObject.dSUID] != null) {
                        oVDC_DSID.push(oDevObject.dSUID);
                        var oArray = oDevObject[oDevObject.dSUID];
                        if (oArray) {
                            for (var jIndex = 0; jIndex < oArray.length; jIndex++) {
                                if (oArray[jIndex].states != null) {
                                    if (oArray[jIndex].states.length > 0) {
                                        newObject.hasStates = true;
                                    }
                                }
                            }
                        }
                    }
                }
                var subObject = oDevObject[oDevObject.dSUID];
                for (var jIndex = 0; subObject && jIndex < subObject.length; jIndex++) {
                    if (subObject[jIndex].outputChannels != null) {
                        for (var kIndex = 0; kIndex < subObject[jIndex].outputChannels.length; kIndex++) {
                            newObject.moc.push(subObject[jIndex].outputChannels[kIndex].channelID);
                        }
                    }
                    if (subObject[jIndex].binaryInputs != null) {
                        newObject.binaryInputs = [];
                        for (var kIndex = 0; kIndex < subObject[jIndex].binaryInputs.length; kIndex++) {
                            newObject.binaryInputs.push({
                                independent: subObject[jIndex].binaryInputs[kIndex].independent,
                                type: subObject[jIndex].binaryInputs[kIndex].inputType,
                                index: subObject[jIndex].binaryInputs[kIndex].inputIndex,
                                group: subObject[jIndex].binaryInputs[kIndex].targetGroupId
                            });
                        }
                    }
                    if (subObject[jIndex].sensorEvents != null) {
                        newObject.sensorEvents = [];
                        for (var sKey in subObject[jIndex]) {
                            if (sKey != 'sensorEvents') {
                                newObject.sensorEvents.push({
                                    name: subObject[jIndex][sKey],
                                    id: sKey
                                });
                            }
                        }
                    }

                    if (subObject[jIndex].sensorInputs != null) {
                        for (var sIndex = 0; sIndex < subObject[jIndex].sensorInputs.length; sIndex++) {
                            newObject.sensors.push({
                                type: subObject[jIndex].sensorInputs[sIndex].type
                            });
                        }
                    }
                    if ((subObject[jIndex].EAN != null) && (dss.staticDataModel.specialAction !== undefined)) {
                        if (dss.staticDataModel.specialAction[subObject[jIndex].EAN] != null) {
                            newObject.specialActions = dss.staticDataModel.specialAction[subObject[jIndex].EAN];
                        } else if (dss.staticDataModel.specialAction[resultObj.result.devices[iIndex].functionID] != null) {
                            newObject.specialActions = dss.staticDataModel.specialAction[resultObj.result.devices[iIndex].functionID];
                        }
                    }

                    if ((dss.staticDataModel.specialDSIDAction !== undefined) && (dss.staticDataModel.specialDSIDAction[oDevObject.dSUID] != null)) {
                        if (dss.staticDataModel.specialDSIDAction[oDevObject.dSUID].actions != null) {
                            for (var sKeyType in dss.staticDataModel.specialDSIDAction[oDevObject.dSUID].actions) {
                                if (newObject.specialActions.actions == null) {
                                    newObject.specialActions.actions = {};
                                }
                                if (newObject.specialActions.actions[sKeyType] == null) {
                                    newObject.specialActions.actions[sKeyType] = {};
                                }
                                for (var sSubSubKey in dss.staticDataModel.specialDSIDAction[oDevObject.dSUID].actions[sKeyType]) {
                                    newObject.specialActions.actions[sKeyType][sSubSubKey] = dss.staticDataModel.specialDSIDAction[oDevObject.dSUID].actions[sKeyType][sSubSubKey];
                                }
                            }
                        }
                        if (dss.staticDataModel.specialDSIDAction[oDevObject.dSUID].triggers != null) {
                            for (var sKeyType in dss.staticDataModel.specialDSIDAction[oDevObject.dSUID].triggers) {
                                if (newObject.specialActions.triggers == null) {
                                    newObject.specialActions.triggers = {};
                                }
                                if (newObject.specialActions.triggers[sKeyType] == null) {
                                    newObject.specialActions.triggers[sKeyType] = {};
                                }
                                newObject.specialActions.triggers[sKeyType] = [];
                                for (var sSubSubKey in dss.staticDataModel.specialDSIDAction[oDevObject.dSUID].triggers[sKeyType]) {
                                    newObject.specialActions.triggers[sKeyType][sSubSubKey] = dss.staticDataModel.specialDSIDAction[oDevObject.dSUID]
                                        .triggers[sKeyType][sSubSubKey];
                                }
                            }
                        }
                    }
                    if (subObject[jIndex].modelFeatures != null) {
                        for (var sKey in subObject[jIndex]) {
                            if (sKey != 'modelFeatures') {
                                newObject.modelFeatures[sKey] = subObject[jIndex][sKey];
                            }
                        }
                    }

                    if (subObject[jIndex].button != null) {
                        if (subObject[jIndex].id != null) {
                            newObject.tasterMode = subObject[jIndex].id;
                        }
                        if (subObject[jIndex].groupMembership != null) {
                            newObject.buttonGroupMembership = subObject[jIndex].groupMembership;
                        }
                        if (subObject[jIndex].activeGroup != null) {
                            newObject.buttonActiveGroup = subObject[jIndex].activeGroup;
                        }
                        if (subObject[jIndex].setsLocalPriority != null) {
                            newObject.buttonSetsLocalPriority = subObject[jIndex].setsLocalPriority;
                        }
                        if (subObject[jIndex].inputMode != null) {
                            newObject.buttonType = subObject[jIndex].inputMode;
                        }
                        if (subObject[jIndex].usage != null) {
                            newObject.buttonUsage = subObject[jIndex].usage;
                        }
                        if (newObject.buttonActiveGroup == 255) {
                            newObject.buttonActiveGroup = 8;
                        }
                    }
                    if (subObject[jIndex].groups != null) {
                        for (var kIndex = 0; kIndex < subObject[jIndex].groups.length; kIndex++) {
                            newObject.GroupMembership.push(subObject[jIndex].groups[kIndex].id);
                            if (subObject[jIndex].groups[kIndex].id == 64) {
                                dss.staticDataModel.globalApplicationsAvaible = true;
                            }
                            if (subObject[jIndex].groups[kIndex].id != 8) // 8
                            // kann
                            // nicht
                            // ausf�hrungsgruppe
                            // sein
                            {
                                if (subObject[jIndex].groups[kIndex].id < 16) // nur
                                // in
                                // den
                                // basisgruppen
                                // bleiben
                                { newObject.GroupID = subObject[jIndex].groups[kIndex].id; }
                            } // base
                            // color
                            // Group
                        }
                        if (newObject.GroupID == null) {
                            if (newObject.GroupMembership.length === 1) {
                                newObject.GroupID = newObject.GroupMembership[0];
                            }
                        }
                    }
                }

                if (newObject.specialActions.triggers != null) {
                    if (newObject.specialActions.triggers.deviceEvents != null) {
                        if (newObject.sensorEvents == null) {
                            newObject.sensorEvents = [];
                        }
                        for (var sKey in newObject.specialActions.triggers.deviceEvents) {
                            var oObject = newObject.specialActions.triggers.deviceEvents[sKey];
                            newObject.sensorEvents.push({
                                name: oObject.name,
                                id: oObject.id
                            });
                        }
                    }
                }

                if (newObject.name == '') {
                    newObject.name = '(' + newObject.DisplayID + ')';
                }
                newObject.icon = dss.staticDataModel.getImageForDevice(newObject);
                dss.staticDataModel.deviceCache.push(newObject);
                if (newObject.sensorEvents != null) {
                    if (newObject.sensorEvents.length > 0) {
                        dss.staticDataModel.deviceWithSensorsCache.push(newObject);
                    }
                }
                if (newObject.binaryInputs != null) {
                    if (newObject.binaryInputs.length > 0) {

                        if (dss.staticDataModel.hasDeviceStates(newObject.dSUID)) {
                            var fShouldAdd = false;
                            for (var sKeyBin in newObject.binaryInputs) {
                                // if device's binaryInput is independent from button
                                // input or has no button input at all
                                if (newObject.binaryInputs[sKeyBin].independent ||
                                            newObject.buttonUsage === 'unsupported') { fShouldAdd = true; }
                            }
                            if (newObject.buttonType >= 16) {
                                fShouldAdd = true;
                            } else if (newObject.isVdcDevice) {
                                fShouldAdd = true;
                            } else if (newObject.specialActions != null) {
                                if (newObject.specialActions.triggers != null) {
                                    if (newObject.specialActions.triggers.binaryInputs != null) {
                                        fShouldAdd = true;
                                    }
                                }
                            }
                            if (fShouldAdd) {
                                dss.staticDataModel.deviceAKMCache.push(newObject);
                            }
                        }
                    }
                }
            }
        }
    }
    var zone0Index = -1;
    var groupToAdd = [];
    if (dss.staticDataModel.nameCache) {
        for (var iIndex = 0; iIndex < dss.staticDataModel.nameCache.length; iIndex++) {
            var oZone = dss.staticDataModel.nameCache[iIndex];
            if (oZone && oZone.ZoneID === 0) {
                zone0Index = iIndex;
            }
            if (oZone && oZone.name == '') {
                if (oZone.ZoneID == 0) {
                    oZone.name = _('All rooms');
                } else {
                    oZone.name = Ext.String.format2(_('Room ID {zone}'), {
                        zone: oZone.ZoneID
                    });
                }
            }
            if (oZone && oZone.groups) {
                for (var jIndex = 0; jIndex < oZone.groups.length; jIndex++) {
                    var oGroup = oZone.groups[jIndex];
                    oGroup.icon = dss.staticDataModel.getImagePathForGroup(oZone.ZoneID, oGroup.group);
                    oGroup.name = dss.staticDataModel.getNameForGroup(oZone.ZoneID, oGroup.group);
                    oGroup.sensors = [];
                    oGroup.reachable = false;
                    if (oGroup.group == 0) {
                        oGroup.reachable = true;
                    }
                    if (oGroup.connectedDevices > 0) {
                        oGroup.reachable = true;
                        if (groupToAdd.indexOf(oGroup.group) === -1) {
                            groupToAdd.push(oGroup.group);
                        }
                    }
                    if (dss.staticDataModel.deviceCache) {
                        for (var iDeviceIndex = 0; iDeviceIndex < dss.staticDataModel.deviceCache.length; iDeviceIndex++) {
                            if ((dss.staticDataModel.deviceCache[iDeviceIndex].ZoneID == oZone.ZoneID) || (oZone.ZoneID == 0)) {
                                if (dss.staticDataModel.deviceCache[iDeviceIndex].GroupMembership != null) {
                                    for (var iDeviceGroupIndex = 0; iDeviceGroupIndex < dss.staticDataModel.deviceCache[iDeviceIndex].GroupMembership.length; iDeviceGroupIndex++) {
                                        if (dss.staticDataModel.deviceCache[iDeviceIndex].GroupMembership[iDeviceGroupIndex] == oGroup.group) {
                                            oGroup.reachable = true;
                                        }
                                    }
                                }
                            }
                        }
                    }
                    dss.staticDataModel.nameCache[iIndex].groups[jIndex] = oGroup;
                }
            }
        }
    }
    if (groupToAdd.length > 0) {
        oZone = dss.staticDataModel.nameCache[zone0Index];
        if (oZone && oZone.groups) {
            for (var jIndex = 0; jIndex < oZone.groups.length; jIndex++) {
                var oGroup = oZone.groups[jIndex];
                if (groupToAdd.indexOf(oGroup.group) !== -1) {
                    oGroup.icon = dss.staticDataModel.getImagePathForGroup(oZone.ZoneID, oGroup.group);
                    oGroup.name = dss.staticDataModel.getNameForGroup(oZone.ZoneID, oGroup.group);
                    oGroup.reachable = true;
                    dss.staticDataModel.nameCache[zone0Index].groups[jIndex] = oGroup;
                }
            }
        }
    }

    dss.staticDataModel.userGroups = [];
    if (dss.staticDataModel.nameCache) {
        for (var iIndex = 0; iIndex < dss.staticDataModel.nameCache.length; iIndex++) {
            if (dss.staticDataModel.nameCache[iIndex].ZoneID == 0) {
                if (dss.staticDataModel.nameCache[iIndex].groups) {
                    for (var jIndex = 0; jIndex < dss.staticDataModel.nameCache[iIndex].groups.length; jIndex++) {
                        if (dss.staticDataModel.isClusterGroup(dss.staticDataModel.nameCache[iIndex].groups[jIndex].group)) {
                            if (dss.staticDataModel.nameCache[iIndex].groups[jIndex].color !== 0) {
                                dss.staticDataModel.userGroups.push({
                                    id: dss.staticDataModel.nameCache[iIndex].groups[jIndex].group,
                                    name: dss.staticDataModel.nameCache[iIndex].groups[jIndex].name,
                                    reachable: dss.staticDataModel.nameCache[iIndex].groups[jIndex].reachable,
                                    color: dss.staticDataModel.nameCache[iIndex].groups[jIndex].color
                                });
                            }
                        }
                    }
                }
            }
        }
    }
    for (var iIndex = 0; iIndex < oVDC_DSID.length; iIndex++) {
        dss.staticDataModel.getActions(oVDC_DSID[iIndex]);
    }

};

dss.staticDataModel.getActions = function (dsid) {
    var device = dss.staticDataModel.getDevice(dsid);
    if (device == null) {
        return;
    }
    if (!device.hasActions) {
        return;
    }
    if (device.deviceStandardActions == null) {
        var sResult;
        try {
            sResult = Ext.JSON.decode(dss.ajaxSyncRequest('/json/device/getInfo', {
                dsuid: device.dSUID,
                lang: dss.staticDataModel.activeLanguage
            }));
        } catch (e) {
            devLog && devLog(e);
        }
        if (sResult && sResult.ok && sResult.result) {
            device.deviceCategory = sResult.result.categories;
            device.deviceStandardActions = sResult.result.standardActions;
            device.deviceCustomActions = sResult.result.customActions;
            if (device.hasStates) {
                device.deviceStates = sResult.result.stateDescriptions;
            }
            device.deviceEvents = sResult.result.eventDescriptions;
            device.deviceActions = sResult.result.deviceActions;
            device.deviceActionDescription = sResult.result.actionDescriptions;
            if (device.deviceStandardActions != null) {
                for (var sKey in device.deviceStandardActions) {
                    var sTemplate = device.deviceStandardActions[sKey].action;
                    for (var dKey in device.deviceActionDescription) {
                        if (dKey == sTemplate) {
                            device.deviceStandardActions[sKey].description = device.deviceActionDescription[dKey];
                            if (device.deviceActionDescription[dKey].tags != null) {
                                var sTags = device.deviceActionDescription[dKey].tags.split(';');
                                for (var tIndex = 0; tIndex < sTags.length; tIndex++) {
                                    if (sTags[tIndex] === 'invisible') {
                                        delete device.deviceStandardActions[sKey];
                                    }
                                }
                            }

                        }
                    }
                }

            }
            if (device.deviceCustomActions != null) {
                for (var sKey in device.deviceCustomActions) {
                    var sTemplate = device.deviceCustomActions[sKey].action;
                    for (var dKey in device.deviceActionDescription) {
                        if (dKey == sTemplate) {
                            device.deviceCustomActions[sKey].description = device.deviceActionDescription[dKey];
                        }
                    }
                }
            }
        } else {
            device.deviceCategory = [];
            device.deviceStandardActions = [];
            device.deviceCustomActions = [];
            if (device.hasStates) {
                device.deviceStates = [];
            }
            device.deviceEvents = [];
            device.deviceActions = [];
            device.deviceActionDescription = [];
        }
    }
};

dss.staticDataModel.getStandardActions = function (dsid) {
    var device = dss.staticDataModel.getDevice(dsid);
    if (device == null) {
        return null;
    }
    if (!device.hasActions) {
        return null;
    }
    dss.staticDataModel.getActions(dsid);
    return device.deviceStandardActions;
};

dss.staticDataModel.getDeviceActions = function (dsid) {
    var device = dss.staticDataModel.getDevice(dsid);
    if (device == null) {
        return null;
    }
    if (!device.hasActions) {
        return null;
    }
    dss.staticDataModel.getActions(dsid);
    return device.deviceActions;
};

dss.staticDataModel.getCustomActions = function (dsid) {
    var device = dss.staticDataModel.getDevice(dsid);
    if (device == null) {
        return null;
    }
    if (!device.hasActions) {
        return null;
    }
    dss.staticDataModel.getActions(dsid);
    return device.deviceCustomActions;
};

dss.staticDataModel.getDeviceStates = function (dsid) {
    var device = dss.staticDataModel.getDevice(dsid);
    if (device == null) {
        return null;
    }
    if (!device.hasActions) {
        return null;
    }
    dss.staticDataModel.getActions(dsid);
    return device.deviceStates;
};

dss.staticDataModel.initZoneSensorCache = function () {
    // json/property/query2?query=/apartment/zones/*(ZoneID)/groups/*(group)/sensor/*(*)

    var resultObj;
    try {
        var sResult = dss.ajaxSyncRequest('/json/property/query2', {
            query: dss.staticDataModel.sQueryStringZoneSensor
        });
        resultObj = Ext.JSON.decode(sResult);
    } catch (e) {
        devLog && devLog(e);
    }
    if (resultObj && resultObj.ok) {
        for (var zoneKey in resultObj.result) {
            var zoneObj = resultObj.result[zoneKey];
            var zoneID = zoneObj.ZoneID;
            for (var groupKey in zoneObj) {
                if (groupKey != 'ZoneID') {
                    var groupObj = zoneObj[groupKey];
                    var groupID = groupObj.group;
                    var sensorObj = [];
                    for (var subZoneKey in groupObj) {
                        if (typeof (groupObj[subZoneKey]) === 'object') {
                            if ([9, 10, 11, 12, 13, 14, 15, 16, 18, 19, 20, 21, 25, 50, 51, 76, 77].indexOf(groupObj[subZoneKey].type) > -1) {
                                sensorObj.push({ type: groupObj[subZoneKey].type, value: groupObj[subZoneKey].value, source: groupObj[subZoneKey].sourcedsuid });
                            }
                        }
                    }
                    if (dss.staticDataModel.nameCache) {
                        for (var iIndex = 0; iIndex < dss.staticDataModel.nameCache.length; iIndex++) {
                            if (dss.staticDataModel.nameCache[iIndex].ZoneID == zoneID && dss.staticDataModel.nameCache[iIndex].groups) {
                                for (var jIndex = 0; jIndex < dss.staticDataModel.nameCache[iIndex].groups.length; jIndex++) {
                                    if (dss.staticDataModel.nameCache[iIndex].groups[jIndex].group == groupID) {
                                        if (dss.staticDataModel.nameCache[iIndex].groups[jIndex] != null) {
                                            dss.staticDataModel.nameCache[iIndex].groups[jIndex].sensors = sensorObj;
                                        }
                                    }
                                }
                            }
                        }
                    }
                }
            }
        }
    }
};

dss.staticDataModel.getNameForSensortype = function (type) {
    switch (type) {
        case 9:
            return _('Temperature indoors');
        case 10:
            return _('Temperature outdoors');
        case 11:
            return _('Brightness indoors');
        case 12:
            return _('Brightness outdoors');
        case 13:
            return _('Relative humidity indoors');
        case 14:
            return _('Relative humidity outdoors');
        case 15:
            return _('Air pressure');
        case 16:
            return _('Wind gust');
        case 18:
            return _('Wind speed');
        case 19:
            return _('Wind direction');
        case 20:
            return _('Precipitation');
        case 21:
            return _('Carbon Dioxide');
        case 25:
            return _('Sound pressure level');
        case 50:
            return _('Room temperature set point');
        case 51:
            return _('Room temperature control variable');
        case 70:
            return _('Generated Energy');
        case 71:
            return _('Water Quantity');
        case 72:
            return _('Water Flow Rate');
        case 73:
            return _('Length');
        case 74:
            return _('Mass');
        case 75:
            return _('Time');
        case 76:
            return _('Sun azimuth');
        case 77:
            return _('Sun elevation');
        default:
            return _('Sensor type') + ' ' + type;
    }
};


dss.staticDataModel.hasDeviceStates = function (dsuid) {
    var states = dss.staticDataModel.getStates();
    if (states) {
        for (var iIndex = 0; iIndex < states.length; iIndex++) {
            var oState = states[iIndex];
            if (oState.name.indexOf(dsuid) != -1) {
                return true;
            }
        }
    }
    return false;
};

dss.staticDataModel.initHLECache = function () {
    dss.staticDataModel.HLECache = [];
    dss.staticDataModel.StateCache = [];

    var resultObj;
    try {
        var sResult = dss.ajaxSyncRequest('/json/property/query', {
            query: dss.staticDataModel.sQueryStringUSR
        });
        resultObj = Ext.JSON.decode(sResult);
    } catch (e) {
        devLog && devLog(e);
    }
    if (resultObj && resultObj.ok && resultObj.result) {
        if (resultObj.result.events != null) {
            for (var iIndex = 0; iIndex < resultObj.result.events.length; iIndex++) {
                var oObject = {
                    name: resultObj.result.events[iIndex].name,
                    id: resultObj.result.events[iIndex].id
                };
                dss.staticDataModel.HLECache.push(oObject);
            }
        }
        if (resultObj.result.states != null) {
            for (var iIndex = 0; iIndex < resultObj.result.states.length; iIndex++) {
                var oObjectState = {
                    name: resultObj.result.states[iIndex].name,
                    value: resultObj.result.states[iIndex].value,
                    valuerange: []
                };
                for (var sKey in resultObj.result.states[iIndex]) {
                    if (sKey != 'name') {
                        if (sKey != 'value') {
                            if (resultObj.result.states[iIndex][sKey]) {
                                for (var jIndex = 0; jIndex < resultObj.result.states[iIndex][sKey].length; jIndex++) {
                                    for (var sSubKey in resultObj.result.states[iIndex][sKey][jIndex]) {
                                        var oValue = {
                                            name: sSubKey,
                                            value: resultObj.result.states[iIndex][sKey][jIndex][sSubKey]
                                        };
                                        oObjectState.valuerange.push(oValue);
                                    }
                                }
                            }
                        }
                    }
                }
                if (oObjectState.valuerange.length === 0) {
                    oObjectState.valuerange.push({
                        name: 'true',
                        value: true
                    });
                    oObjectState.valuerange.push({
                        name: 'false',
                        value: false
                    });
                }
                dss.staticDataModel.StateCache.push(oObjectState);
            }

        }
    }
};

dss.staticDataModel.initClimateControllerSettingsCache = function () {
    dss.staticDataModel.ClimateControllerSettingsCache = [];

    var resultObj;
    try {
        var sResult = dss.ajaxSyncRequest('/json/property/query2', {
            query: dss.staticDataModel.sQueryStringClimateSettings
        });
        resultObj = Ext.JSON.decode(sResult);
    } catch (e) {
        devLog && devLog(e);
    }
    if (resultObj && resultObj.ok && resultObj.result) {
        if (resultObj.result.generalSettings != null) {
            dss.staticDataModel.ClimateControllerSettingsCache = resultObj.result.generalSettings;
        }
    }
};

dss.staticDataModel.getClimateSettings = function () {
    if (dss.staticDataModel.ClimateControllerSettingsCache == null) {
        dss.staticDataModel.initClimateControllerSettingsCache();
    }
    return dss.staticDataModel.ClimateControllerSettingsCache;
};

dss.staticDataModel.getHLE = function () {
    if (dss.staticDataModel.HLECache == null) {
        dss.staticDataModel.initHLECache();
    }
    return dss.staticDataModel.HLECache;
};

dss.staticDataModel.getStates = function () {
    if (dss.staticDataModel.StateCache == null) {
        dss.staticDataModel.initHLECache();
    }

    var oResult = [];
    if (dss.staticDataModel.StateCache) {
        for (var iIndex = 0; iIndex < dss.staticDataModel.StateCache.length; iIndex++) {
            var valuerange = [];
            if (dss.staticDataModel.StateCache[iIndex].valuerange) {
                for (var jIndex = 0; jIndex < dss.staticDataModel.StateCache[iIndex].valuerange.length; jIndex++) {
                    valuerange.push({
                        name: dss.staticDataModel.StateCache[iIndex].valuerange[jIndex].name,
                        value: dss.staticDataModel.StateCache[iIndex].valuerange[jIndex].value
                    });
                }
            }
            oResult.push({
                value: dss.staticDataModel.StateCache[iIndex].value,
                name: dss.staticDataModel.StateCache[iIndex].name,
                valuerange: valuerange
            });
        }
    }
    return oResult;
};

dss.staticDataModel.getSensorDeviceArray = function () {
    if (dss.staticDataModel.nameCache == null) {
        dss.staticDataModel.initCaches();
    }
    return dss.staticDataModel.deviceWithSensorsCache;
};

dss.staticDataModel.getAKMDeviceArray = function () {
    if (dss.staticDataModel.nameCache == null) {
        dss.staticDataModel.initCaches();
    }
    return dss.staticDataModel.deviceAKMCache;
};

dss.staticDataModel.getZoneArray = function () {
    if (dss.staticDataModel.nameCache == null) {
        dss.staticDataModel.initCaches();
    }

    var oResult = [];
    if (dss.staticDataModel.nameCache) {
        for (var iIndex = 0; iIndex < dss.staticDataModel.nameCache.length; iIndex++) {
            if (dss.staticDataModel.nameCache[iIndex].ZoneID === 65534) {
                continue;
            }
            oResult.push({
                ZoneID: dss.staticDataModel.nameCache[iIndex].ZoneID,
                name: dss.staticDataModel.nameCache[iIndex].name
            });
        }
    }
    return oResult;
};

dss.staticDataModel.getDevicesForZone = function (zoneID, groupID) {
    if (dss.staticDataModel.nameCache == null) {
        dss.staticDataModel.initCaches();
    }
    var oRetArray = [];
    if (dss.staticDataModel.deviceCache) {
        for (var i = 0; i < dss.staticDataModel.deviceCache.length; i++) {
            var oDeviceObject = dss.staticDataModel.deviceCache[i];
            var fAdded = false;
            if (oDeviceObject.GroupMembership) {
                for (var j = 0; j < oDeviceObject.GroupMembership.length; j++) {
                    var oDeviceGroup = oDeviceObject.GroupMembership[j];
                    if (oDeviceGroup == 8) {
                        oDeviceGroup = oDeviceObject.buttonActiveGroup;
                    }
                    if ((oDeviceObject.ZoneID == zoneID) || (zoneID == 0)) {
                        if ((oDeviceGroup == groupID) || (groupID == 0)) {
                            if (!fAdded) {
                                oRetArray.push(oDeviceObject);
                                fAdded = true;
                            }
                        }
                    }
                }
            }
        }
    }
    return oRetArray;
};
dss.staticDataModel.getNameForSensorEvent = function (device, eventid) {
    if (dss.staticDataModel.nameCache == null) {
        dss.staticDataModel.initCaches();
    }
    if (device == null) {
        return '';
    }
    if (device.sensorEvents == null) {
        return '';
    }
    for (var iIndex = 0; iIndex < device.sensorEvents.length; iIndex++) {
        if (device.sensorEvents[iIndex].id == eventid) {
            return device.sensorEvents[iIndex].name;
        }
    }
    return '';
};


dss.staticDataModel.getImageForAKMValue = function (type, value) {
    if (dss.staticDataModel.nameCache == null) {
        dss.staticDataModel.initCaches();
    }

    return '<img src="' + dss.staticDataModel.getImagePathForAKMValue(type, value) + '" alt="' + dss.staticDataModel.getNameForAKMValue(type, value) + '">';
};

dss.staticDataModel.getImagePathForAKMValue = function (type, value) {
    // aktuell device == akm

    switch (type) {
        case 7:
            if (value) {
                return dss.staticDataModel.baseImagePath + 'state/fire.png';
            } else {
                return dss.staticDataModel.baseImagePath + 'state/fire_No.png';
            }
        case 8:
            if (value) {
                return dss.staticDataModel.baseImagePath + 'state/wind.png';
            } else {
                return dss.staticDataModel.baseImagePath + 'state/wind_No.png';
            }
        case 9:
            if (value) {
                return dss.staticDataModel.baseImagePath + 'state/rain.png';
            } else {
                return dss.staticDataModel.baseImagePath + 'state/rain_No.png';
            }
        case 13: // window open
            if (value) {
                return dss.staticDataModel.baseImagePath + 'state/window-closed.png';
            } else {
                return dss.staticDataModel.baseImagePath + 'state/window-open.png';
            }
        case 14: // door open
            if (value) {
                return dss.staticDataModel.baseImagePath + 'state/door-closed.png';
            } else {
                return dss.staticDataModel.baseImagePath + 'state/door-open.png';
            }
        case 15: // window tilted
            if (value == dss.whState.WH_Closed) {
                return dss.staticDataModel.baseImagePath + 'state/window-closed.png';
            } else if (value == dss.whState.WH_Open) {
                return dss.staticDataModel.baseImagePath + 'state/window-open.png';
            } else if (value == dss.whState.WH_Tilted) {
                return dss.staticDataModel.baseImagePath + 'state/window-tilted.png';
            }
        case 22: // malfunc
            if (value) {
                return dss.staticDataModel.baseImagePath + 'state/malfunction.png';
            } else {
                return dss.staticDataModel.baseImagePath + 'state/malfunction_no.png';
            }
        case 23: // service
            if (value) {
                return dss.staticDataModel.baseImagePath + 'state/akm_service.png';
            } else {
                return dss.staticDataModel.baseImagePath + 'state/akm_service_no.png';
            }
        default:
            if (value) {
                return dss.staticDataModel.baseImagePath + 'state/sensor.png';
            } else {
                return dss.staticDataModel.baseImagePath + 'state/sensor_No.png';
            }
    }
};

dss.staticDataModel.getNameForAKMValue = function (type, value) {
    // aktuell device == akm
    switch (type) {
        case 22:
            if (value) {
                return _('Malfunction detected');
            } else {
                return _('no Malfunction');
            }
        case 23:
            if (value) {
                return _('Service needed');
            } else {
                return _('no Service needed');
            }
        case 0:
            if (value) {
                return _('Contact closed');
            } else {
                return _('Contact open');
            }
        case 1:
            if (value) {
                return Ext.String.format2(_('{#akm_state_value_name}Presence'), {});
            } else {
                return Ext.String.format2(_('{#akm_state_value_name}No presence'), {});
            }
        case 2:
            if (value) {
                return _('bright');
            } else {
                return _('dark');
            }
        case 3:
            if (value) {
                return _('Present');
            } else {
                return _('Absent');
            }
        case 4:
            if (value) {
                return _('dark');
            } else {
                return _('bright');
            }
        case 5:
            if (value) {
                return _('Motion detected');
            } else {
                return _('no Motion');
            }
        case 6:
            if (value) {
                return _('Motion detected');
            } else {
                return _('no Motion');
            }
        case 7:
            if (value) {
                return _('Smoke detected');
            } else {
                return _('no Smoke');
            }
        case 8:
            if (value) {
                return _('Wind');
            } else {
                return _('no Wind');
            }
        case 9:
            if (value) {
                return _('Rain detected');
            } else {
                return _('no Rain');
            }
        case 10:
            if (value) {
                return _('bright');
            } else {
                return _('dark');
            }
        case 11:
            if (value) {
                return _('temperature low');
            } else {
                return _('temperature high');
            }
        case 13:
            if (value) {
                return _('window closed');
            } else {
                return _('window open');
            }
        case 14:
            if (value) {
                return _('door closed');
            } else {
                return _('door open');
            }
        case 15:
            if (value == dss.whState.WH_Closed) {
                return _('window closed');
            } else if (value == dss.whState.WH_Open) {
                return _('window open');
            } else if (value == dss.whState.WH_Tilted) {
                return _('window tilted');
            }
            break;
        case 255:
            if (value) {
                return _('Contact closed');
            } else {
                return _('Contact open');
            }
        default:
            if (value) {
                return _('Contact closed');
            } else {
                return _('Contact open');
            }
    }
};

dss.staticDataModel.getNameForAKMType = function (type) {
    // aktuell device == akm
    switch (type) {
        case 0:
            return _('Generic (App)');
        case 1:
            return _('Presence');
        case 2:
            return _('Brightness (Room)');
        case 3:
            return _('Presence in darkness');
        case 4:
            return _('Twilight (outdoors)');
        case 5:
            return _('Movement');
        case 6:
            return _('Movement in darkness');
        case 7:
            return _('Smoke detector');
        case 8:
            return _('Wind monitor');
        case 9:
            return _('Rain monitor');
        case 10:
            return _('Sunlight');
        case 11:
            return _('Room thermostat');
        case 13:
            return _('window contact');
        case 14:
            return _('door contact');
        case 15:
            return _('window  handle');
        case 255:
            return _('Reserved');
        default:
            return _('Generic (App)');
    }
};

dss.staticDataModel.getNameForGroup = function (zoneID, group) {
    if (dss.staticDataModel.nameCache == null) {
        dss.staticDataModel.initCaches();
    }
    var defaultName = Ext.String.format2(_('Group {group}') + ' ', {
        group: group
    });
    var baseName = '';
    var baseName2 = 'XXXXXXXXXXX';

    switch (group) {
        case 0:
            defaultName = 'Alle Gruppen';
            baseName = 'broadcast';
            baseName2 = 'broadcast';
            break;
        case 1:
            defaultName = _('Light');
            baseName = 'yellow';
            baseName2 = 'yellow';
            break;
        case 2:
            defaultName = _('Shadow');
            baseName = 'gray';
            baseName2 = 'gray';
            break;
        case 3:
            defaultName = _('Climate');
            baseName = 'blue';
            baseName2 = 'heating';
            break;
        case 4:
            defaultName = _('Audio');
            baseName = 'cyan';
            baseName2 = 'cyan';
            break;
        case 5:
            defaultName = _('Video');
            baseName = 'magenta';
            baseName2 = 'magenta';
            break;
        case 6:
            defaultName = _('Security');
            baseName = 'red';
            baseName2 = 'reserved1';
            break;
        case 7:
            defaultName = _('Access');
            baseName = 'green';
            baseName2 = 'reserved2';
            break;
        case 8:
            defaultName = _('Joker');
            baseName = 'black';
            baseName2 = 'black';
            break;
        case 9:
            defaultName = _('cooling');
            baseName = 'white';
            baseName2 = 'cooling';
            break;
        case 10:
            defaultName = _('ventilation');
            baseName = 'ventilation';
            baseName2 = 'ventilation';
            break;
        case 11:
            defaultName = _('window');
            baseName = 'display';
            baseName2 = 'window';
            break;
        case 12:
            defaultName = _('recirculation');
            baseName = 'recirculation';
            baseName2 = 'recirculation';
            break;
        case 48:
            defaultName = _('controltemperature');
            baseName = 'controltemperature';
            baseName2 = 'controltemperature';
            break;
        case 64:
            defaultName = _('Apartment ventilation');
            baseName = 'Apartment ventilation';
            baseName2 = 'Apartment ventilation';
            break;
    }

    if (dss.staticDataModel.nameCache) {
        for (var iZoneIndex = 0; iZoneIndex < dss.staticDataModel.nameCache.length; iZoneIndex++) {
            if (dss.staticDataModel.nameCache[iZoneIndex].ZoneID == zoneID) {
                if (dss.staticDataModel.nameCache[iZoneIndex].groups) {
                    for (var iGroupIndex = 0; iGroupIndex < dss.staticDataModel.nameCache[iZoneIndex].groups.length; iGroupIndex++) {
                        if (dss.staticDataModel.nameCache[iZoneIndex].groups[iGroupIndex].group == group) {
                            if (
                                dss.staticDataModel.nameCache[iZoneIndex].groups[iGroupIndex].name &&
                                dss.staticDataModel.nameCache[iZoneIndex].groups[iGroupIndex].name != baseName
                            ) {
                                if (
                                    dss.staticDataModel.nameCache[iZoneIndex].groups[iGroupIndex].name &&
                                    dss.staticDataModel.nameCache[iZoneIndex].groups[iGroupIndex].name != baseName2
                                ) {
                                    return dss.staticDataModel.nameCache[iZoneIndex].groups[iGroupIndex].name;
                                }
                            }
                        }
                    }
                }
            }
        }
    }
    return defaultName;
};

dss.staticDataModel.getDefaultNameForScene = function (zoneID, group, scene) {
    if (dss.staticDataModel.nameCache == null) {
        dss.staticDataModel.initCaches();
    }
    if (zoneID == 0) {
        if (dss.staticDataModel.isClusterGroup(group)) {
            group = dss.staticDataModel.getUserGroupColor(group);
        }
    }

    var defaultName = _('Scene') + ' ' + scene;
    switch (scene) {
        case 67:
            defaultName = _('Standby');
            break;
        case 64:
            defaultName = _('Auto Standby');
            break;
        case 40:
            defaultName = _('Auto Off');
            break;
        case 41:
            defaultName = _('Impulse');
            break;
        case 68:
            defaultName = _('Deep Off');
            break;
        case 69:
            defaultName = _('Sleeping');
            break;
        case 70:
            defaultName = _('Wake up');
            break;
        case 72:
            defaultName = _('Absent');
            break;
        case 71:
            defaultName = _('Present');
            break;
        case 73:
            defaultName = _('Bell');
            break;

        case 74:
            defaultName = _('Alarm') + ' 1';
            break;
        case 83:
            defaultName = _('Alarm') + ' 2';
            break;
        case 84:
            defaultName = _('Alarm') + ' 3';
            break;
        case 85:
            defaultName = _('Alarm') + ' 4';
            break;
        case 86:
            defaultName = _('Wind');
            break;
        case 87:
            defaultName = _('no Wind');
            break;
        case 88:
            defaultName = _('rain');
            break;
        case 89:
            defaultName = _('no Rain');
            break;
        case 90:
            defaultName = _('Hail');
            break;
        case 91:
            defaultName = _('hail inactive');
            break;
        case 76:
            defaultName = _('fire');
            break;

        case 65:
            defaultName = _('Panic');
            break;
        case 0:
            if (group == 2) {
                defaultName = _('Close');
            } else {
                defaultName = _('Off');
            }
            break;
        case 5:
            if (group == 2) {
                defaultName = _('Open');
            } else if (group == 48) {
                defaultName = _('HOLIDAY Mode');
            } else if (group == 64 || group == 10 || group == 12) {
                defaultName = _('Level 1');
            } else {
                defaultName = _('scene 1');
            }
            break;
        case 17:
            if (group == 64 || group == 10 || group == 12) {
                defaultName = _('Level 2');
            } else {
                defaultName = _('scene 2');
            }
            break;
        case 15:
            defaultName = _('Stop');
            break;
        case 18:
            if (group == 64 || group == 10 || group == 12) {
                defaultName = _('Level 3');
            } else {
                defaultName = _('scene 3');
            }
            break;
        case 19:
            if (group == 64 || group == 10 || group == 12) {
                defaultName = _('Level 4');
            } else {
                defaultName = _('scene 4');
            }
            break;
        case 1:
            if (group == 2) {
                defaultName = _('area 1 close');
            } else if (group == 48) {
                defaultName = _('COMFORT Mode');
            } else {
                defaultName = _('area 1 off');
            }
            break;
        case 6:
            if (group == 64 || group == 10 || group == 12) {
                defaultName = _('Boost');
            } else if (group == 2) {
                defaultName = _('area 1 open');
            } else if (group == 48) {
                defaultName = _('COOLING Mode');
            } else {
                defaultName = _('area 1 on');
            }
            break;
        case 2:
            if (group == 2) {
                defaultName = _('area 2 close');
            } else if (group == 48) {
                defaultName = _('ECO Mode');
            } else {
                defaultName = _('area 2 off');
            }
            break;
        case 7:
            if (group == 2) {
                defaultName = _('area 2 open');
            } else if (group == 48) {
                defaultName = _('COOLING OFF Mode');
            } else {
                defaultName = _('area 2 on');
            }
            break;
        case 3:
            if (group == 2) {
                defaultName = _('area 3 close');
            } else if (group == 48) {
                defaultName = _('NotUsed Mode');
            } else {
                defaultName = _('area 3 off');
            }
            break;
        case 8:
            if (group == 2) {
                defaultName = _('area 3 open');
            } else {
                defaultName = _('area 3 on');
            }
            break;
        case 4:
            if (group == 2) {
                defaultName = _('area 4 close');
            } else if (group == 48) {
                defaultName = _('NIGHT Mode');
            } else {
                defaultName = _('area 4 off');
            }
            break;
        case 9:
            if (group == 2) {
                defaultName = _('area 4 open');
            } else {
                defaultName = _('area 4 on');
            }
            break;
        case 11:
            defaultName = _('Dim down');
            break;
        case 12:
            defaultName = _('Dim up');
            break;
        case 32:
            defaultName = _('scene 10');
            break;
        case 33:
            defaultName = _('scene 11');
            break;
        case 34:
            defaultName = _('scene 20');
            break;
        case 35:
            defaultName = _('scene 21');
            break;
        case 36:
            defaultName = _('scene 30');
            break;
        case 37:
            defaultName = _('scene 31');
            break;
        case 38:
            defaultName = _('scene 40');
            break;
        case 39:
            defaultName = _('scene 41');
            break;
        case 20:
            defaultName = _('scene 12');
            break;
        case 21:
            defaultName = _('scene 13');
            break;
        case 22:
            defaultName = _('scene 14');
            break;
        case 23:
            defaultName = _('scene 22');
            break;
        case 24:
            defaultName = _('scene 23');
            break;
        case 25:
            defaultName = _('scene 24');
            break;
        case 26:
            defaultName = _('scene 32');
            break;
        case 27:
            defaultName = _('scene 33');
            break;
        case 28:
            defaultName = _('scene 34');
            break;
        case 29:
            defaultName = _('scene 42');
            break;
        case 30:
            defaultName = _('scene 43');
            break;
        case 31:
            defaultName = _('scene 44');
            break;
        case 50:
            defaultName = _('local off');
            break;
        case 51:
            defaultName = _('local on');
            break;
        case 56:
            defaultName = _('Sun protection');
            break;
    }
    return defaultName;
};

dss.staticDataModel.getNameForScene = function (zoneID, group, scene) {
    if (dss.staticDataModel.nameCache == null) {
        dss.staticDataModel.initCaches();
    }
    var baseName = '';

    switch (scene) {
        case 67:
            baseName = 'STANDBY';
            break;
        case 64:
            baseName = 'AUTO_STANDBY';
            break;
        case 40:
            baseName = 'AUTO_OFF';
            break;
        case 68:
            baseName = 'DEEP_OFF';
            break;
        case 69:
            baseName = 'SLEEPING';
            break;
        case 70:
            baseName = 'WAKE_UP';
            break;
        case 72:
            baseName = 'ABSENT';
            break;
        case 71:
            baseName = 'PRESENT';
            break;
        case 73:
            baseName = 'SIG_BELL';
            break;
        case 65:
            baseName = 'PANIC';
            break;
        case 0:
            baseName = 'T0_S0';
            break;
        case 5:
            baseName = 'T0_S1';
            break;
        case 17:
            baseName = 'T0_S2';
            break;
        case 18:
            baseName = 'T0_S3';
            break;
        case 19:
            baseName = 'T0_S4';
            break;
        case 1:
            baseName = 'T1_S0';
            break;
        case 6:
            baseName = 'T1_S1';
            break;
        case 2:
            baseName = 'T2_S0';
            break;
        case 7:
            baseName = 'T2_S1';
            break;
        case 3:
            baseName = 'T3_S0';
            break;
        case 8:
            baseName = 'T3_S1';
            break;
        case 4:
            baseName = 'T4_S0';
            break;
        case 9:
            baseName = 'T4_S1';
            break;
        case 32:
            baseName = 'T1E_S0';
            break;
        case 33:
            baseName = 'T1E_S1';
            break;
        case 34:
            baseName = 'T2E_S0';
            break;
        case 35:
            baseName = 'T2E_S1';
            break;
        case 36:
            baseName = 'T3E_S0';
            break;
        case 37:
            baseName = 'T3E_S1';
            break;
        case 38:
            baseName = 'T4E_S0';
            break;
        case 39:
            baseName = 'T4E_S1';
            break;
        case 20:
            baseName = 'T1_S2';
            break;
        case 21:
            baseName = 'T1_S3';
            break;
        case 22:
            baseName = 'T1_S4';
            break;
        case 23:
            baseName = 'T2_S2';
            break;
        case 24:
            baseName = 'T2_S3';
            break;
        case 25:
            baseName = 'T2_S4';
            break;
        case 26:
            baseName = 'T3_S2';
            break;
        case 27:
            baseName = 'T3_S3';
            break;
        case 28:
            baseName = 'T3_S4';
            break;
        case 29:
            baseName = 'T4_S2';
            break;
        case 30:
            baseName = 'T4_S3';
            break;
        case 31:
            baseName = 'T4_S4';
            break;
    }

    if (dss.staticDataModel.nameCache) {
        for (var iZoneIndex = 0; iZoneIndex < dss.staticDataModel.nameCache.length; iZoneIndex++) {
            if (dss.staticDataModel.nameCache[iZoneIndex].ZoneID == zoneID) {
                if (dss.staticDataModel.nameCache[iZoneIndex].groups) {
                    for (var iGroupIndex = 0; iGroupIndex < dss.staticDataModel.nameCache[iZoneIndex].groups.length; iGroupIndex++) {
                        if (
                            dss.staticDataModel.nameCache[iZoneIndex].groups[iGroupIndex].group == group &&
                            dss.staticDataModel.nameCache[iZoneIndex].groups[iGroupIndex].scenes !== undefined
                        ) {
                            if (dss.staticDataModel.nameCache[iZoneIndex].groups[iGroupIndex].scenes.length !== undefined) {
                                for (
                                    var iSceneIndex = 0;
                                    iSceneIndex < dss.staticDataModel.nameCache[iZoneIndex].groups[iGroupIndex].scenes.length;
                                    iSceneIndex++
                                ) {
                                    if (dss.staticDataModel.nameCache[iZoneIndex].groups[iGroupIndex].scenes[iSceneIndex].scene == scene) {
                                        if (dss.staticDataModel.nameCache[iZoneIndex].groups[iGroupIndex].scenes[iSceneIndex].name != null) {
                                            if (dss.staticDataModel.nameCache[iZoneIndex].groups[iGroupIndex].scenes[iSceneIndex].name != '') {
                                                if (dss.staticDataModel.nameCache[iZoneIndex].groups[iGroupIndex].scenes[iSceneIndex].name != baseName) {
                                                    return dss.staticDataModel.nameCache[iZoneIndex].groups[iGroupIndex].scenes[iSceneIndex].name;
                                                }
                                            }
                                        }
                                    }
                                }
                            }
                        }
                    }
                }
            }
        }
    }
    return dss.staticDataModel.getDefaultNameForScene(zoneID, group, scene);
};

dss.staticDataModel.getNameForZone = function (zoneID) {
    if (dss.staticDataModel.nameCache == null) {
        dss.staticDataModel.initCaches();
    }
    var baseName = Ext.String.format(_('Room ID {zone}'), {
        zone: zoneID
    });
    if (zoneID == 0) {
        baseName = _('whole installation');
    }
    if (zoneID == -1) {
        baseName = _('any room');
    }
    if (dss.staticDataModel.nameCache) {
        for (var iZoneIndex = 0; iZoneIndex < dss.staticDataModel.nameCache.length; iZoneIndex++) {
            if (dss.staticDataModel.nameCache[iZoneIndex].ZoneID == zoneID) {
                if (dss.staticDataModel.nameCache[iZoneIndex].name != '') {
                    return dss.staticDataModel.nameCache[iZoneIndex].name;
                }
            }
        }
    }
    return baseName;
};

dss.staticDataModel.hasConnectedDevicesForGroup = function (zoneID, group) {
    if (dss.staticDataModel.nameCache == null) {
        dss.staticDataModel.initCaches();
    }
    if (dss.staticDataModel.nameCache) {
        for (var iZoneIndex = 0; iZoneIndex < dss.staticDataModel.nameCache.length; iZoneIndex++) {
            if (dss.staticDataModel.nameCache[iZoneIndex].ZoneID == zoneID) {
                if (dss.staticDataModel.nameCache[iZoneIndex].groups) {
                    for (var iGroupIndex = 0; iGroupIndex < dss.staticDataModel.nameCache[iZoneIndex].groups.length; iGroupIndex++) {
                        if (dss.staticDataModel.nameCache[iZoneIndex].groups[iGroupIndex].group == group) {
                            return (dss.staticDataModel.nameCache[iZoneIndex].groups[iGroupIndex].connectedDevices != 0);
                        }
                    }
                }
            }
        }
    }
    return false;
};

dss.staticDataModel.getGroupForZone = function (zoneID) {
    if (dss.staticDataModel.nameCache == null) {
        dss.staticDataModel.initCaches();
    }
    if (dss.staticDataModel.nameCache) {
        for (var iZoneIndex = 0; iZoneIndex < dss.staticDataModel.nameCache.length; iZoneIndex++) {
            if (dss.staticDataModel.nameCache[iZoneIndex].ZoneID == zoneID) {
                var oResult = [];
                if (dss.staticDataModel.nameCache[iZoneIndex].groups) {
                    for (var iIndex = 0; iIndex < dss.staticDataModel.nameCache[iZoneIndex].groups.length; iIndex++) {
                        if (dss.staticDataModel.nameCache[iZoneIndex].groups[iIndex].group != 6) {
                            if (dss.staticDataModel.nameCache[iZoneIndex].groups[iIndex].group != 7) {
                                if (
                                    (dss.staticDataModel.nameCache[iZoneIndex].groups[iIndex].group < 16) ||
                                    ((zoneID == 0) && dss.staticDataModel.isClusterGroup(dss.staticDataModel.nameCache[iZoneIndex].groups[iIndex].group)) ||
                                    ((zoneID == 0) && dss.staticDataModel.nameCache[iZoneIndex].groups[iIndex].group > 63)
                                ) {
                                    oResult.push({
                                        group: dss.staticDataModel.nameCache[iZoneIndex].groups[iIndex].group,
                                        name: dss.staticDataModel.nameCache[iZoneIndex].groups[iIndex].name,
                                        icon: dss.staticDataModel.nameCache[iZoneIndex].groups[iIndex].icon,
                                        reachable: dss.staticDataModel.nameCache[iZoneIndex].groups[iIndex].reachable,
                                        connectedDevices: dss.staticDataModel.nameCache[iZoneIndex].groups[iIndex].connectedDevices
                                    });
                                }
                            }
                        }
                    }
                }
                return oResult;
            }
        }
    }
    return null;
};

dss.staticDataModel.getDevice = function (dSUID) {
    if (dss.staticDataModel.nameCache == null) {
        dss.staticDataModel.initCaches();
    }
    if (DSS.types.dsuid.isDSUID(dSUID)) {
        if (dss.staticDataModel.deviceCache) {
            for (var i = 0; i < dss.staticDataModel.deviceCache.length; i++) {
                if (dSUID == dss.staticDataModel.deviceCache[i].dSUID) {
                    return dss.staticDataModel.deviceCache[i];
                }
            }
        }
        return {
            dSID: DSS.types.dsuid.dsuid_to_dsid(dSUID),
            dSUID: dSUID,
            name: '',
            functionID: 0x1000,
            specialActions: {}
        };
    } else {
        if (dss.staticDataModel.deviceCache) {
            for (var i = 0; i < dss.staticDataModel.deviceCache.length; i++) {
                if (dSUID == dss.staticDataModel.deviceCache[i].dSID) {
                    return dss.staticDataModel.deviceCache[i];
                }
            }
        }
        return {
            dSID: dSUID,
            dSUID: DSS.types.dsuid.dsuid_from_dsid(dSUID),
            name: '',
            functionID: 0x1000
        };
    }
};

dss.staticDataModel.deviceHasSpecialActions = function (oDeviceObject) {
    return oDeviceObject.specialActions && oDeviceObject.specialActions.actions && oDeviceObject.specialActions.actions.deviceActions != null;
};
dss.staticDataModel.deviceCanDoImpulse = function (oDeviceObject) {
    if (oDeviceObject.productID != 12488) {
        return false;
    }
    if (oDeviceObject.functionID == 33052 || oDeviceObject.functionID == 33044) {
        return true;
    }
    if (oDeviceObject.functionID != 33232 && oDeviceObject.functionID != 33240) {
        return false;
    }

    const noActions = !oDeviceObject.hasActions && !dss.staticDataModel.deviceHasSpecialActions(oDeviceObject);
    const outputModeOk = oDeviceObject.outputMode != 34 && oDeviceObject.outputMode != 38;
    const groupMembershipOk = oDeviceObject.GroupID != DSS.Groups.LIGHT && oDeviceObject.buttonActiveGroup == DSS.Groups.JOKER &&
                oDeviceObject.GroupID == DSS.Groups.JOKER &&
                oDeviceObject.GroupMembership[0] == DSS.Groups.JOKER;
    return noActions && outputModeOk && groupMembershipOk;
};
dss.staticDataModel.deviceHasTwoStages = function (oDeviceObject) {
    if (oDeviceObject.hasActions || dss.staticDataModel.deviceHasSpecialActions(oDeviceObject)) {
        return false;
    }
    return oDeviceObject.outputMode == 34;
};
dss.staticDataModel.deviceHasThreeStages = function (oDeviceObject) {
    if (oDeviceObject.hasActions || dss.staticDataModel.deviceHasSpecialActions(oDeviceObject)) {
        return false;
    }
    return oDeviceObject.outputMode == 38;
};

const groupHasActiveIcon = [ 1, 2, 3, 4, 5, 6, 7, 64 ];
const groupHasInactiveIcon = [ 1, 2, 3, 4, 5, 6, 7, 64 ];
const groupHasAutoOffIcon = [ 1 ];

dss.staticDataModel.getImagePathForScene = function (zoneID, groupID, sceneID, fUndoIcon) {
    if (dss.staticDataModel.nameCache == null) {
        dss.staticDataModel.initCaches();
    }
    if (fUndoIcon == null) {
        fUndoIcon = false;
    }
    var fUndoPrefix = '';
    if (fUndoIcon) {
        fUndoPrefix = 'undo_';
    }
    var baseGroupName = dss.staticDataModel.baseImagePath + 'groupIcons/' + fUndoPrefix + 'group_1';
    if (zoneID == 0) {
        if (dss.staticDataModel.isClusterGroup(groupID)) {
            groupID = dss.staticDataModel.getUserGroupColor(groupID);
        }
    }
    if (groupID == 48) {
        groupID = 3;
        if (sceneID == 1) {
            sceneID = 5;
        }
        if (sceneID == 2) {
            sceneID = 5;
        }
        if (sceneID == 3) {
            sceneID = 0;
        }
        if (sceneID == 4) {
            sceneID = 0;
        }
        if (sceneID == 5) {
            sceneID = 0;
        }
    }

    switch (groupID) {
        case 1:
        case 3:
        case 4:
        case 5:
        case 6:
        case 7:
            baseGroupName = dss.staticDataModel.baseImagePath + 'groupIcons/' + fUndoPrefix + 'group_' + groupID;
            break;
        case 2:
            baseGroupName = dss.staticDataModel.baseImagePath + 'groupIcons/' + fUndoPrefix + 'group_2';
            if (sceneID == 5) {
                return baseGroupName + '_scene_5.png';
            }
            if (sceneID == 6) {
                return baseGroupName + '_scene_5.png';
            }
            if (sceneID == 7) {
                return baseGroupName + '_scene_5.png';
            }
            if (sceneID == 8) {
                return baseGroupName + '_scene_5.png';
            }
            if (sceneID == 9) {
                return baseGroupName + '_scene_5.png';
            }
            break;
        case 10:
        case 12:
        case 64:
            baseGroupName = dss.staticDataModel.baseImagePath + 'groupIcons/' + fUndoPrefix + 'group_64';
            if (sceneID == 6) {
                return groupHasActiveIcon.includes(64)
                    ? baseGroupName + '_active.png'
                    : null;
            }
            break;
    }

    switch (sceneID) {
        case 0:
        case 1:
        case 2:
        case 3:
        case 4:
        case 32:
        case 34:
        case 36:
        case 38:
        case 11:
        case 13:
        case 50:
            return groupHasInactiveIcon.includes(groupID)
                ? baseGroupName + '_deact.png'
                : null;
        case 40:
            return groupHasAutoOffIcon.includes(groupID)
                ? baseGroupName + '_auto_off.png'
                : null;
        case 5:
        case 6:
        case 7:
        case 8:
        case 9:
        case 14:
        case 17:
        case 18:
        case 19:
        case 20:
        case 21:
        case 22:
        case 23:
        case 24:
        case 25:
        case 26:
        case 27:
        case 28:
        case 29:
        case 30:
        case 31:
        case 33:
        case 35:
        case 37:
        case 39:
        case 12:
        case 51:
            return groupHasActiveIcon.includes(groupID)
                ? baseGroupName + '_active.png'
                : null;
        case 15:
            return dss.staticDataModel.baseImagePath + 'sceneIcons/' + fUndoPrefix + 'scene_stop.png';
        case 56:
            return dss.staticDataModel.baseImagePath + 'sceneIcons/' + fUndoPrefix + 'scene_sun_protection.png';
        case 64:
            return dss.staticDataModel.baseImagePath + 'sceneIcons/' + fUndoPrefix + 'scene_standby.png';
        case 65:
            return dss.staticDataModel.baseImagePath + 'sceneIcons/' + fUndoPrefix + 'scene_panic.png';
        case 67:
            return dss.staticDataModel.baseImagePath + 'sceneIcons/' + fUndoPrefix + 'scene_standby.png';
        case 68:
            return dss.staticDataModel.baseImagePath + 'sceneIcons/' + fUndoPrefix + 'scene_deep_off.png';
        case 69:
            return dss.staticDataModel.baseImagePath + 'sceneIcons/' + fUndoPrefix + 'scene_sleeping.png';
        case 70:
            return dss.staticDataModel.baseImagePath + 'sceneIcons/' + fUndoPrefix + 'scene_wakeup.png';
        case 71:
            return dss.staticDataModel.baseImagePath + 'sceneIcons/' + fUndoPrefix + 'scene_present.png';
        case 72:
            return dss.staticDataModel.baseImagePath + 'sceneIcons/' + fUndoPrefix + 'scene_absent.png';
        case 73:
            return dss.staticDataModel.baseImagePath + 'sceneIcons/' + fUndoPrefix + 'scene_bell.png';
        case 74:
            return dss.staticDataModel.baseImagePath + 'sceneIcons/' + fUndoPrefix + 'scene_alarm1.png';
        case 83:
            return dss.staticDataModel.baseImagePath + 'sceneIcons/' + fUndoPrefix + 'scene_alarm2.png';
        case 84:
            return dss.staticDataModel.baseImagePath + 'sceneIcons/' + fUndoPrefix + 'scene_alarm3.png';
        case 85:
            return dss.staticDataModel.baseImagePath + 'sceneIcons/' + fUndoPrefix + 'scene_alarm4.png';
        case 88:
            return dss.staticDataModel.baseImagePath + 'sceneIcons/' + fUndoPrefix + 'scene_rain.png';
        case 89:
            return dss.staticDataModel.baseImagePath + 'sceneIcons/' + fUndoPrefix + 'scene_norain.png';
        case 90:
            return dss.staticDataModel.baseImagePath + 'state/hail.png';
        case 91:
            return dss.staticDataModel.baseImagePath + 'state/hail_No.png';
        case 76:
            return dss.staticDataModel.baseImagePath + 'sceneIcons/' + fUndoPrefix + 'scene_fire.png';
    }
};

dss.staticDataModel.getScene = function (zoneId, groupId, sceneID) {
    var oSceneArray = dss.staticDataModel.getScenes(zoneId, groupId);
    if (oSceneArray) {
        for (var iIndex = 0; iIndex < oSceneArray.length; iIndex++) {
            if (oSceneArray[iIndex].id == sceneID) {
                return oSceneArray[iIndex];
            }
        }
    }
    return null;
};

dss.staticDataModel.getScenes = function (zoneId, groupId) {
    if (dss.staticDataModel.nameCache == null) {
        dss.staticDataModel.initCaches();
    }


    var oDeviceArray = dss.staticDataModel.getDevicesForZone(zoneId, groupId);

    var oRetArray = [];
    var sceneArray = [
        0, 5, 17, 18, 19, 1, 6, 2, 7, 3, 8, 4, 9, 32, 33, 20, 21, 22, 34, 35, 23, 24, 25, 36, 37, 26, 27, 28, 38, 39, 29, 30, 31, 11, 12, 67,
        64, 68, 69, 70, 72, 71, 73, 65, 76, 74, 83, 84, 85, 88, 89
    ];
    if (groupId == 1) {
        sceneArray = [
            0, 40, 5, 17, 18, 19, 1, 6, 2, 7, 3, 8, 4, 9, 32, 33, 20, 21, 22, 34, 35, 23, 24, 25, 36, 37, 26, 27, 28, 38, 39, 29, 30, 31, 11,
            12, 67, 64, 68, 69, 70, 72, 71, 73, 65, 76, 74, 83, 84, 85, 88, 89
        ];
    }
    if (zoneId == 0) {
        if (groupId == 0) {
            sceneArray.push(90);
            sceneArray.push(91);
        }
    }
    if ((groupId == 2) || (dss.staticDataModel.getUserGroupColor(groupId) == 2)) {
        sceneArray.splice(5, 0, 56);
        sceneArray.splice(34, 0, 15);
    }

    if (((zoneId == 0) && (groupId > 63)) || (groupId == 10) || (groupId == 12)) {
        if (dss.staticDataModel.nameCache) {
            for (var iIndex = 0; iIndex < dss.staticDataModel.nameCache.length; iIndex++) {
                if (dss.staticDataModel.nameCache[iIndex].ZoneID == zoneId && dss.staticDataModel.nameCache[iIndex].groups) {
                    for (var jIndex = 0; jIndex < dss.staticDataModel.nameCache[iIndex].groups.length; jIndex++) {
                        if (dss.staticDataModel.nameCache[iIndex].groups[jIndex].group == groupId) {
                            if (dss.staticDataModel.nameCache[iIndex].groups[jIndex].activeBasicScenes != null) {
                                sceneArray = JSON.parse('[' + dss.staticDataModel.nameCache[iIndex].groups[jIndex].activeBasicScenes + ']');
                            }
                        }
                    }
                }
            }
        }
        var activeBasicScenes = sceneArray.slice();
    }

    for (var iIndex = 0; iIndex < sceneArray.length; iIndex++) {
        var oSceneObject = {
            id: sceneArray[iIndex],
            groupID: groupId,
            zoneID: zoneId,
            reachable: false,
            undoable: false
        };
        oSceneObject.name = dss.staticDataModel.getNameForScene(oSceneObject.zoneID, oSceneObject.groupID, oSceneObject.id);
        oSceneObject.defaultName = dss.staticDataModel.getDefaultNameForScene(oSceneObject.zoneID, oSceneObject.groupID, oSceneObject.id);
        oSceneObject.icon = dss.staticDataModel.getImagePathForScene(oSceneObject.zoneID, oSceneObject.groupID, oSceneObject.id);
        oRetArray.push(oSceneObject);
    }


    function searchAndEnableScene (id) {
        for (var iIndex = 0; iIndex < oRetArray.length; iIndex++) {
            if (oRetArray[iIndex].id == id) {
                oRetArray[iIndex].reachable = true;
            }
        }
    }

    function searchAndSetUndo (id, undo) {
        for (var iIndex = 0; iIndex < oRetArray.length; iIndex++) {
            if (oRetArray[iIndex].id == id) {
                oRetArray[iIndex].undoable = undo;
            }
        }
    }

    if ([0, 6, 7, 10, 12].indexOf(groupId) < 0) {
        if (oDeviceArray) {
            for (var iDeviceIndex = 0; iDeviceIndex < oDeviceArray.length; iDeviceIndex++) {
                var oDev = oDeviceArray[iDeviceIndex];

                if (oDev.productID == 12488) {
                    if ((oDev.functionID == 33052) || (oDev.functionID == 33044)) {
                        if (groupId != 8) {
                            searchAndEnableScene(0);
                            searchAndEnableScene(5);
                            searchAndEnableScene(17);
                            searchAndEnableScene(18);
                            searchAndEnableScene(19);
                        }
                    }
                }

                if (oDev.tasterMode > 0) {
                    if ((groupId == 2) || (dss.staticDataModel.getUserGroupColor(groupId) == 2)) {
                        searchAndEnableScene(56);
                        searchAndEnableScene(15);
                    }
                    if (groupId == 1) {
                        searchAndEnableScene(40);
                    }
                }

                switch (oDev.tasterMode) {
                    case 0:
                        searchAndEnableScene(17);
                        searchAndEnableScene(18);
                        searchAndEnableScene(19);
                        searchAndEnableScene(5);
                        searchAndEnableScene(0);
                        break;
                    case 1:
                        searchAndEnableScene(1);
                        searchAndEnableScene(6);
                        searchAndEnableScene(17);
                        searchAndEnableScene(18);
                        searchAndEnableScene(19);
                        break;
                    case 2:
                        searchAndEnableScene(2);
                        searchAndEnableScene(7);
                        searchAndEnableScene(17);
                        searchAndEnableScene(18);
                        searchAndEnableScene(19);
                        break;
                    case 3:
                        searchAndEnableScene(3);
                        searchAndEnableScene(8);
                        searchAndEnableScene(17);
                        searchAndEnableScene(18);
                        searchAndEnableScene(19);
                        break;
                    case 4:
                        searchAndEnableScene(4);
                        searchAndEnableScene(9);
                        searchAndEnableScene(17);
                        searchAndEnableScene(18);
                        searchAndEnableScene(19);
                        break;
                    case 5:
                        searchAndEnableScene(0);
                        searchAndEnableScene(5);
                        searchAndEnableScene(17);
                        searchAndEnableScene(18);
                        searchAndEnableScene(19);
                        break;
                    case 6:
                        searchAndEnableScene(32);
                        searchAndEnableScene(33);
                        searchAndEnableScene(20);
                        searchAndEnableScene(21);
                        searchAndEnableScene(22);
                        break;
                    case 7:
                        searchAndEnableScene(34);
                        searchAndEnableScene(35);
                        searchAndEnableScene(23);
                        searchAndEnableScene(24);
                        searchAndEnableScene(25);
                        break;
                    case 8:
                        searchAndEnableScene(36);
                        searchAndEnableScene(37);
                        searchAndEnableScene(26);
                        searchAndEnableScene(27);
                        searchAndEnableScene(28);
                        break;
                    case 9:
                        searchAndEnableScene(38);
                        searchAndEnableScene(39);
                        searchAndEnableScene(29);
                        searchAndEnableScene(30);
                        searchAndEnableScene(31);
                        break;
                    case 10:
                        searchAndEnableScene(1);
                        searchAndEnableScene(6);
                        searchAndEnableScene(20);
                        searchAndEnableScene(21);
                        searchAndEnableScene(22);
                        break;
                    case 11:
                        searchAndEnableScene(2);
                        searchAndEnableScene(7);
                        searchAndEnableScene(23);
                        searchAndEnableScene(24);
                        searchAndEnableScene(25);
                        break;
                    case 12:
                        searchAndEnableScene(3);
                        searchAndEnableScene(8);
                        searchAndEnableScene(26);
                        searchAndEnableScene(27);
                        searchAndEnableScene(28);
                        break;
                    case 13:
                        searchAndEnableScene(4);
                        searchAndEnableScene(9);
                        searchAndEnableScene(29);
                        searchAndEnableScene(30);
                        searchAndEnableScene(31);
                        break;
                }
            }
        }
    }
    if ((zoneId != 0) && (groupId == 0)) {
        searchAndEnableScene(67);
        searchAndEnableScene(68);
        searchAndEnableScene(69);
        searchAndEnableScene(70);
    }


    if ((zoneId == 0) && (groupId == 0)) {
        if (dss.staticDataModel.deviceCache) {
            for (var i = 0; i < dss.staticDataModel.deviceCache.length; i++) {
                var oDeviceObject = dss.staticDataModel.deviceCache[i];
                if (oDeviceObject.functionID == 0x7010) {
                    searchAndEnableScene(73);
                } // klingel
                if (oDeviceObject.functionID == 0x6001) {
                    searchAndEnableScene(65); // panik
                }
                if (oDeviceObject.functionID == 0x7050) {
                    searchAndEnableScene(71); // kommen
                    searchAndEnableScene(72); // gehen
                }
            }
        }
        searchAndEnableScene(67);
        searchAndEnableScene(68);
        searchAndEnableScene(69);
        searchAndEnableScene(70);
    }
    searchAndSetUndo(65, true);

    if (zoneId == 0) {
        if (groupId == 0) {
            searchAndEnableScene(90);
            searchAndEnableScene(91);
        }
    }

    if (groupId === 4) {
        searchAndEnableScene(0); // preset0
        searchAndEnableScene(5); // preset1
        searchAndEnableScene(17); // preset2
        searchAndEnableScene(18); // preset3
        searchAndEnableScene(19); // preset4
    }

    if (((zoneId == 0) && (groupId > 63)) || (groupId == 10) || (groupId == 12)) {
        activeBasicScenes.forEach(function (sceneId) {
            searchAndEnableScene(sceneId);
        });
    } else if (dss.staticDataModel.hasConnectedDevicesForGroup(zoneId, groupId)) {
        searchAndEnableScene(0); // preset0
        searchAndEnableScene(5); // preset1
        searchAndEnableScene(17); // preset2
        searchAndEnableScene(18); // preset3
        searchAndEnableScene(19); // preset4
    }

    searchAndSetUndo(76, true);
    searchAndSetUndo(74, true);
    searchAndSetUndo(83, true);
    searchAndSetUndo(84, true);
    searchAndSetUndo(85, true);

    return oRetArray;
};

dss.staticDataModel.getUserGroupColor = function (groupID) {
    var oArray = dss.staticDataModel.getUsergroups();
    if (oArray != null) {
        for (var iIndex = 0; iIndex < oArray.length; iIndex++) {
            if (oArray[iIndex].id == groupID) {
                return oArray[iIndex].color;
            }
        }
    }
    return -1;
};

dss.staticDataModel.getUserGroupName = function (groupID) {
    var oArray = dss.staticDataModel.getUsergroups();
    if (oArray) {
        for (var iIndex = 0; iIndex < oArray.length; iIndex++) {
            if (oArray[iIndex].id == groupID) {
                return oArray[iIndex].name;
            }
        }
    }
    return '';
};

dss.staticDataModel.isClusterGroup = function (groupID) {
    return ((groupID >= 16) && (groupID < 40));
};

dss.staticDataModel.getImagePathForGroup = function (zoneID, groupID) {
    if (dss.staticDataModel.nameCache == null) {
        dss.staticDataModel.initCaches();
    }

    if (zoneID == 0) {
        if (dss.staticDataModel.isClusterGroup(groupID)) {
            switch (dss.staticDataModel.getUserGroupColor(groupID)) {
                case 1:
                    return dss.staticDataModel.baseImagePath + 'groupIcons/usergroup_yellow.png';
                case 2:
                    return dss.staticDataModel.baseImagePath + 'groupIcons/usergroup_grey.png';
                case 3:
                case 9:
                case 10:
                case 48:
                    return dss.staticDataModel.baseImagePath + 'groupIcons/usergroup_blue.png';
                case 4:
                    return dss.staticDataModel.baseImagePath + 'groupIcons/usergroup_cyan.png';
                case 5:
                    return dss.staticDataModel.baseImagePath + 'groupIcons/usergroup_magenta.png';
                case 6:
                    return dss.staticDataModel.baseImagePath + 'groupIcons/usergroup_red.png';
                case 7:
                    return dss.staticDataModel.baseImagePath + 'groupIcons/usergroup_green.png';
                case 8:
                    return dss.staticDataModel.baseImagePath + 'groupIcons/usergroup_black.png';
            }
        }
    }

    switch (groupID) {
        case 0:
            return dss.staticDataModel.baseImagePath + 'groupIcons/group_0.png';
        case 1:
            return dss.staticDataModel.baseImagePath + 'groupIcons/group_1.png';
        case 2:
            return dss.staticDataModel.baseImagePath + 'groupIcons/group_2.png';
        case 3:
        case 9:
        case 48:
            return dss.staticDataModel.baseImagePath + 'groupIcons/group_3.png';
        case 4:
            return dss.staticDataModel.baseImagePath + 'groupIcons/group_4.png';
        case 5:
            return dss.staticDataModel.baseImagePath + 'groupIcons/group_5.png';
        case 6:
            return dss.staticDataModel.baseImagePath + 'groupIcons/group_6.png';
        case 7:
            return dss.staticDataModel.baseImagePath + 'groupIcons/group_7.png';
        case 8:
            return dss.staticDataModel.baseImagePath + 'groupIcons/group_8.png';
        case 10:
        case 12:
        case 64:
            return dss.staticDataModel.baseImagePath + 'groupIcons/group_64.png';
        default:
            return dss.staticDataModel.baseImagePath + 't.gif';
    }
};

dss.staticDataModel.getImageForScene = function (zoneID, groupID, sceneID, fUndoIcon) {
    if (dss.staticDataModel.nameCache == null) {
        dss.staticDataModel.initCaches();
    }

    var src = dss.staticDataModel.getImagePathForScene(zoneID, groupID, sceneID, fUndoIcon);
    return src
        ? '<img src="' + src + '" alt="' + dss.staticDataModel.getNameForScene(zoneID, groupID, sceneID) + '">'
        : '';
};

dss.staticDataModel.getImageForGroup = function (zoneID, groupID) {
    if (dss.staticDataModel.nameCache == null) {
        dss.staticDataModel.initCaches();
    }
    return '<img src="' + dss.staticDataModel.getImagePathForGroup(zoneID, groupID) + '" alt="' + dss.staticDataModel.getNameForGroup(zoneID, groupID) + '">';

};

dss.staticDataModel.getImageForDevice = function (device) {
    if (dss.staticDataModel.nameCache == null) {
        dss.staticDataModel.initCaches();
    }
    if (device.dSUID) {
        return '/icons/getDeviceIcon?dsuid=' + device.dSUID + '&' + cachingURL;
    } else {
        return '/icons/getDeviceIcon?dsid=' + device.dSID + '&' + cachingURL;
    }
};

dss.staticDataModel.getDeviceType = function (device) {
    if (dss.staticDataModel.nameCache == null) {
        dss.staticDataModel.initCaches();
    }
    return ((device.productID >> 10) & 0x3f);
};

dss.staticDataModel.getProductNameForDevice = function (device) {
    if (dss.staticDataModel.nameCache == null) {
        dss.staticDataModel.initCaches();
    }

    if (device.isVdcDevice) {
        return device.HWInfo;
    }

    if (device.HWInfo) {
        return device.HWInfo;
    }

    var sName = '';
    switch ((device.functionID & 0xf000) >> 12) {
        case 1:
            sName += 'GE';
            break;
        case 2:
            sName += 'GR';
            break;
        case 3:
            sName += 'BL';
            break;
        case 4:
            sName += 'TK';
            break;
        case 5:
            sName += 'MG';
            break;
        case 6:
            sName += 'RT';
            break;
        case 7:
            sName += 'GN';
            break;
        case 8:
            sName += 'SW';
            break;
        case 9:
            sName += 'WE';
            break;
    }
    sName += '-';
    switch ((device.productID >> 10) & 0x3f) {
        case 0:
            sName += 'KM';
            break;
        case 1:
            sName += 'TKM';
            break;
        case 2:
            sName += 'SDM';
            break;
        case 3:
            sName += 'KL';
            break;
        case 4:
            sName += 'TUP';
            break;
        case 5:
            sName += 'ZWS';
            break;
        case 6:
            sName += 'SDS';
            break;
        case 8:
            sName += 'AKM';
            break;
        case 10:
            sName += 'TNY';
            break;
        case 11:
            sName += 'UMV';
            break;
        case 12:
            sName += 'UMR';
            break;
    }
    var deviceNumber = (device.productID & 0x3ff);
    // remove housing colour
    if (deviceNumber > 200 && deviceNumber < 210) {
        deviceNumber = 200;
    }
    return sName + ' ' + deviceNumber;
};

dss.staticDataModel.getTooltipForDevice = function (device) {
    // TODO dsuid
    return '<table>' +
        '<tr>' +
        '<td>' + _('Produkt') + ':</td><td>' + dss.staticDataModel.getProductNameForDevice(device) + '</td>' +
        '</tr><tr>' +
        '<td>' + _('Room') + ':</td><td>' + dss.staticDataModel.getNameForZone(device.ZoneID) + '</td>' +
        '</tr><tr>' +
        '<td>' + _('Meter') + ':</td><td>' + dss.staticDataModel.getMeterName(device.DSMeterDSUID) + '</td>' +
        '</tr><tr>' +
        '<td>' + _('ID') + ':</td><td colspan=&quot;2&quot;>' + device.DisplayID + '</td>' +
        '</tr>' +
        '</table>';
};

dss.staticDataModel.getTooltipForState = function (type, value) {
    switch (type) {
        case 7:
            if (value) {
                return _('If smoke is detected, a fire alarm will be raised automaticly by default');
            }
            break;
        case 8:
            if (value) {
                return _('If wind is detected, a wind alarm will be raised automaticly by default');
            }
            break;
        case 9:
            if (value) {
                return _('If rain is detected, a rain alarm will be raised automaticly by default');
            }
            break;
        case 13:
            if (value) {
                return _('The window is completely closed');
            } else {
                return _('The window is either tilted or completely open');
            }
        case 14:
            if (value) {
                return _('The door is closed');
            } else {
                return _('The door is open');
            }
        case 15:
            if (value == dss.whState.WH_Closed) {
                return _('The window is completely closed');
            } else if (value == dss.whState.WH_Open) {
                return _('The window is fully open');
            } else if (value == dss.whState.WH_Tilted) {
                return _('The window is tilted open');
            }
            break;
    }
    return '';
};

dss.staticDataModel.IsWindowHandle = function (oDevice) {
    var bResult = false;
    if (oDevice && oDevice.binaryInputs) {
        for (var iIndex = 0; iIndex < oDevice.binaryInputs.length; iIndex++) {
            if (oDevice.binaryInputs[iIndex].type == 15) {
                bResult = true;
                break;
            }
        }
    }
    return bResult;
};

dss.staticDataModel.GetWindowHandleBinaryInputIndex = function (oDevice) {
    if (oDevice && oDevice.binaryInputs) {
        for (var iIndex = 0; iIndex < oDevice.binaryInputs.length; iIndex++) {
            if (oDevice.binaryInputs[iIndex].type == 15) {
                return iIndex;
            }
        }
    }
    return -1;
};

/*
                    * oJsonObject: Typical dss-Answer Object paramDef: array of expected Objects in
                    * AnswerArray { name: properyName, type:
                    * simple|action|trigger|condition/enabled }
                    */
dss.parsePropertyQueryAnswer = function (oJsonObject, paramDef) {
    if (oJsonObject.ok == null) {
        return null;
    }
    if (!oJsonObject.ok) {
        return null;
    }
    if (oJsonObject.result == null) {
        return null;
    }
    var oReturnArray = [];
    for (var sKey in oJsonObject.result) {
        var oRootArray = oJsonObject.result[sKey];
        for (var iIndex = 0; oRootArray && iIndex < oRootArray.length; iIndex++) {
            var oObject = {};
            var oIteratorObject = oRootArray[iIndex];
            for (var sIndex = 0; sIndex < paramDef.length; sIndex++) {
                var oKeyParameter = paramDef[sIndex].name;

                if (paramDef[sIndex].type === 'simple') {
                    if (oIteratorObject[oKeyParameter] != null) {
                        oObject[oKeyParameter] = escape(dss.unescapeProperty(oIteratorObject[oKeyParameter]));
                    } else {
                        oObject[oKeyParameter] = null;
                    }
                } else if (paramDef[sIndex].type === 'date') {
                    if (oIteratorObject[oKeyParameter] != null) {
                        oObject[oKeyParameter] = new Date(parseInt(oIteratorObject[oKeyParameter], 10) * 1000);
                    }
                } else if ((paramDef[sIndex].type === 'action') || (paramDef[sIndex].type === 'trigger')) {
                    oObject[oKeyParameter] = [];
                    for (var subIndex in oIteratorObject) {
                        if (typeof (oIteratorObject[subIndex]) === 'object') {
                            var aArray = oIteratorObject[subIndex];
                            if (aArray) {
                                for (var jIndex = 0; jIndex < aArray.length; jIndex++) {
                                    var oArrayObject = aArray[jIndex];
                                    if (oArrayObject[oKeyParameter] != null) {

                                        for (var kIndex = 0; kIndex < oArrayObject[oKeyParameter].length; kIndex++) {
                                            var resultObject = oArrayObject[oKeyParameter][kIndex];
                                            var oActionObject = {};
                                            for (var oKey in resultObject) {
                                                oActionObject[oKey] = resultObject[oKey];
                                                if (oKey === 'type') {
                                                    if (oActionObject[oKey] == 'bus-zone-scene') {
                                                        oActionObject[oKey] = 'zone-scene';
                                                    }
                                                }
                                                if (oKey == 'addon-id') {
                                                    oActionObject.addonId = oActionObject[oKey];
                                                }
                                            }

                                            if (oActionObject.type === 'event') {
                                                for (var oKey in resultObject) {
                                                    if (typeof (resultObject[oKey]) === 'object') {
                                                        var mayBeParameter = {};
                                                        var fIsParameter = false;
                                                        for (var sSubKey in resultObject[oKey]) {
                                                            for (var sSubSubKey in resultObject[oKey][sSubKey]) {
                                                                if (sSubSubKey != 'parameter') {
                                                                    mayBeParameter[sSubSubKey] = resultObject[oKey][sSubKey][sSubSubKey];
                                                                } else {
                                                                    fIsParameter = true;
                                                                }
                                                            }
                                                        }
                                                        if (fIsParameter) {
                                                            oActionObject.parameter = mayBeParameter;
                                                        }
                                                    }
                                                }
                                            }

                                            oObject[oKeyParameter].push(oActionObject);
                                        }

                                    }
                                }
                            }
                        }
                    }
                } else if ((paramDef[sIndex].type === 'condition')) {
                    var oConditionObject = {
                        enabled: null, // singular value
                        weekdays: null, // singularvalue
                        timeframe: null, // array ....
                        zoneState: null,
                        systemState: null,
                        date: null
                    };
                    for (var subIndex in oIteratorObject) {
                        if (typeof (oIteratorObject[subIndex]) === 'object') {
                            var aArray = oIteratorObject[subIndex];
                            for (var jIndex = 0; aArray && jIndex < aArray.length; jIndex++) {
                                var oArrayObject = aArray[jIndex];
                                if (oArrayObject[oKeyParameter] != null) {
                                    if (oArrayObject.enabled != null) {
                                        oConditionObject.enabled = oArrayObject.enabled;
                                    }
                                    if (oArrayObject.conditions != null) {
                                        for (var kIndex = 0; kIndex < oArrayObject.conditions.length; kIndex++) {
                                            if (oArrayObject.conditions[kIndex] != null) {
                                                if (oArrayObject.conditions[kIndex].date != null) {
                                                    oConditionObject.date = oArrayObject.conditions[kIndex].date;
                                                }
                                            }
                                        }
                                    }
                                    if ((oArrayObject['time-start'] != null) && (oArrayObject['time-end'] != null)) {
                                        var startArray = oArrayObject['time-start'].split(':');
                                        var iStart = parseInt(startArray[0], 10) * 3600 + parseInt(startArray[1], 10) * 60 + parseInt(startArray[2], 10);
                                        var endArray = oArrayObject['time-end'].split(':');
                                        var iEnd = parseInt(endArray[0], 10) * 3600 + parseInt(endArray[1], 10) * 60 + parseInt(endArray[2], 10);
                                        oConditionObject.timeframe = [ {
                                            start: {
                                                timeBase: 'daily',
                                                offset: iStart
                                            },
                                            end: {
                                                timeBase: 'daily',
                                                offset: iEnd
                                            }
                                        } ];
                                    }
                                    if (oArrayObject.weekdays != null) {
                                        var weekArray = oArrayObject.weekdays.split(',');
                                        oConditionObject.weekdays = [];
                                        for (var wIndex = 0; wIndex < weekArray.length; wIndex++) {
                                            switch (weekArray[wIndex]) {
                                                case '0':
                                                    oConditionObject.weekdays.push('SU');
                                                    break;
                                                case '1':
                                                    oConditionObject.weekdays.push('MO');
                                                    break;
                                                case '2':
                                                    oConditionObject.weekdays.push('TU');
                                                    break;
                                                case '3':
                                                    oConditionObject.weekdays.push('WE');
                                                    break;
                                                case '4':
                                                    oConditionObject.weekdays.push('TH');
                                                    break;
                                                case '5':
                                                    oConditionObject.weekdays.push('FR');
                                                    break;
                                                case '6':
                                                    oConditionObject.weekdays.push('SA');
                                                    break;
                                            }
                                        }
                                    }
                                    for (var kIndex = 0; kIndex < oArrayObject[oKeyParameter].length; kIndex++) {
                                        var resultObject = oArrayObject[oKeyParameter][kIndex];
                                        if (resultObject.states != null) {
                                            oConditionObject.systemState = [];
                                            for (var sKey in resultObject) {
                                                if (sKey != 'states') {
                                                    oConditionObject.systemState.push({
                                                        name: sKey,
                                                        value: resultObject[sKey]
                                                    });
                                                }
                                            }
                                        }
                                        if (resultObject['zone-states'] != null) {
                                            oConditionObject.zoneState = [];
                                            for (var zIndex = 0; zIndex < resultObject['zone-states'].length; zIndex++) {
                                                oConditionObject.zoneState.push({
                                                    zone: resultObject['zone-states'][zIndex].zone,
                                                    group: resultObject['zone-states'][zIndex].group,
                                                    scene: resultObject['zone-states'][zIndex].scene
                                                });
                                            }
                                        }
                                        if (resultObject['addon-states'] != null) {
                                            oConditionObject.addonStates = [];

                                            for (var sKey in resultObject['addon-states']) {
                                                var oTempArray = [];
                                                var value;
                                                var name;
                                                var addonID;
                                                for (var stateName in resultObject['addon-states'][sKey]) {
                                                    // for UDSes: 1 - set, 2 - not set
                                                    // for climate addon states (apartment_controller): 0 - off, 1 - heating, 2 - cooling
                                                    if (typeof resultObject['addon-states'][sKey][stateName] === 'number') {
                                                        value = resultObject['addon-states'][sKey][stateName];
                                                        name = stateName;
                                                        oTempArray.push({ name: name, value: value });
                                                    } else {
                                                        addonID = stateName;
                                                    }
                                                }
                                                for (var tIndex = 0; tIndex < oTempArray.length; tIndex++) {
                                                    oConditionObject.addonStates.push({
                                                        scriptID: addonID,
                                                        name: oTempArray[tIndex].name,
                                                        value: oTempArray[tIndex].value
                                                    });
                                                }
                                            }
                                        }
                                    }

                                }
                            }
                        }
                    }
                    oObject[oKeyParameter] = oConditionObject;

                } else if ((paramDef[sIndex].type == 'condition/enabled')) {
                    for (var subIndex in oIteratorObject) {
                        if (typeof (oIteratorObject[subIndex]) === 'object') {
                            var aArray = oIteratorObject[subIndex];
                            if (aArray) {
                                for (var jIndex = 0; jIndex < aArray.length; jIndex++) {
                                    var oArrayObject = aArray[jIndex];
                                    if (oArrayObject.conditions != null) {
                                        if (oArrayObject.enabled != null) {
                                            oObject[oKeyParameter] = !oArrayObject.enabled;
                                        }
                                    }
                                }
                            }
                        }
                    }
                } else if ((paramDef[sIndex].type == 'condition/state')) {
                    for (var subIndex in oIteratorObject) {
                        if (typeof (oIteratorObject[subIndex]) === 'object') {
                            var aArray = oIteratorObject[subIndex];
                            if (aArray) {
                                for (var jIndex = 0; jIndex < aArray.length; jIndex++) {
                                    var oArrayObject = aArray[jIndex];
                                    if (oArrayObject.conditions != null) {
                                        for (var kIndex = 0; kIndex < oArrayObject.conditions.length; kIndex++) {
                                            if (oArrayObject.conditions[kIndex] != null) {
                                                if (oArrayObject.conditions[kIndex][paramDef[sIndex].name] != null) {
                                                    oObject[oKeyParameter] = oArrayObject.conditions[kIndex][paramDef[sIndex].name];
                                                }
                                            }
                                        }
                                    }
                                }
                            }
                        }
                    }
                } else if ((paramDef[sIndex].type === 'message')) {
                    for (var subIndex in oIteratorObject) {
                        if (typeof (oIteratorObject[subIndex]) === 'object') {
                            var aArray = oIteratorObject[subIndex];
                            if (aArray) {
                                for (var jIndex = 0; jIndex < aArray.length; jIndex++) {
                                    var oArrayObject = aArray[jIndex];
                                    if (oArrayObject.message != null) {
                                        oObject[oKeyParameter] = {};
                                        oObject[oKeyParameter].messageType = oArrayObject.messageType;
                                        oObject[oKeyParameter].receiver = oArrayObject.receiver;
                                        oObject[oKeyParameter].subject = escape(dss.unescapeProperty(oArrayObject.subject));
                                        oObject[oKeyParameter].message = escape(dss.unescapeProperty(oArrayObject.messageBody));
                                    }
                                }
                            }
                        }
                    }
                } else if ((paramDef[sIndex].type === 'recurrence')) {
                    var oTimeObject = {};
                    for (var subIndex in oIteratorObject) {
                        if (typeof (oIteratorObject[subIndex]) === 'object') {
                            var aArray = oIteratorObject[subIndex];
                            if (aArray) {
                                for (var jIndex = 0; jIndex < aArray.length; jIndex++) {
                                    var oArrayObject = aArray[jIndex];
                                    if (oArrayObject.recurrenceBase != null) {
                                        oTimeObject.recurrenceBase = oArrayObject.recurrenceBase;
                                    }
                                    if (oArrayObject.time != null) {
                                        oTimeObject.timeArray = [];
                                        for (var oKey in oArrayObject.time) {
                                            var oSubArray = oArrayObject.time[oKey];
                                            for (var aKey in oSubArray) {
                                                if (oSubArray[aKey] != null) {
                                                    if (typeof (oSubArray[aKey]) === 'string') {
                                                        oTimeObject.timeArray.push(oSubArray[aKey]);
                                                    }
                                                }
                                            }
                                        }
                                    }
                                }
                            }
                        }
                    }
                    oObject[oKeyParameter] = oTimeObject;
                } else if ((paramDef[sIndex].type === 'time')) {
                    var oTimeObject = {};
                    for (var subIndex in oIteratorObject) {
                        if (typeof (oIteratorObject[subIndex]) === 'object') {
                            var aArray = oIteratorObject[subIndex];
                            if (aArray) {
                                for (var jIndex = 0; jIndex < aArray.length; jIndex++) {
                                    var oArrayObject = aArray[jIndex];
                                    if (oArrayObject.timeBase != null) {
                                        oTimeObject.timeBase = oArrayObject.timeBase;
                                    }
                                    if (oArrayObject.offset != null) {
                                        oTimeObject.offset = oArrayObject.offset;
                                    }
                                    if (oArrayObject.limitBefore != null) {
                                        oTimeObject.limitBefore = oArrayObject.limitBefore;
                                    }
                                    if (oArrayObject.limitAfter != null) {
                                        oTimeObject.limitAfter = oArrayObject.limitAfter;
                                    }
                                    if (oArrayObject.random != null) {
                                        oTimeObject.random = oArrayObject.random;
                                    }
                                }
                            }
                        }
                    }
                    oObject[oKeyParameter] = oTimeObject;
                }
            }
            oReturnArray.push(oObject);
        }

    }
    return oReturnArray;
};

dss.validateTriggerObject = function (oTriggerObject) {
    switch (oTriggerObject.type) {
        case 'addon-state-change': {
            if (oTriggerObject['addon-id'] == 'system-addon-user-defined-states') {
                var stateName = oTriggerObject.name;
                var oUserStates = dss.staticDataModel.getUserStates();
                var stateValue = oTriggerObject.state;
                if (oUserStates) {
                    for (var iIndex = 0; iIndex < oUserStates.length; iIndex++) {
                        if (oUserStates[iIndex].name == stateName) {
                            if (stateValue === 'active') {
                                return dss.validValue.ok;
                            }
                            if (stateValue === 'inactive') {
                                return dss.validValue.ok;
                            }
                        }
                    }
                }
                return dss.validValue.completlyWrong;
            } else {
                if (oTriggerObject.state != null) {
                    if (oTriggerObject.statename != null) {
                        return dss.validValue.ok;
                    }
                }
                return dss.validValue.completlyWrong;
            }
        }
        case 'device-named-event': {
            var oDevice = dss.staticDataModel.getDevice(oTriggerObject.dsuid);
            if (typeof oDevice.present === 'undefined' || oDevice.present === null) {
                return dss.validValue.completlyWrong; // gel�schtes ger�t ->
                // vollkommen falsch
            }
            if (!oDevice.present) {
                return dss.validValue.temporaryWrong; // nicht vorhandene Geraete
            }
            if (oDevice != null) {
                if (oDevice.deviceEvents != null) {
                    if (oDevice.deviceEvents[oTriggerObject.id] != null) {
                        return dss.validValue.ok;
                    }
                }
            }
            return dss.validValue.completlyWrong;
        }
        case 'device-named-action': {
            var oDevice = dss.staticDataModel.getDevice(oTriggerObject.dsuid);
            if (typeof oDevice.present === 'undefined' || oDevice.present === null) {
                return dss.validValue.completlyWrong; // gel�schtes ger�t ->
                // vollkommen falsch
            }
            if (!oDevice.present) {
                return dss.validValue.temporaryWrong; // nicht vorhandene Geraete
            }
            if (oDevice != null) {
                if (oDevice.deviceCustomActions != null) {
                    if (oDevice.deviceCustomActions[oTriggerObject.id] != null) {
                        return dss.validValue.ok;
                    }
                }
            }
            if (oDevice != null) {
                if (oDevice.deviceStandardActions != null) {
                    if (oDevice.deviceStandardActions[oTriggerObject.id] != null) {
                        return dss.validValue.ok;
                    }
                }
            }
            if (oDevice != null) {
                if (oDevice.deviceActions != null) {
                    if (oDevice.deviceActions[oTriggerObject.id] != null) {
                        return dss.validValue.ok;
                    }
                }
            }
            return dss.validValue.completlyWrong;
        }
        case 'event': {
            switch (oTriggerObject.name) {
                case 'sunshine':
                    if (oTriggerObject.parameter.value == 1) {
                        return dss.validValue.ok;
                    }
                    if (oTriggerObject.parameter.value == 2) {
                        return dss.validValue.ok;
                    }
            }
            break;
        }
        case 'state-change': {

            var stateName = oTriggerObject.name;
            var oUserStates = dss.staticDataModel.getUserStates();
            var stateValue = oTriggerObject.state;
            if (oUserStates) {
                for (var iIndex = 0; iIndex < oUserStates.length; iIndex++) {
                    if (oUserStates[iIndex].name == stateName) {
                        if (stateValue === 'active') {
                            return dss.validValue.ok;
                        }
                        if (stateValue === 'inactive') {
                            return dss.validValue.ok;
                        }
                    }
                }
            }

            var oArray = oTriggerObject.name.split('.');
            if (oArray.length === 3) {
                if (oArray[0] === 'dev') {
                    var oDevice = dss.staticDataModel.getDevice(oArray[1]);
                    if (oDevice != null) {

                        if ((oDevice.hasStates) && (dss.staticDataModel.getDeviceStates(oArray[1]))) {
                            var states = dss.staticDataModel.getDeviceStates(oArray[1]);
                            for (var sKey in states) {
                                if (sKey == oArray[2]) {
                                    for (var sSubkey in states[sKey].options) {
                                        if (sSubkey == oTriggerObject.state) {
                                            return dss.validValue.ok;
                                        }
                                    }
                                }
                            }
                        }


                        var typeIndex = parseInt(oArray[2], 10);
                        if (oDevice.specialActions != null) {
                            if (oDevice.specialActions.triggers != null) {
                                if (oDevice.specialActions.triggers.binaryInputs != null) {
                                    for (var sKey in oDevice.specialActions.triggers.binaryInputs) {
                                        if (('' + oDevice.specialActions.triggers.binaryInputs[sKey].id) == ('' + typeIndex)) {
                                            if (oTriggerObject.state === 'active') {
                                                if (oDevice.specialActions.triggers.binaryInputs[sKey].value == 2) {
                                                    return dss.validValue.ok;
                                                }
                                            }

                                            if (oTriggerObject.state === 'inactive') {
                                                if (oDevice.specialActions.triggers.binaryInputs[sKey].value == 1) {
                                                    return dss.validValue.ok;
                                                }
                                            }
                                        }
                                    }
                                }
                            }
                        }

                        if (oDevice.binaryInputs != null) {
                            var type = 255;
                            for (var kIndex = 0; kIndex < oDevice.binaryInputs.length; kIndex++) {
                                if (oDevice.binaryInputs[kIndex].index == typeIndex) {
                                    type = oDevice.binaryInputs[kIndex].type;
                                }
                            }
                            if (type != 255) {
                                if (type == 15) { // window handle
                                    if (oTriggerObject.state === 'closed') {
                                        return dss.validValue.ok;
                                    } else if (oTriggerObject.state === 'open') {
                                        return dss.validValue.ok;
                                    } else if (oTriggerObject.state === 'tilted') {
                                        return dss.validValue.ok;
                                    }
                                } else {
                                    if (oTriggerObject.state === 'active') {
                                        return dss.validValue.ok;
                                    }
                                    if (oTriggerObject.state === 'inactive') {
                                        return dss.validValue.ok;
                                    }
                                }
                            }
                        }
                    }
                    return dss.validValue.completlyWrong;
                }
            }
            if (oArray[0] === 'zone') {
                var states = dss.staticDataModel.getStates();
                if (states) {
                    for (var iIndex = 0; iIndex < states.length; iIndex++) {
                        if (states[iIndex].name == oTriggerObject.name) {
                            if (oTriggerObject.state === 'active') {
                                return dss.validValue.ok;
                            }
                            if (oTriggerObject.state === 'inactive') {
                                return dss.validValue.ok;
                            }
                        }
                    }
                }
                return dss.validValue.completlyWrong;
            }
            if (oArray[0] === 'cluster') {
                var states = dss.staticDataModel.getStates();
                if (states) {
                    for (var iIndex = 0; iIndex < states.length; iIndex++) {
                        if (states[iIndex].name == oTriggerObject.name) {
                            if (oTriggerObject.state === 'active') {
                                return dss.validValue.ok;
                            }
                            if (oTriggerObject.state === 'inactive') {
                                return dss.validValue.ok;
                            }
                        }
                    }
                }
                return dss.validValue.completlyWrong;
            }

            switch (oArray[0]) {
                case 'fire':
                case 'hail':
                case 'rain':
                case 'wind':
                case 'frost':
                case 'sun':
                case 'service':
                    if (oTriggerObject.state === 'active') {
                        return dss.validValue.ok;
                    }
                    if (oTriggerObject.state === 'inactive') {
                        return dss.validValue.ok;
                    }
                    break;
                case 'heating_mode':
                    if (oTriggerObject.state === 'off') {
                        return dss.validValue.ok;
                    }
                    if (oTriggerObject.state === 'heating') {
                        return dss.validValue.ok;
                    }
                    if (oTriggerObject.state === 'cooling') {
                        return dss.validValue.ok;
                    }
                    if (oTriggerObject.state === 'auto') {
                        return dss.validValue.ok;
                    }
                    break;
                case 'heating_water_system':
                    if (oTriggerObject.state === 'off') {
                        return dss.validValue.ok;
                    }
                    if (oTriggerObject.state == 'hot water') {
                        return dss.validValue.ok;
                    }
                    if (oTriggerObject.state == 'cold water') {
                        return dss.validValue.ok;
                    }
                    if (oTriggerObject.state === 'auto') {
                        return dss.validValue.ok;
                    }
                    break;
            }
            break;
        }
        case 'device-sensor': {
            var oDevice = dss.staticDataModel.getDevice(oTriggerObject.dsuid);
            if (typeof oDevice.present === 'undefined' || oDevice.present === null) {
                return dss.validValue.completlyWrong; // gel�schtes ger�t ->
                // vollkommen falsch
            }
            if (!oDevice.present) {
                return dss.validValue.temporaryWrong; // nicht vorhandene Geraete
                // -> Fehler
            }
            if (oDevice.sensorEvents == null) {
                return dss.validValue.completlyWrong;
            }
            for (var iIndex = 0; iIndex < oDevice.sensorEvents.length; iIndex++) {
                if (oDevice.sensorEvents[iIndex].id == oTriggerObject.eventid) {
                    return dss.validValue.ok;
                }
            }

            return dss.validValue.completlyWrong;
        }
        case 'device-scene': {
            var oDevice = dss.staticDataModel.getDevice(oTriggerObject.dsuid);
            if (typeof oDevice.present === 'undefined' || oDevice.present === null) {
                return dss.validValue.completlyWrong; // gel�schtes ger�t ->
                // vollkommen falsch
            }
            if (!oDevice.present) {
                return dss.validValue.temporaryWrong; // nicht vorhandene Geraete
                // -> Fehler
            }
            if ((oDevice.tasterMode == 0) && (oDevice.buttonType == 65)) {
                switch (oTriggerObject.scene) {
                    case 50:
                        return dss.validValue.ok;
                    case 51:
                        return dss.validValue.ok;
                    case 15:
                        return dss.validValue.ok;
                }
            }
            if (oDevice.tasterMode != 0) {
                return dss.validValue.completlyWrong; // reconfigured device
            }
            if (oTriggerObject.scene == null) {
                return dss.validValue.completlyWrong;
            }
            switch (oTriggerObject.scene) {
                case 50:
                    return dss.validValue.ok;
                case 51:
                    return dss.validValue.ok;
            }
            return dss.validValue.temporaryWrong;
        }
        case 'device-msg': {
            var oDevice = dss.staticDataModel.getDevice(oTriggerObject.dsuid);
            if (typeof oDevice.present === 'undefined' || oDevice.present === null) {
                return dss.validValue.completlyWrong; // gel�schtes ger�t ->
                // vollkommen falsch
            }
            if (!oDevice.present) {
                return dss.validValue.temporaryWrong; // nicht vorhandene Geraete
                // -> Fehler
            }


            if (oDevice.specialActions != null) {
                if (oDevice.specialActions.triggers != null) {
                    if (oDevice.specialActions.triggers.deviceMsg != null) {
                        for (var sKey in oDevice.specialActions.triggers.deviceMsg) {
                            if (oDevice.specialActions.triggers.deviceMsg[sKey].id == oTriggerObject.msg) {
                                return dss.validValue.ok;
                            }
                        }
                    }
                }
            }


            if (oDevice.tasterMode != 15) { // the device is not anymore configured
                // as pushbutton
                return dss.validValue.completlyWrong;
            }
            if (oTriggerObject.msg == null) {
                return dss.validValue.completlyWrong;
            }
            switch (oTriggerObject.msg) {
                case 0:
                    if ((oTriggerObject.buttonIndex != null) && (oDevice.buttonType != 2)) {
                        return dss.validValue.completlyWrong;
                    } else {
                        return dss.validValue.ok;
                    }
                case 7:
                case 8:
                case 1:
                case 9:
                case 2:
                case 3:
                case 6:
                case 10:
                    return dss.validValue.ok;
            }
            return dss.validValue.completlyWrong;
        }
        case 'device-action': {
            var oDevice = dss.staticDataModel.getDevice(oTriggerObject.dsuid);
            if (typeof oDevice.present === 'undefined' || oDevice.present === null) {
                return dss.validValue.completlyWrong; // gel�schtes ger�t ->
                // vollkommen falsch
            }
            if (!oDevice.present) {
                return dss.validValue.temporaryWrong; // nicht vorhandene Geraete
                // -> Fehler
            }
            if (oTriggerObject.action == null) {
                return dss.validValue.completlyWrong;
            }
            if (dss.staticDataModel.specialDSIDAction[oTriggerObject.dsuid] != null) {
                if (dss.staticDataModel.specialDSIDAction[oTriggerObject.dsuid].triggers != null) {
                    if (dss.staticDataModel.specialDSIDAction[oTriggerObject.dsuid].triggers.deviceAction != null) {
                        for (var sKey in dss.staticDataModel.specialDSIDAction[oTriggerObject.dsuid].triggers.deviceAction) {
                            if (dss.staticDataModel.specialDSIDAction[oTriggerObject.dsuid].triggers.deviceAction[sKey].id == oTriggerObject.action) {
                                return dss.validValue.ok;
                            }
                        }
                    }
                }
            }
            if (oDevice.specialActions != null) {
                if (oDevice.specialActions.triggers != null) {
                    if (oDevice.specialActions.triggers.deviceActions != null) {
                        for (var sKey in oDevice.specialActions.triggers.deviceActions) {
                            if (oDevice.specialActions.triggers.deviceActions[sKey].action == oTriggerObject.action) {
                                return dss.validValue.ok;
                            }
                        }
                    }
                }
            }

            if (oDevice.tasterMode != 15) { // the device is not anymore configured
                // as pushbutton
                return dss.validValue.completlyWrong;
            }

            return dss.validValue.completlyWrong;
        }
        case 'device-binary-input': {
            if (!oTriggerObject || oTriggerObject.dsuid == null) {
                return dss.validValue.completlyWrong;
            }
            var oDevice = dss.staticDataModel.getDevice(oTriggerObject.dsuid);
            if (!oDevice || oDevice.present == null) {
                return dss.validValue.completlyWrong; // gel�schtes ger�t ->
                // vollkommen falsch
            }
            if (!oDevice.present) {
                return dss.validValue.temporaryWrong; // nicht vorhandene Geraete
                // -> Fehler
            }
            if ((oDevice.tastermode != 15) || !(oDevice.inputType >= 16 <= 21)) {
                return dss.validValue.completlyWrong; // reconfigured device
            }
            if (oTriggerObject.state == null) {
                return dss.validValue.completlyWrong;
            }
            switch (oTriggerObject.state) {
                case 0:
                case 1:
                    return dss.validValue.ok;
            }
            return dss.validValue.completlyWrong;
        }
        case 'undo-zone-scene':
            var oSceneObject = dss.staticDataModel.getScene(oTriggerObject.zone, oTriggerObject.group, oTriggerObject.scene);
            if (oSceneObject == null) {
                return dss.validValue.completlyWrong;
            }
            if (oSceneObject.undoable == null) {
                return dss.validValue.completlyWrong;
            }
            return (oSceneObject.undoable ? dss.validValue.ok : dss.validValue.completlyWrong);
        case 'zone-scene':
        case 'bus-zone-scene':
            if (oTriggerObject.zone == 0) {
                // Apartment
                if (oTriggerObject.group != 0) {
                    if (oTriggerObject.scene > 63) {
                        return dss.validValue.completlyWrong;
                    } // apartment-szenen und raumszenen sind immer falsch
                    var oGroups = dss.staticDataModel.getGroupForZone(oTriggerObject.zone);
                    if (oGroups) {
                        for (var iIndex = 0; iIndex < oGroups.length; iIndex++) {
                            if (oGroups[iIndex].group == oTriggerObject.group) {
                                return (oGroups[iIndex].reachable ? dss.validValue.ok : dss.validValue.temporaryWrong);
                            } // nur wenn gruppe erreichbar ist, dann kann aktion laufen
                        }
                    }
                    return dss.validValue.completlyWrong; // gruppe ist nicht in
                    // der liste mit drin
                } else if (oTriggerObject.scene > 63) {
                    return dss.validValue.ok;
                    // Alle R�ume + Deepoff ist auswaehlbar, daher hier genehmigt
                } else {
                    return dss.validValue.completlyWrong;
                }
            } else {
                // Zone
                var oZones = dss.staticDataModel.getZoneArray();
                var fValue = false;
                if (oZones) {
                    for (var iIndex = 0; iIndex < oZones.length; iIndex++) {
                        if (oZones[iIndex].ZoneID == oTriggerObject.zone) {
                            fValue = true;
                        }
                    }
                }
                if (oTriggerObject.zone == -1) {
                    fValue = true;
                }

                if (fValue == false) {
                    return dss.validValue.completlyWrong;
                } // zone is not valid
                if (oTriggerObject.group == 0) {
                    if (oTriggerObject.scene == 64) {
                        return dss.validValue.ok;
                    }
                    if (oTriggerObject.scene == 67) {
                        return dss.validValue.ok;
                    }
                    if (oTriggerObject.scene == 68) {
                        return dss.validValue.ok;
                    }
                    if (oTriggerObject.scene == 69) {
                        return dss.validValue.ok;
                    }
                    if (oTriggerObject.scene == 70) {
                        return dss.validValue.ok;
                    }
                    return dss.validValue.completlyWrong;
                } else {
                    if (oTriggerObject.scene > 63) {
                        return dss.validValue.completlyWrong;
                    } // apartment-szenen
                    // und raumszenen
                    // sind
                    // immer falsch
                    if (oTriggerObject.zone == -1) {
                        return dss.validValue.ok;
                    }

                    var oGroups = dss.staticDataModel.getGroupForZone(oTriggerObject.zone);
                    if (oGroups) {
                        for (var iIndex = 0; iIndex < oGroups.length; iIndex++) {
                            if (oGroups[iIndex].group == oTriggerObject.group) {
                                return (oGroups[iIndex].reachable ? dss.validValue.ok : dss.validValue.temporaryWrong);
                            } // nur
                            // wenn
                            // gruppe
                            // erreichbar
                            // ist, dann
                            // kann aktion
                            // laufen
                        }
                    }
                    return dss.validValue.completlyWrong; // gruppe ist nicht in
                    // der liste mit drin
                }
            }
        case 'custom-event': {
            var hle = dss.staticDataModel.getHLE();
            if (hle) {
                for (var iIndex = 0; iIndex < hle.length; iIndex++) {
                    if ('' + hle[iIndex].id == oTriggerObject.event) {
                        return dss.validValue.ok;
                    }
                }
            }
            return dss.validValue.completlyWrong;
        }
    }
    return dss.validValue.completlyWrong;
};

dss.validateActionObject = function (oActionObject) {     // , groupID
    var fValue = dss.validValue.ok;
    switch (oActionObject.type) {
        case DSS.ActionTypes.DEVICE_MOC_VALUE: {
            var oDevice = dss.staticDataModel.getDevice(oActionObject.dsuid);
            if (typeof oDevice.present === 'undefined' || oDevice.present === null) {
                return dss.validValue.completlyWrong; // gel�schtes ger�t ->
                // vollkommen falsch
            }
            if (!oDevice.present) {
                return dss.validValue.temporaryWrong; // nicht vorhandene Geraete
                // -> Fehler
            }
            var iIndex = 0;
            while (oActionObject['channelid-' + iIndex] != null) {
                if (oDevice.moc.indexOf(oActionObject['channelid-' + iIndex]) == -1) {
                    return dss.validValue.completlyWrong;
                }
                iIndex++;
            }
            return dss.validValue.ok;
        }

        case DSS.ActionTypes.DEVICE_ACTION: {
            var oDevice = dss.staticDataModel.getDevice(oActionObject.dsuid);
            if (typeof oDevice.present === 'undefined' || oDevice.present === null) {
                return dss.validValue.completlyWrong; // gel�schtes ger�t ->
                // vollkommen falsch
            }
            if (!oDevice.present) {
                return dss.validValue.temporaryWrong; // nicht vorhandene Geraete
                // -> Fehler
            }

            var actions = dss.staticDataModel.getStandardActions(oActionObject.dsuid);
            for (var sKey in actions) {
                if (sKey == oActionObject.id) {
                    return dss.validValue.ok;
                }
            }
            actions = dss.staticDataModel.getDeviceActions(oActionObject.dsuid);
            for (var sKey in actions) {
                if (sKey == oActionObject.id) {
                    return dss.validValue.ok;
                }
            }
            actions = dss.staticDataModel.getCustomActions(oActionObject.dsuid);
            for (var sKey in actions) {
                if (sKey == oActionObject.id) {
                    return dss.validValue.ok;
                }
            }
            return dss.validValue.completlyWrong;
        }

        case DSS.ActionTypes.CHANGE_ADDON_STATE: {
            if (oActionObject['addon-id'] === 'system-addon-user-defined-states') {
                var stateName = oActionObject.statename;
                var oUserStates = dss.staticDataModel.getUserStates();
                if (oUserStates) {
                    for (var iIndex = 0; iIndex < oUserStates.length; iIndex++) {
                        if (oUserStates[iIndex].name == stateName) {
                            if (oActionObject.state === 'active') {
                                return dss.validValue.ok;
                            }
                            if (oActionObject.state === 'inactive') {
                                return dss.validValue.ok;
                            }
                        }
                    }
                }
                return dss.validValue.completlyWrong;
            } else {
                if (oActionObject.state != null) {
                    if (oActionObject.statename != null) {
                        return dss.validValue.ok;
                    }
                }
                return dss.validValue.completlyWrong;
            }
        }

        case DSS.ActionTypes.CHANGE_STATE: {
            var stateName = oActionObject.statename.split('.')[0];
            switch (stateName) {
                case 'fire':
                case 'hail':
                case 'rain':
                case 'wind':
                    if (oActionObject.state === 'active') {
                        return dss.validValue.ok;
                    }
                    if (oActionObject.state === 'inactive') {
                        return dss.validValue.ok;
                    }
                    break;

                case 'heating_water_system':
                    if (oActionObject.state === 'off') {
                        return dss.validValue.ok;
                    }
                    if (oActionObject.state == 'cold water') {
                        return dss.validValue.ok;
                    }
                    if (oActionObject.state == 'hot water') {
                        return dss.validValue.ok;
                    }
                    return dss.validValue.completlyWrong;
            }
            break;
        }
        case DSS.ActionTypes.ZONE_VALUE: {
            return dss.validValue.completlyWrong; // nicht mehr unterstuetzt
        }

        case DSS.ActionTypes.HEATING_MODE: {
            if (dss.staticDataModel.ClimateControllerSettingsCache &&
                                    dss.staticDataModel.ClimateControllerSettingsCache.capabilities &&
                                    (dss.staticDataModel.ClimateControllerSettingsCache.capabilities.heatingSupported ||
                                        dss.staticDataModel.ClimateControllerSettingsCache.capabilities.coolingSupported) &&
                                        dss.staticDataModel.heating[oActionObject.zone].receiveOperationMode &&
                                        oActionObject.mode < 7) {
                return dss.validValue.ok;
            }
            return dss.validValue.completlyWrong;
        }

        case DSS.ActionTypes.ZONE_SCENE: {
            if (oActionObject.zone == 0) {
                // Apartment
                if (dss.staticDataModel.isClusterGroup(oActionObject.group)) {
                    if (oActionObject.scene == 90) {
                        return dss.validValue.ok;
                    } // A
                    if (oActionObject.scene == 91) {
                        return dss.validValue.ok;
                    } // A
                }

                if (oActionObject.group != 0) {
                    if (oActionObject.scene > 63) {
                        return dss.validValue.completlyWrong;
                    } // apartment-szenen
                    // und raumszenen
                    // sind
                    // immer falsch
                    var oGroups = dss.staticDataModel.getGroupForZone(oActionObject.zone);
                    if (oGroups) {
                        for (var iIndex = 0; iIndex < oGroups.length; iIndex++) {
                            if (oGroups[iIndex].group == oActionObject.group) {
                                return (oGroups[iIndex].reachable ? dss.validValue.ok : dss.validValue.temporaryWrong);
                            } // nur
                            // wenn
                            // gruppe
                            // erreichbar
                            // ist, dann
                            // kann aktion
                            // laufen
                        }
                    }
                    return dss.validValue.completlyWrong; // gruppe ist nicht in
                    // der liste mit drin
                } else if (oActionObject.scene > 63) {
                    return dss.validValue.ok;
                // Alle R�ume + Deepoff ist auswaehlbar, daher hier genehmigt
                } else {
                    return dss.validValue.completlyWrong;
                }
            } else {
                // Zone
                var oZones = dss.staticDataModel.getZoneArray();
                var fValue = false;
                if (oZones) {
                    for (var iIndex = 0; iIndex < oZones.length; iIndex++) {
                        if (oZones[iIndex].ZoneID == oActionObject.zone) {
                            fValue = true;
                        }
                    }
                }
                if (oActionObject.group == 6) {
                    if (oActionObject.scene < 64) {
                        return dss.validValue.completlyWrong;
                    }
                } // zone is not valid
                if (oActionObject.group == 7) {
                    if (oActionObject.scene < 64) {
                        return dss.validValue.completlyWrong;
                    }
                } // zone is not valid
                if (!fValue) {
                    return dss.validValue.completlyWrong;
                } // zone is not valid
                if (oActionObject.group == 0) {
                    if (oActionObject.scene == 64) {
                        return dss.validValue.ok;
                    }
                    if (oActionObject.scene == 67) {
                        return dss.validValue.ok;
                    }
                    if (oActionObject.scene == 68) {
                        return dss.validValue.ok;
                    }
                    if (oActionObject.scene == 69) {
                        return dss.validValue.ok;
                    }
                    if (oActionObject.scene == 70) {
                        return dss.validValue.ok;
                    }
                    return dss.validValue.completlyWrong;
                } else {
                    if (oActionObject.scene > 63) {
                        return dss.validValue.completlyWrong;
                    } // apartment-szenen und raumszenen sind immer falsch
                    var oGroups = dss.staticDataModel.getGroupForZone(oActionObject.zone);
                    if (oGroups) {
                        for (var iIndex = 0; iIndex < oGroups.length; iIndex++) {
                            if (oGroups[iIndex].group == oActionObject.group) {
                                return (oGroups[iIndex].reachable
                                    ? dss.validValue.ok
                                    : dss.validValue.temporaryWrong
                                );
                            } // nur wenn gruppe erreichbar ist, dann kann aktion laufen
                        }
                    }
                    return dss.validValue.completlyWrong; // gruppe ist nicht in
                    // der liste mit drin
                }
            }
        }
        case DSS.ActionTypes.UNDO_ZONE_SCENE: {
            var oSceneObject = dss.staticDataModel.getScene(oActionObject.zone, oActionObject.group, oActionObject.scene);
            if (oSceneObject == null) {
                return dss.validValue.completlyWrong;
            }
            if (oSceneObject.undoable == null) {
                return dss.validValue.completlyWrong;
            }
            return (oSceneObject.undoable ? dss.validValue.ok : dss.validValue.completlyWrong);
        }

        case DSS.ActionTypes.DEVICE_SCENE: { // device ohne ausgang!
            var oDevice = dss.staticDataModel.getDevice(oActionObject.dsuid);
            if (typeof oDevice.present === 'undefined' || oDevice.present === null) {
                return dss.validValue.completlyWrong; // gel�schtes ger�t ->
                // vollkommen falsch
            }
            if (!oDevice.present) {
                return dss.validValue.temporaryWrong; // nicht vorhandene Geraete
                // -> Fehler
            }

            if (oDevice.specialActions.actions != null) {
                if (oDevice.specialActions.actions.deviceActions != null) {
                    for (var sKey in oDevice.specialActions.actions.deviceActions) {
                        if (oDevice.specialActions.actions.deviceActions[sKey].scene != null) {
                            if (oDevice.specialActions.actions.deviceActions[sKey].scene == oActionObject.scene) {
                                return dss.validValue.ok;
                            }
                        }
                    }
                }
            }

            if (oDevice.outputMode == 0) {
                return dss.validValue.completlyWrong;
            }
            if (oActionObject.scene > 63) {
                return dss.validValue.completlyWrong;
            }
            return dss.validValue.ok;
        }

        case DSS.ActionTypes.DEVICE_VALUE: {
            var oDevice = dss.staticDataModel.getDevice(oActionObject.dsuid);
            if (typeof oDevice.present === 'undefined' || oDevice.present === null) {
                return dss.validValue.completlyWrong; // gel�schtes ger�t ->
                // vollkommen falsch
            }
            if (!oDevice.present) {
                return dss.validValue.temporaryWrong; // nicht vorhandene Geraete
                // -> Fehler
            }

            if (oDevice.specialActions.actions != null) {
                if (oDevice.specialActions.actions.deviceActions != null) {
                    for (var sKey in oDevice.specialActions.actions.deviceActions) {

                        if (oDevice.specialActions.actions.deviceActions[sKey].outputValue != null) {
                            if (oDevice.specialActions.actions.deviceActions[sKey].outputValue == oActionObject.value) {
                                return dss.validValue.ok;
                            }
                        }

                    }
                }
            }

            if (oDevice.outputMode == 34) {
                if (oActionObject.value == 0) {
                    return dss.validValue.ok;
                }
                if (oActionObject.value == 128) {
                    return dss.validValue.ok;
                }
                if (oActionObject.value == 255) {
                    return dss.validValue.ok;
                }
            }
            if (oDevice.outputMode == 38) {
                if (oActionObject.value == 0) {
                    return dss.validValue.ok;
                }
                if (oActionObject.value == 96) {
                    return dss.validValue.ok;
                }
                if (oActionObject.value == 160) {
                    return dss.validValue.ok;
                }
                if (oActionObject.value == 255) {
                    return dss.validValue.ok;
                }
            }

            return dss.validValue.completlyWrong; // nicht mehr unterstuetzt
        }

        case DSS.ActionTypes.DEVICE_BLINK: {
            var oDevice = dss.staticDataModel.getDevice(oActionObject.dsuid);
            if (typeof oDevice.present === 'undefined' || oDevice.present === null) {
                return dss.validValue.completlyWrong; // gel�schtes ger�t ->
                // vollkommen falsch
            }
            if (!oDevice.present) {
                return dss.validValue.temporaryWrong; // nicht vorhandene Geraete
                // -> Fehler
            }
            if (oDevice.outputMode == 0) {
                return dss.validValue.completlyWrong;
            }
            return dss.validValue.ok;
        }

        case DSS.ActionTypes.ZONE_BLINK: {
            if (oActionObject.zone == 0) {
                if (oActionObject.group == 1) { // nur licht darf blinken
                    return dss.validValue.ok;
                } else {
                    if (dss.staticDataModel.getUserGroupColor(oActionObject.group) == 1) {   // gelbe benutzergruppen aber auch
                        return dss.validValue.ok;
                    }
                    return dss.validValue.completlyWrong;
                }
            }
            var oZones = dss.staticDataModel.getZoneArray();
            var fValue = dss.validValue.completlyWrong;
            if (oZones) {
                for (var iIndex = 0; iIndex < oZones.length; iIndex++) {
                    if (oZones[iIndex].ZoneID == oActionObject.zone) {
                        fValue = dss.validValue.ok;
                    }
                }
            }
            if (fValue == dss.validValue.completlyWrong) {
                return dss.validValue.completlyWrong;
            }
            if (oActionObject.group != 1) { // nur licht darf blinken
                return dss.validValue.completlyWrong;
            }
            var oGroups = dss.staticDataModel.getGroupForZone(oActionObject.zone);
            if (oGroups) {
                for (var iIndex = 0; iIndex < oGroups.length; iIndex++) {
                    if (oGroups[iIndex].group == oActionObject.group) {
                        return (oGroups[iIndex].reachable ? dss.validValue.ok : dss.validValue.temporaryWrong);
                    } // falls licht da ist, dann blinkt es nur dann, wenn es reachable ist
                }
            }
            return dss.validValue.completlyWrong;
        }

        case DSS.ActionTypes.CUSTOM_EVENT: {
            var hle = dss.staticDataModel.getHLE();
            if (hle) {
                for (var iIndex = 0; iIndex < hle.length; iIndex++) {
                    if ('' + hle[iIndex].id == oActionObject.event) {
                        return dss.validValue.ok;
                    }
                }
            }
            return dss.validValue.completlyWrong;
        }

        case DSS.ActionTypes.URL: {
            return dss.validValue.ok;
        }

        case DSS.ActionTypes.AUDIO_ACTIVITY: {
            dss.staticDataModel.getAudioActivities();
            const activity = dss.staticDataModel.audioActivities.find(function (activity) {
                return activity.id === oActionObject.contentSource;
            });
            if (activity) {
                switch (activity.whatToPlay.type) {
                    case 'playlist': {
                        const playlist = dss.staticDataModel.playlists.find(function (playlist) {
                            return playlist.id === activity.whatToPlay.playlistId;
                        });
                        if (playlist) {
                            return dss.validValue.ok;
                        }
                        break;
                    }
                    case 'favorite': {
                        const favorite = dss.staticDataModel.favorites.find(function (favorite) {
                            return favorite.id === activity.whatToPlay.favoriteId;
                        });
                        if (favorite) {
                            return dss.validValue.ok;
                        }
                        break;
                    }
                    case 'clip': {
                        const clip = dss.staticDataModel.clips.find(function (clip) {
                            return clip.id === activity.whatToPlay.clipId;
                        });
                        if (clip) {
                            return dss.validValue.ok;
                        }
                        break;
                    }
                }
            }
            return dss.validValue.completlyWrong;
        }

        default:
            return dss.validValue.completlyWrong;
    }

    return dss.validValue.completlyWrong;
};

/*
                            * als default wird framework/locale/{languageSuffix}/dss-addon-framework.po
                            * geladen es k�nnen aber auch weitere urls angegeben werden, wobei
                            * {languageSuffix} durch den zweiten parameter ersetzt wird oLangdefinition
                            * m�ssen ['url'] sein
                            */
dss.buildUpLang = function (oLangDefinitions, languageSuffix) {
    /*
                                * _ = function (s) { if (dss.translation[s]==null) return s; return
                                * dss.translation[s]; }
                                */

    var params = {
        domain: 'dss-addon-framework',
        locale_data: {}
    };

    if (languageSuffix == null) {
        var oLang;
        try {
            oLang = Ext.JSON.decode(dss.ajaxSyncRequest('/json/property/query', {
                query: '/system/language(locale)'
            }));
        } catch (e) {
            devLog && devLog(e);
        }
        if (oLang != null) {
            if (oLang.ok && (oLang.result && oLang.result.language && oLang.result.language[0] !== undefined)) {
                languageSuffix = oLang.result.language[0].locale;
            } else {
                languageSuffix = 'en_US';
            }
        }
    }
    if (languageSuffix == null) {
        languageSuffix = 'de_DE';
    }
    dss.staticDataModel.activeLanguage = languageSuffix;
    if (oLangDefinitions == null) {
        oLangDefinitions = [];
    }

    oLangDefinitions.splice(0, 0, '/framework/locale/{languageSuffix}/dss-addon-framework.po');
    for (var l = 0, langLen = oLangDefinitions.length; l < langLen; l++) {
        var sDomain = params.domain;

        var sURL = Ext.String.format2(oLangDefinitions[l], {
            languageSuffix: dss.staticDataModel.activeLanguage
        }, false);

        if (params.locale_data[sDomain] == null) {
            params.locale_data[sDomain] = {};
        }

        var sString = dss.ajaxSyncRequest(sURL, {},
            '#\n' +
                                        '#default text, no content\n' +
                                        'msgid ""\n' +
                                        'msgstr ""\n' +
                                        '"Plural-Forms: nplurals=2; plural=(n != 1);\n"' +
                                        '\n');

        var oArray = sString.split('\n');
        var sTempString;
        var iState = 0;
        var msgid;
        var msgidPlural;
        var msgstr;   // array: [ msgidPlural, msgstr, msgstrPlural, msgstrPluralN ]
        var match;
        var pluralIndex;
        function ltrim (sString) {
            return sString.replace(/^\s+/, '');
        }
        function sanitizeQuotes (sString) {
            var sanitizedDouble = sString.replace(/\\\"/g, '"');
            var sanitizedSingle = sanitizedDouble.replace(/\\\\'/g, '\'');
            return sanitizedSingle;
        }
        for (var i = 0, len = oArray.length; i < len; i++) {
            if (oArray[i][0] === '#') {
                iState = 0;
                msgidPlural = null;
                msgstr = [];
                pluralIndex = null;
            } else if (oArray[i].indexOf('msgid_plural') > -1) {
                iState = 2;
                msgidPlural = oArray[i].substr(oArray[i].indexOf('"') + 1);
                msgidPlural = msgidPlural.substr(0, msgidPlural.lastIndexOf('"'));
            } else if (oArray[i].indexOf('msgid') > -1) {
                iState = 1;
                msgid = oArray[i].substr(oArray[i].indexOf('"') + 1);
                msgid = msgid.substr(0, msgid.lastIndexOf('"'));
            } else if (iState >= 2 && oArray[i].indexOf('msgstr[') > -1) {
                iState = 3;
                match = oArray[i].match(/^msgstr\[(\d+)\]/g);
                pluralIndex = parseInt(match[0].slice(7, -1), 10);
                sTempString = oArray[i].substr(oArray[i].indexOf('"') + 1);
                sTempString = sTempString.substr(0, sTempString.lastIndexOf('"'));
                msgstr[0] = msgidPlural;
                msgstr[pluralIndex + 1] = sanitizeQuotes(sTempString);
                params.locale_data[sDomain][sanitizeQuotes(msgid)] = msgstr;
            } else if (oArray[i].indexOf('msgstr') > -1) {
                if (iState >= 0) {
                    iState = 3;
                    pluralIndex = 0;
                    sTempString = oArray[i].substr(oArray[i].indexOf('"') + 1);
                    msgstr[0] = msgidPlural;
                    msgstr[pluralIndex + 1] = sanitizeQuotes(sTempString.substr(0, sTempString.lastIndexOf('"')));
                    params.locale_data[sDomain][sanitizeQuotes(msgid)] = msgstr;
                }
            } else {
                sTempString = ltrim(oArray[i]);
                if (sTempString[0] === '"') {
                    sTempString = sTempString.substr(1);
                    sTempString = sTempString.substr(0, sTempString.lastIndexOf('"'));

                    if (iState === 1) {  // msgid continues for more than one line
                        msgid += sTempString;
                    } else if (iState === 2) {  // msgid_plural continues for more than one line
                        msgidPlural += sTempString;
                    } else if (iState === 3) {  // msgstr continues for more than one line
                        msgstr[pluralIndex + 1] += sanitizeQuotes(sTempString);
                        params.locale_data[sDomain][sanitizeQuotes(msgid)] = msgstr;
                    }
                } else {
                    iState = 0;
                    msgidPlural = null;
                    msgstr = [];
                    pluralIndex = null;
                }
            }
        }
    }

    dss.staticDataModel.gt = new Gettext(params);

    Ext.Date.dayNames = [ _('sunday'), _('monday'), _('tuesday'), _('wednesday'), _('thursday'), _('friday'), _('saturday') ];
    Ext.Date.weekdayToWeekdayDescriptionMapping = {
        MO: _('on monday'),
        TU: _('on tuesday'),
        WE: _('on wednesday'),
        TH: _('on thursday'),
        FR: _('on friday'),
        SA: _('on saturday'),
        SU: _('on sunday')
    };
    Ext.Date.weekdayToWeekdayDescription = function (weekday) {
        return Ext.Date.weekdayToWeekdayDescriptionMapping[weekday];
    };

    Ext.Date.monthNames = [
        _('january'), _('february'), _('march'), _('april'), _('may'), _('june'), _('july'),
        _('august'), _('september'), _('october'), _('november'), _('december')
    ];
    Ext.Date.monthNameMapping = {
        1: _('january'),
        2: _('february'),
        3: _('march'),
        4: _('april'),
        5: _('may'),
        6: _('june'),
        7: _('july'),
        8: _('august'),
        9: _('september'),
        10: _('october'),
        11: _('november'),
        12: _('december')
    };
    Ext.Date.monthNumberToMonthName = function (monthNumber) {
        return Ext.Date.monthNameMapping[monthNumber];
    };
};

dss.cloudAvaible = null;

dss.isCloudAvaible = function (fRefresh) {
    if (fRefresh == null) {
        if (dss.cloudAvaible != null) {
            return dss.cloudAvaible;
        }
    }
    if (fRefresh == false) {
        if (dss.cloudAvaible != null) {
            return dss.cloudAvaible;
        }
    }
    dss.cloudAvaible = false;
    var resultObj;
    try {
        var sResponseRemoteAccess = dss.ajaxSyncRequest('/cgi-bin/remoteaccessinfo.sh', {});
        resultObj = Ext.JSON.decode(sResponseRemoteAccess);
        if (resultObj && resultObj.success) {
            if (resultObj.data.remoteaccess !== null) {
                dss.cloudAvaible = resultObj.data.remoteaccess;
            }
        }
    } catch (e) {
        devLog && devLog(e);
    }
    return dss.cloudAvaible;
};

var DEFAULT_SUBSCRIPTION_MAX_RETRIES = 3;

dss.pushAndQuery = function (eventName, parameter, answerEvent, callbackFunction, failureConfig) {
    if (dss.activeSubscription != null) {
        dss.endSubscription();
    }

    dss.activeSubscription = {
        id: Math.floor(Math.random() * 100000000000),
        name: answerEvent,
        callback: callbackFunction
    };

    dss.ajaxSyncRequest('/json/event/subscribe', {
        subscriptionID: dss.activeSubscription.id,
        name: dss.activeSubscription.name
    });

    dss.raiseEvent(eventName, parameter);
    dss.refreshListener(failureConfig);
};

dss.activeSubscription = null;

dss.makeSubscription = function (eventName, callbackFunction, failureConfig) {

    dss.activeSubscription = {
        id: Math.floor(Math.random() * 100000000000),
        name: eventName,
        callback: callbackFunction
    };

    dss.ajaxSyncRequest('/json/event/subscribe', {
        subscriptionID: dss.activeSubscription.id,
        name: dss.activeSubscription.name
    });
    dss.refreshListener(failureConfig);
};

dss.refreshListener = function (failureConfig) {
    var maxRetries = failureConfig && failureConfig.maxRetries != null
        ? failureConfig.maxRetries
        : DEFAULT_SUBSCRIPTION_MAX_RETRIES;
    var reloadPage = failureConfig && failureConfig.reloadPage != null
        ? failureConfig.reloadPage
        : true;
    var errorMessage = failureConfig && failureConfig.errorMessage != null
        ? failureConfig.errorMessage
        : 'An error occurred when processing the request. The page has to be reloaded.';
    var errorCallback = failureConfig && failureConfig.errorCallback instanceof Function
        ? failureConfig.errorCallback
        : null;
    if (dss.activeSubscription != null) {
        dss.ajaxAssyncRequestWithoutAnswer('/json/event/get', {
            subscriptionID: dss.activeSubscription.id,
            timeout: 5 * 1000
        }, function (answer) {
            var result;
            try {
                result = Ext.JSON.decode(answer);
            } catch (e) {
                devLog && devLog(e);
            }
            if (!result || !result.ok) {
                if (dss.activeSubscription) {
                    dss.activeSubscription.id = Math.floor(Math.random() * 100000000000);
                    dss.ajaxSyncRequest('/json/event/subscribe', {
                        subscriptionID: dss.activeSubscription.id,
                        name: dss.activeSubscription.name
                    });
                    if (maxRetries > 0) {
                        dss.refreshListener({
                            maxRetries: maxRetries - 1,
                            reloadPage: reloadPage,
                            errorMessage: errorMessage,
                            errorCallback: errorCallback
                        });
                    } else {
                        dss.endSubscription();
                        if (errorCallback) {
                            errorCallback(result);
                        }
                        if (errorMessage) {
                            Ext.MessageBox.alert(
                                _('Error'),
                                _(errorMessage),
                                reloadPage
                                    ? function () {
                                        window.location.reload();
                                    }
                                    : null
                            );
                        } else if (reloadPage) {
                            window.location.reload();
                        }
                    }

                }

            } else {
                var fRepeat = true;
                if (result.result && result.result.events) {
                    for (var i = 0; i < result.result.events.length; i++) {
                        if (dss.activeSubscription != null && dss.activeSubscription.callback(result.result.events[i]) === true) {
                            fRepeat = false;
                        }
                    }
                }
                if (fRepeat && maxRetries > 0) {
                    dss.refreshListener({
                        maxRetries: maxRetries - 1,
                        reloadPage: reloadPage,
                        errorMessage: errorMessage,
                        errorCallback: errorCallback
                    });
                } else {
                    dss.endSubscription();
                    if (errorCallback) {
                        errorCallback(result);
                    }
                }
            }
        });
    }
};

dss.endSubscription = function () {
    var id = dss.activeSubscription.id;
    var name = dss.activeSubscription.name;
    dss.activeSubscription = null;
    dss.ajaxAssyncRequestWithoutAnswer('/json/event/unsubscribe', {
        subscriptionID: id,
        name: name
    });
};

dss.kelvinToMired = function (kelvin) {
    return Math.round(1000000 / kelvin);
};

dss.miredToMOCValue = function (mired) {
    return Math.round(((mired - 100) / 900) * 255);
};

dss.MOCValueToMired = function (value) {
    return Math.round(((value / 255) * 900) + 100);
};

dss.MiredToKelvin = function (mired) {
    return Math.round((1000000 / mired) / 100) * 100;
};

dss.missingTranslationsCache = [];

/**
                                * Gettext translation method
                                *
                                * @param {String}
                                *            msgid The string to translate
                                * @param {String}
                                *            arg1 Typically the plural form
                                * @param {int}
                                *            arg1 Typically the count
                                * @return {String} The translated string if available, otherwise return msgid
                                */
function _ (msgid, arg1, arg2) {
    if (dss.staticDataModel.gt) {
        var msgstr = '';
        if (arguments.length === 1) {
            msgstr = dss.staticDataModel.gt.gettext(msgid);
        } else if (arguments.length === 2) {
            msgstr = dss.staticDataModel.gt.ngettext(msgid, arg1);
        } else if (arguments.length === 3) {
            msgstr = dss.staticDataModel.gt.ngettext(msgid, arg1, arg2);
        }
        if (!Gettext._locale_data[dss.staticDataModel.gt.domain].msgs[msgid] && !msgid.toString().match(/^[0-9:]*$/g)) {
            if (dss.missingTranslationsCache.indexOf(msgid) === -1) {
                devLog && devLog('%c! missing translation: %c' + msgid, 'color:#777777', 'color:#cd5200');
                dss.missingTranslationsCache.push(msgid);
            }
        } else {
            return msgstr;
        }
    }
    return msgid;
}

function formatDelayString (delay) {     // eslint-disable-line no-unused-vars
    var hasSeconds = (delay % 60) > 0;
    var hasMinutes = ((Math.floor(delay / 60)) % 60) > 0;
    var hasHours = (delay >= 3600);

    var pad = function (i) {
        return (i < 10 ? '0' + i : i);
    };

    if (!hasHours) {
        if (!hasMinutes) {
            return delay + ' ' + _('sec.');
        } else if (!hasSeconds) {
            return Math.floor(delay / 60) + ' ' + _('min.');
        } else {
            return Math.floor(delay / 60) + ':' + pad(delay % 60) + ' ' + _('min.');
        }
    } else if (!hasMinutes) {
        if (!hasSeconds) {
            return Math.floor(delay / 3600) + ' ' + _('hrs.');
        } else {
            return Math.floor(delay / 3600) + ':00:' + pad(delay % 60) + ' ' + _('hrs.');
        }
    } else if (!hasSeconds) {
        return Math.floor(delay / 3600) + ':' + pad(Math.floor(delay / 60) % 60) + ':00 ' + _('hrs.');
    } else {
        return Math.floor(delay / 3600) + ':' + pad(Math.floor(delay / 60) % 60) + ':' + pad(delay % 60) + ' ' + _('hrs.');
    }
}
